/* gvSIG. Geographic Information System of the Valencian Government
 *
 * Copyright (C) 2007-2008 Infrastructures and Transports Department
 * of the Valencian Government (CIT)
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, 
 * MA  02110-1301, USA.
 * 
 */

/*
 * AUTHORS (In addition to CIT):
 * 2009 {Iver T.I.}   {Task}
 */

package org.gvsig.fmap.dal.store.gpe;

import java.util.ArrayList;
import java.util.NoSuchElementException;

import org.gvsig.fmap.dal.exception.DataException;
import org.gvsig.fmap.dal.exception.ReadRuntimeException;
import org.gvsig.tools.dispose.DisposableIterator; 
import org.gvsig.fmap.dal.feature.FeatureQuery;
import org.gvsig.fmap.dal.feature.FeatureType;
import org.gvsig.fmap.dal.feature.impl.DefaultFeature;
import org.gvsig.fmap.dal.feature.spi.DefaultFeatureProvider;
import org.gvsig.fmap.dal.feature.spi.FeatureProvider;
import org.gvsig.fmap.dal.feature.spi.FeatureSetProvider;
import org.gvsig.tools.evaluator.EvaluatorException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * @author <a href="mailto:jpiera@gvsig.org">Jorge Piera</a>
 */
public class GPESetProvider implements FeatureSetProvider {

	
	static private Logger logger = LoggerFactory.getLogger(GPESetProvider.class);
	
	private GPEStoreProvider gpeStProvider = null;
	private FeatureQuery query = null;
	private FeatureType featType = null;
	
	private ArrayList includedIndices = null;
	private DefaultFeatureProvider auxProvider;
	
	protected Object[] features = null;

	public GPESetProvider(GPEStoreProvider store,
			FeatureQuery query) {
		
		try {
			featType = store.getFeatureStore().getDefaultFeatureType();
		} catch (DataException e) {
			logger.error("While creating GPESetProvider: " + e.getMessage());
		} 
		gpeStProvider = store;
		query = null;
		auxProvider = new DefaultFeatureProvider(featType);
		features = store.getContentHandler().getFeatureSet().values().toArray();
	}

	public GPESetProvider(GPEStoreProvider store,
			FeatureQuery q, FeatureType featureType) {

		featType = featureType;
		gpeStProvider = store;
		query = q;
		auxProvider = new DefaultFeatureProvider(featureType);
		features = store.getContentHandler().getFeatureSet().values().toArray();
	}

	public boolean canFilter() {
		return true;
	}

	public boolean canIterateFromIndex() {
		return true;
	}

	public boolean canOrder() {
		// TODO Auto-generated method stub
		return false;
	}

	/* (non-Javadoc)
	 * @see org.gvsig.fmap.dal.feature.spi.FeatureSetProvider#getSize()
	 */
	public long getSize() throws DataException {
		return getIncludedIndices(auxProvider).size();
	}

	/* (non-Javadoc)
	 * @see org.gvsig.fmap.dal.feature.spi.FeatureSetProvider#isEmpty()
	 */
	public boolean isEmpty() throws DataException {
		return getIncludedIndices(auxProvider).size() == 0;
	}

	public DisposableIterator fastIterator() throws DataException {
		return fastIterator(0);
	}

	public DisposableIterator fastIterator(long i) throws DataException {
		return new GPEFastIterator(i);
	}

	public DisposableIterator iterator() throws DataException {
		return iterator(0);
	}

	public DisposableIterator iterator(long i) throws DataException {
		return new GPEIterator(i);
	}

	public void dispose() {
		gpeStProvider = null;
		query = null;;
		featType = null;
	}

	protected class GPEIterator implements DisposableIterator {
		
		protected long my_index;
		protected FeatureType my_type;

		public GPEIterator(long starton)
				throws DataException {
			my_index = starton;
		}

		/*
		 * (non-Javadoc)
		 * @see java.util.Iterator#remove()
		 */
		public void remove() {
			throw new UnsupportedOperationException();
		}

		public void dispose() {
			// TODO Auto-generated method stub
			
		}

		public boolean hasNext() {
			
			long sz = 0;
			try {
				sz = getSize();
			} catch (DataException e) {
				logger.error("While getting size: " + e.getMessage());
			}
			return my_index < sz;
		}

		public Object next() {
			if (!hasNext()) {
				throw new NoSuchElementException();
			}
			
			try {

				int overallindex = ((Integer) getIncludedIndices(auxProvider).get((int) my_index)).intValue();
				//  
				FeatureProvider ret = ((DefaultFeature) features[overallindex]).getData();
//					new GPEFeatureProvider(
//							(DefaultFeature) features[overallindex],
//							featType,
//							(long) overallindex);
				my_index++;
				return ret;
			} catch (Exception e) {
				throw new ReadRuntimeException(gpeStProvider.getName(), e);
			}
		}
	}
	
	protected class GPEFastIterator extends GPEIterator {

		protected FeatureProvider data;

		public GPEFastIterator(long starton) throws DataException {
			super(starton);
		}
		
		
		public Object next() {
			if (!hasNext()) {
				throw new NoSuchElementException();
			}

			try {
				int overallindex = ((Integer) getIncludedIndices(data).get((int) my_index)).intValue();
				data = ((DefaultFeature) features[overallindex]).getData();
			} catch (Exception e) {
				throw new ReadRuntimeException("next", e);
			}
			my_index++;
			return this.data;
		}
		
		public void dispose() {
			super.dispose();
			data = null;
		}
	}

	
	private ArrayList getIncludedIndices(FeatureProvider fp) {
		if (includedIndices == null) {
			try {
				includedIndices = gpeStProvider.getIntersectingIndices(query, fp, features);
			} catch (EvaluatorException e) {
				logger.error("While getting intersecting indexes: " + e.getMessage());
				includedIndices = new ArrayList();
			}
		}
		return includedIndices;
	}

//	public void loadFeatureProvider(DefaultFeatureProvider ret,
//			DefaultFeature fea, long longindex) {
//		
//		FeatureType ft = ret.getType();
//		FeatureAttributeDescriptor[] atts = ft.getAttributeDescriptors();
//		int len = atts.length;
//		for (int i=0; i<len; i++) {
//			ret.set(i, fea.get(i));
//		}
//		ret.setDefaultGeometry(fea.getDefaultGeometry());
//		ret.setOID(new Long(longindex));
//	}
}

