/* gvSIG. Geographic Information System of the Valencian Government
*
* Copyright (C) 2007-2008 Infrastructures and Transports Department
* of the Valencian Government (CIT)
* 
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
* 
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
* 
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, 
* MA  02110-1301, USA.
* 
*/

/*
* AUTHORS (In addition to CIT):
* 2009 {Iver T.I.}   {Task}
*/
 
package org.gvsig.fmap.dal.store.gpe;

import java.io.File;

import org.cresques.cts.IProjection;
import org.gvsig.fmap.dal.DataStoreParameters;
import org.gvsig.fmap.dal.library.FileHelper;
import org.gvsig.fmap.dal.serverexplorer.filesystem.FilesystemStoreParameters;
import org.gvsig.fmap.dal.spi.AbstractDataParameters;
import org.gvsig.fmap.dal.spi.DataStoreProviderServices;
import org.gvsig.fmap.geom.primitive.Envelope;
import org.gvsig.tools.dynobject.DelegatedDynObject;
import org.gvsig.tools.dynobject.DynStruct;

/**
 * @author <a href="mailto:jpiera@gvsig.org">Jorge Piera</a>
 */
//public class GPEStoreParameters extends AbstractDataParameters implements
//DataStoreParameters, FilesystemStoreParameters {
//	protected static final String FIELD_FILENAME = "filename";
//	protected static final String FIELD_SRS = "srs";
//	
//	public static final String DYNCLASS_NAME = "GPEStoreParameters";
//	protected static DynClass DYNCLASS = null;
//	private DelegatedDynObject delegatedDynObject;
//	
//	public GPEStoreParameters() {
//		super();
//		initialize();
//	}
//
//	protected void initialize() {
//		this.delegatedDynObject = (DelegatedDynObject) ToolsLocator
//				.getDynObjectManager().createDynObject(
//						GPEStoreParameters.DYNCLASS);
//	}
//	
//	protected DelegatedDynObject getDelegatedDynObject() {
//		return delegatedDynObject;
//	}
//
//	/* (non-Javadoc)
//	 * @see org.gvsig.fmap.dal.DataStoreParameters#getDataStoreName()
//	 */
//	public String getDataStoreName() {
//		return GPEStoreProvider.NAME;
//	}
//
//	/* (non-Javadoc)
//	 * @see org.gvsig.fmap.dal.DataStoreParameters#getDescription()
//	 */
//	public String getDescription() {
//		return GPEStoreProvider.DESCRIPTION;
//	}
//
//	/* (non-Javadoc)
//	 * @see org.gvsig.fmap.dal.DataStoreParameters#isValid()
//	 */
//	public boolean isValid() {
//		return (this.getFileName() != null);
//	}
//
//	/* (non-Javadoc)
//	 * @see org.gvsig.fmap.dal.serverexplorer.filesystem.FilesystemStoreParameters#getFile()
//	 */
//	public File getFile() {
//		return new File(this.getFileName());
//	}
//
//	/* (non-Javadoc)
//	 * @see org.gvsig.fmap.dal.serverexplorer.filesystem.FilesystemStoreParameters#setFile(java.io.File)
//	 */
//	public void setFile(File file) {
//		this.setDynValue(FIELD_FILENAME, file.getPath());	
//	}	
//	
//	
//	public String getFileName() {
//		return (String) this.getDynValue(FIELD_FILENAME);
//	}
//	
//	public void setFileName(String aFileName) {
//		this.setDynValue(FIELD_FILENAME, aFileName);
//	}
//	
//	public IProjection getSRS(){
//		return (IProjection) this.getDynValue(FIELD_SRS);
//	}
//	
//	public void setSRS(IProjection srs){
//		this.setDynValue(FIELD_SRS, srs);
//	}
//	
//	public static void registerPersistent() {
//		 DynStruct definition =
//			 ToolsLocator.getPersistenceManager().addDefinition(GPEStoreParameters.class,
//					 DYNCLASS_NAME,
//					 DYNCLASS_NAME,
//	                null,
//	                null);
//	      
//		 	definition.addDynFieldString(FIELD_FILENAME).setMandatory(true);
//		 	definition.addDynFieldObject(FIELD_SRS).setMandatory(true).setClassOfValue(IProjection.class);
//	}	
//
//}
public class GPEStoreParameters extends AbstractDataParameters implements
DataStoreParameters, FilesystemStoreParameters {

    public static final String PARAMETERS_DEFINITION_NAME = "GPEStoreParameters";

    private static final String FILE_PARAMETER_NAME = "File";
    private static final String CRS_PARAMETER_NAME = "CRS";
    private static final String ENVELOPE_PARAMETER_NAME = "Envelope";

    private DelegatedDynObject parameters;

    public GPEStoreParameters() {
        this(PARAMETERS_DEFINITION_NAME);
    }

    protected GPEStoreParameters(String parametersDefinitionName) {
        this(parametersDefinitionName, GPEStoreProvider.NAME);
    }

    public GPEStoreParameters(String parametersDefinitionName, String name) {
        super();
        this.parameters = (DelegatedDynObject) FileHelper.newParameters(parametersDefinitionName);
        this.setDynValue(DataStoreProviderServices.PROVIDER_PARAMTER_NAME, name);       
    }

    protected static void registerParametersDefinition(){ 
        DynStruct dynStruct = FileHelper.registerParametersDefinition(
            GPEStoreParameters.PARAMETERS_DEFINITION_NAME,
            GPEStoreParameters.class, "GPEParameters.xml");
        dynStruct.addDynFieldObject("Envelope").setClassOfValue(Envelope.class).setMandatory(false);
    }

    protected DelegatedDynObject getDelegatedDynObject() {
        return parameters;
    }

    public String getDataStoreName() {
        return GPEStoreProvider.NAME;
    }

    public String getDescription() {
        return GPEStoreProvider.DESCRIPTION;
    }

    public boolean isValid() {
        return (this.getFile() != null);
    }

    public File getFile() {
        return (File) this.getDynValue(FILE_PARAMETER_NAME);
    }

    public void setFile(File file) {
        this.setDynValue(FILE_PARAMETER_NAME, file);
    }	

    public String getFileName() {
        return this.getFile().getAbsolutePath();
    }

    public void setFileName(String fileName) {
        this.setDynValue(FILE_PARAMETER_NAME, fileName);
    }

    public IProjection getCRS() {
        return (IProjection) this.getDynValue(CRS_PARAMETER_NAME);
    }

    public void setCRS(IProjection srs) {
        this.setDynValue(CRS_PARAMETER_NAME, srs);
    }

    public void setCRS(String srs) {
        this.setDynValue(CRS_PARAMETER_NAME, srs);
    }

    public Envelope getEnvelope() {
        return (Envelope) this.getDynValue(ENVELOPE_PARAMETER_NAME);
    }

    public void setEnvelope(Envelope envelope) {
        this.setDynValue(ENVELOPE_PARAMETER_NAME, envelope);
    }
}

