package org.gvsig.mobile.app;

import org.gvsig.mobile.core.auxfunctions.ResourceReader;
import org.gvsig.mobile.core.auxfunctions.ShowMessage;
import org.gvsig.mobile.core.auxfunctions.StringConstructor;
import org.gvsig.mobile.core.constants.GvSigConstants;
import org.gvsig.mobile.core.extensions.interfaces.IFunctionality;
import org.gvsig.mobile.core.extensions.interfaces.ITool;
import org.gvsig.mobile.core.interfaces.IGvSigContext;
import org.gvsig.mobile.core.interfaces.IMouseEventProvider;
import org.gvsig.mobile.ui.awt.ImageUtils;
import org.gvsig.mobile.ui.awt.WindowManager;
import org.gvsig.mobile.ui.awt.mapcontrol.MapControl;
import org.gvsig.mobile.ui.awt.mapcontrol.MapEventsThread;
import org.gvsig.mobile.ui.awt.statusbar.StatusBarManager;
import org.gvsig.mobile.ui.lw.AppSettings;
import org.osgi.framework.Bundle;
import org.osgi.framework.BundleContext;
import org.osgi.service.wireadmin.Envelope;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.zaval.lw.LwPanel;
import org.zaval.lw.LwToolkit;


public class Application implements IGvSigContext {
	
	static private Logger logger = LoggerFactory.getLogger(Application.class);
	
	public static String APP_FOLDER = "";
	public static String APP_NAME = "gvSIG Mobile";
	public static String APP_VERSION = "1.0";
	public static boolean RUNNING_ON_JME = false;
	public BundleContext bc;
	
	private static Application instance;
	MapControl map;
	MainFrame frame; 
	MapEventsThread events;
	ExtensionManagerToolbar EMT;
	
	StatusBarManager statusBarManager;
	LwPanel mainContainer;
	
	public Application(String dir, BundleContext context) {
		this.bc=context;
		
		Bundle corebundle = getCoreBundle(context);
		ImageUtils.CORE_FOLDER = corebundle.getSymbolicName();
		ImageUtils.CORE_VERSION = corebundle.getHeaders().get("Bundle-Version").toString();
		ImageUtils.setAppDir(dir);
		ResourceReader.resourceDir = dir + "resources";
		
		ShowMessage.Show(StringConstructor.Construct(new String[]{"Resources Directory-->" ,ResourceReader.resourceDir}), true);
//		LwToolkit.startVCL("/org/gvsig/mobile/ui/theme");
		
		boolean icon48 = isImg48();
		if (icon48) {
			LwToolkit.startVCL("/org/zaval/lw/theme/big");
			ImageUtils.IMG_SKIN_PREFIX = "img48";
		} else {
			LwToolkit.startVCL("/org/zaval/lw/theme");
			ImageUtils.IMG_SKIN_PREFIX = "img24";
		}

		LwToolkit.startVCL("/org/zaval/lw/theme/p3d");

		frame = new MainFrame(APP_NAME + " " + APP_VERSION, this.bc);
		context.registerService(IGvSigContext.class.getName(), (IGvSigContext)this, null);
		map= frame.getMapControl();
		events = new MapEventsThread(map);
		events.start();
		
		frame.init(events);
		// adding status bar manager......
		statusBarManager = new StatusBarManager(frame);
		frame.setStatusBarManager(statusBarManager);

		WindowManager.getInstance().showMain(frame);
		instance = this;
		
		GvSigMouseEventProvider MEP=new GvSigMouseEventProvider(context, GvSigConstants.GvSIGMobile_MOUSE_EVENT_PROVIDER);
		events.addListener(MEP);
		bc.registerService(IMouseEventProvider.class.getName(), (IMouseEventProvider)MEP, null);
		
		EMT=new ExtensionManagerToolbar(GvSigConstants.EXTENSION_MANAGER_TOOLBAR,new Class[] {Envelope.class, ITool.class, IFunctionality.class}, this.bc);

	}
	
	private boolean isImg48() {
		return AppSettings.size_Factor.indexOf("48") == 0; 
	}

	private Bundle getCoreBundle(BundleContext bc) {
		
		Bundle[] bbs = bc.getBundles();
		int sz = bbs.length;
		for (int i=0; i<sz; i++) {
			if (bbs[i].getSymbolicName().toLowerCase().indexOf("gvsigcore") != -1) {
				return bbs[i]; 
			}
		}
		logger.error("Did not find Core bundle!!");
		return null;
	}




	public static Application getInstance() {
		return instance;
	}


	public MapEventsThread getEvents() {
		return events;
	}

	public void setEvents(MapEventsThread events) {
		this.events = events;
	}

	public Object getMET() {
		
		return getEvents();
	}

	public Object getViewPort() {
		return this.map.getViewPort();
	}
	
	public Object getMapControl() {
		return map;
	}
	
	public Object getMainFrame() {
		return frame;
	}

	public Object getStatusBarManager() {
		return statusBarManager;
	}
}
