package org.gvsig.mobile.core.auxfunctions;

import org.gvsig.mobile.core.constants.GvSigConstants;
import org.gvsig.mobile.core.exceptions.GvSigServiceNotFoundException;
import org.gvsig.mobile.core.extensions.interfaces.IFunctionality;
import org.gvsig.mobile.core.interfaces.IGvSigContext;
import org.osgi.framework.BundleContext;
import org.osgi.framework.ServiceReference;

public class FunctionalityResolutor {
	
	/**
	 * Try to solve a functionality on the basis of parameters
	 * @param functionalityID A possible ID of a functionality
	 * @param functionalityName A possible Name of a functionality
	 * @param functionalityVersion A possible Version of a functionality
	 * @param extensionID A possible ID of a extension that contains the functionality
	 * @param interfaceID A possible ID of a interface that was implemented by the functionality
	 * @param interfaceName A possible Name of a interface that was implemented by the functionality
	 * @param interfaceVersion A possible Version of a interface that was implemented by the functionality
	 * @param bc The OSGi Bundle Context
	 * @return The functionality that best fits the parameters provided 
	 * @throws GvSigServiceNotFoundException If the functionality is not installed
	 */
	public static IFunctionality solve(String functionalityID,
			String functionalityName, String functionalityVersion,
			String extensionID, String interfaceID, String interfaceName,
			String interfaceVersion, BundleContext bc) throws GvSigServiceNotFoundException {

		try {

			// TODO:Aqui se deberia razonar en base a los parametros recibidos para
			// construir las posibles consultas
			String filter = StringConstructor.Construct(new String[] { "(",
					GvSigConstants.FUNCTIONALITY_ID, "=", functionalityID, ")" });

			ServiceReference funcReferences[] = bc.getServiceReferences(
					IFunctionality.class.getName(), filter.toString());
			if (funcReferences != null) {
				ShowMessage.Show(
						"(gvSIG Mobile Funct. Resolutor) Functionality found!",
						true);
				IFunctionality func = (IFunctionality) bc
						.getService(funcReferences[0]);

				return func;
			}
			throw new GvSigServiceNotFoundException("Functionallity");

		} catch (Exception e) {
			e.printStackTrace();
			ShowMessage.Show(
					"(gvSIG Mobile) Error testing functionalities ...", true);
			throw new GvSigServiceNotFoundException("Functionallity");
		}
	}

	/**
	 * Executes the functionality 
	 * @param func The functionality to be executed
	 * @param bc The bundle context
	 * @return True if all went well and False if something went wrong
	 * @throws GvSigServiceNotFoundException If GvSig Mobile Context Provider is not found
	 */
	public static boolean execute(IFunctionality func, BundleContext bc) throws GvSigServiceNotFoundException {
		ServiceReference cont=null;		
		cont=bc.getServiceReference(IGvSigContext.class.getName());
		if(cont==null){
			throw new GvSigServiceNotFoundException("GvSig Mobile Context");
		}		
		return func.execute(cont);
	}
}
