/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2006 Prodevelop and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *   Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *   +34 963862235
 *   gvsig@gva.es
 *   http://www.gvsig.gva.es
 *
 *    or
 *
 *   Prodevelop Integracin de Tecnologas SL
 *   Conde Salvatierra de lava , 34-10
 *   46004 Valencia
 *   Spain
 *
 *   +34 963 510 612
 *   +34 963 510 968
 *   gis@prodevelop.es
 *   http://www.prodevelop.es
 *
 *    or
 *
 *   Instituto de Robtica
 *   Apartado de correos 2085
 *   46071 Valencia
 *   (Spain)
 *   
 *   +34 963 543 577
 *   jjordan@robotica.uv.es
 *   http://robotica.uv.es
 *   
 */
package org.gvsig.mobile.core.concepts;

import java.util.Vector;

import org.gvsig.mobile.core.exceptions.GvSigPropertyNotAttachedException;
import org.gvsig.mobile.core.extensions.interfaces.ITool;
import org.osgi.framework.BundleContext;

/**
 * A Tool that contains several tools with the same position attribute.
 *   
 * @author aromeu
 *
 */
public class ComboTool extends DefaultTool {

	private Vector tools;

	/**
	 * The constructor
	 * @param tools The tools of the combo
	 * @param toolBarKey The key of the toolbar
	 * @param key The key of the tool
	 * @param bc The BundleContext
	 */
	public ComboTool(Vector tools, String toolBarKey, String key,
			BundleContext bc) {
		super();
		super.setToolbarKey(toolBarKey);
		super.setToolType(COMBO_TOOL);
		super.setBundleContext(bc);
		super.setKey(key);
		this.tools = tools;
		try {
			super.setIcon(((ITool)this.tools.elementAt(0)).getIcon());
		} catch (GvSigPropertyNotAttachedException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (Exception e) {
			
		}
	}

	/**
	 * 
	 * @return A Vector of ITools of the combo
	 */
	public Vector getTools() {
		return tools;
	}
	
	/**
	 * Sets the tools. The Vector must contains ITool instances
	 * @param tools A Vector of ITools
	 */
	public void setTools(Vector tools) {
		this.tools = tools;;
	}

	/**
	 * Adds a tool except it is a ComboTool
	 * @param tool The tool to add
	 */
	public void addTool(ITool tool) {
		if (tools == null) {
			tools = new Vector();
		}
		if(!tools.contains(tool) && !(tool instanceof ComboTool))
			tools.add(tool);		
	}

	/**
	 * Removes a tool
	 * @param tool The tool to remove
	 * @return true if it is removed; false if not
	 */
	public boolean removeTool(ITool tool) {
		if(tools != null && tools.contains(tool)) {
			tools.remove(tool);
			return true;
		} else {
			return false;
		}		
	}
	
	/**
	 * 
	 * @param index the index in the tools Vector of the ITool to retrieve
	 * @return A ITool at the specified index
	 */
	public ITool getToolAt(int index) {
		if (tools != null) {
			return (ITool)tools.get(index);
		}
		return null;
	}
}
