package org.gvsig.mobile.core.concepts;

import java.util.ArrayList;
import java.util.Hashtable;
import java.util.Vector;

import org.gvsig.mobile.core.auxfunctions.ShowMessage;
import org.gvsig.mobile.core.constants.GvSigConstants;
import org.gvsig.mobile.core.extensions.interfaces.IExtension;
import org.gvsig.mobile.core.extensions.interfaces.IFunctionality;
import org.gvsig.mobile.core.extensions.interfaces.ITool;
import org.gvsig.mobile.core.interfaces.IGvSigMouseEvent;
import org.osgi.framework.BundleContext;


public class Extension implements IExtension {
	
	private Hashtable props = new Hashtable();
BundleContext bc = null;
	
	public ArrayList myTools;
	public ArrayList myFunctionalities;
	
	public ArrayList services;
	
	public Extension() {
		this.initializeCollections();
	}
	
	public Extension(String ID, String name, String version, String provider, String publicationDate, String description, String comments, BundleContext bc)
	{
		this.setBundleContext(bc);
		this.setID(ID);
		this.setName(name);
		this.setVersion(version);
		this.setProvider(provider);
		this.setPublicationDate(publicationDate);
		this.setDescription(description);
		this.setComments(comments);
		this.initializeCollections();
	}
	
	public Extension(String ID, BundleContext bc) {
		this.setBundleContext(bc);
		this.setID(ID);
		this.initializeCollections();
	}
	
	public void initializeCollections(){
		this.myTools=new ArrayList();
		this.myFunctionalities=new ArrayList();
		services = new ArrayList();
	}
	
	public void setID(String ID) {
		this.props.put(GvSigConstants.EXTENSION_ID, ID);
	}
	
	public void setName(String name) {
		this.props.put(GvSigConstants.EXTENSION_NAME, name);	
	}
	
	public void setVersion(String version) {
		this.props.put(GvSigConstants.EXTENSION_VERSION, version);
	}
	
	public void setProvider(String provider) {
		this.props.put(GvSigConstants.EXTENSION_PROVIDER, provider);
	}
	
	public void setPublicationDate(String publicationDate) {
		this.props.put(GvSigConstants.EXTENSION_PUBLICATIONDATE, publicationDate);
	}
	
	public void setDescription(String description) {
		this.props.put(GvSigConstants.EXTENSION_DESCRIPTION, description);
	}
	
	public void setComments(String comments) {
		this.props.put(GvSigConstants.EXTENSION_COMMENTS, comments);
	}
	
	public void setBundleContext(BundleContext bc) {
		this.bc=bc;
		
	}

	public String getID() {
		return (String)this.props.get(GvSigConstants.EXTENSION_ID);
	}
	
	public String getName() {
		return (String)this.props.get(GvSigConstants.EXTENSION_NAME);
	}
	
	public String getVersion() {
		return (String)this.props.get(GvSigConstants.EXTENSION_VERSION);
	}
	
	public String getProvider() {
		return (String)this.props.get(GvSigConstants.EXTENSION_PROVIDER);
	}

	public String getPublicationDate() {
		return (String)this.props.get(GvSigConstants.EXTENSION_PUBLICATIONDATE);
	}
	
	public String getDescription() {
		return (String)this.props.get(GvSigConstants.EXTENSION_DESCRIPTION);
	}
	
	public String getComments() {
		return (String)this.props.get(GvSigConstants.EXTENSION_COMMENTS);
	}
	
	public BundleContext getBundleContext() {
		return this.bc;
	}

	
	
	
	
	public Hashtable getProps() {
		return props;
	}
	
	
	public boolean addFunctionality(IFunctionality Functionality) {
		try{
		this.myFunctionalities.add(Functionality);
		ShowMessage.Show("The functionality has been properly added", true);
		return true;
		}
		catch(Exception e)
		{
			ShowMessage.Show("An error occurred adding functionality", true);
			return false;
		}
	}

	public boolean addTool(ITool tool) {
		try{
			this.myTools.add(tool);
			ShowMessage.Show("The tool has been properly added", true);
			return true;
			}
			catch(Exception e)
			{
				ShowMessage.Show("An error occurred adding tool", true);
				return false;
			}
	}

	public boolean removeFunctionality(IFunctionality Functionality) {
		try{
			boolean res=this.myFunctionalities.remove(Functionality); 
			if (res){
				ShowMessage.Show("The functionality has been properly removed", true);
				return true;
				}
			ShowMessage.Show("The functionality is not present", true);
				return false;
			}
			catch(Exception e)
			{
				ShowMessage.Show("An error occurred removing functionality", true);
				return false;
			}
	}

	public boolean removeTool(ITool tool) {
		try{
			boolean res=this.myTools.remove(tool);
			if (res){
				ShowMessage.Show("The tool has been properly removed", true);
			return true;
			}
			ShowMessage.Show("The tool is not present", true);
			return false;
			}
			catch(Exception e)
			{
				ShowMessage.Show("An error occurred removing tool", true);
				
				return false;
			}
	}
	
	
	public ArrayList getFunctionalities() {
		return this.myFunctionalities;
	}

	public ArrayList getTools() {
		return this.myTools;
	}

	public boolean registerAll() {
		try{
			bc.registerService(IExtension.class.getName(), (IExtension)this, this.getProps());
			
			IFunctionality f = null;
			
			for (int i = 0; i < myFunctionalities.size(); i++) {
				f = (IFunctionality)myFunctionalities.get(i);
				bc.registerService(IFunctionality.class.getName(), f, f.getProps());
				Vector myClasses=new Vector();
				boolean registerAsConsumerNeeded=false;
				if(f.getListenToMouseEvents()){
					registerAsConsumerNeeded=true;
					myClasses.add(IGvSigMouseEvent.class);					
				}
				//TODO: INCLOURE LA RESTA DE EVENTS
//				registerAsConsumerNeeded=true;
//				myClasses.add(IToolEnvelope.class);
				
				if(registerAsConsumerNeeded) {
					f.registerAsEventConsumer(myClasses.toArray());
				}
			}
			
			for (int i = 0; i < myTools.size(); i++) {
				services.add(bc.registerService(ITool.class.getName(), (ITool)myTools.get(i), ((ITool)myTools.get(i)).getProps()));				
			}
			
			ShowMessage.Show("All components have been properly registered", true);
		return true;	
		}
		catch(Exception e){
			ShowMessage.Show("An error occurred during registration of components", true);
		return false;
		}
	}

}
