package org.gvsig.mobile.core.concepts;

import java.util.Hashtable;

import org.gvsig.mobile.core.auxfunctions.OSGiMethods;
import org.gvsig.mobile.core.auxfunctions.ShowMessage;
import org.gvsig.mobile.core.auxfunctions.StringConstructor;
import org.gvsig.mobile.core.constants.GvSigConstants;
import org.gvsig.mobile.core.exceptions.GvSigServiceNotFoundException;
import org.gvsig.mobile.core.extensions.interfaces.IExtensionManager;
import org.gvsig.mobile.core.extensions.interfaces.IFunctionality;
import org.gvsig.mobile.core.extensions.interfaces.IFunctionalityEnvelope;
import org.gvsig.mobile.core.extensions.interfaces.ITool;
import org.gvsig.mobile.core.extensions.interfaces.IToolEnvelope;
import org.osgi.framework.BundleContext;
import org.osgi.framework.InvalidSyntaxException;
import org.osgi.framework.ServiceEvent;
import org.osgi.framework.ServiceListener;
import org.osgi.framework.ServiceReference;
import org.osgi.framework.ServiceRegistration;
import org.osgi.service.wireadmin.Envelope;
import org.osgi.service.wireadmin.Producer;
import org.osgi.service.wireadmin.Wire;
import org.osgi.service.wireadmin.WireAdmin;
import org.osgi.service.wireadmin.WireConstants;


public abstract class ExtensionManager implements ServiceListener, Producer,IExtensionManager  {

	public BundleContext bc = null;
	public String myFilter;
	public ServiceRegistration reg;
	public Wire wire;
	public String gvSigEMPID;
	public String myPID;
	public Class[] myClasses;
	public Object objectChanged;
	public Envelope event;

	public ExtensionManager(String myPID, Class[]Classes, BundleContext bCon) {
		bc=bCon;
		this.myPID=myPID;
		this.myClasses=Classes;
		this.myFilter=getFilter();
		
		try {
			this.gvSigEMPID=getGvSigEMPID();
		} catch (GvSigServiceNotFoundException e) {
			e.toString();
		}
		
		openConnection();
	}

	public void serviceChanged(ServiceEvent event) {
		WireAdmin wa = null;
		try {
			wa = OSGiMethods.getWireAdmin(bc);
		} catch (GvSigServiceNotFoundException e1) {
			ShowMessage.Show(e1.toString());
		}
		if(wa!=null){
		this.objectChanged=bc.getService(event.getServiceReference());
		
		//metemos el servicio registrado y el evento actual en un Envelope
		//para que el Consumer en funcin del evento decida qu accin realizar
		if (this.objectChanged instanceof IFunctionality) {
			this.event = new FunctionalityEnvelope("", IFunctionalityEnvelope.class.getName(), (IFunctionality)objectChanged, event.getType());
		} else if (this.objectChanged instanceof ITool){
			this.event = new ToolEnvelope("", IToolEnvelope.class.getName(), (ITool)objectChanged, event.getType());
		} else {
			return;
		}
		
		
		switch (event.getType()) { 
		case ServiceEvent.REGISTERED:
			ShowMessage.Show("REGISTERING",true);
			try {				
			consumersConnected(wa.getWires(myFilter));
			}
			catch(Exception e){
				e.printStackTrace();
			}
			break; 
		
	    //Thrown when a property is changed
		//see ServiceRegistration
		case ServiceEvent.MODIFIED:
			ShowMessage.Show("MODIFIED",true);
			break; 
		
		case ServiceEvent.UNREGISTERING: 
			ShowMessage.Show("UNREGISTERING",true);
			
			try{
			consumersConnected(wa.getWires(myFilter));
			}
			catch(Exception e){
				e.printStackTrace();
			}
			break; 
	}
		}
	}
	
	public String getGvSigEMPID() throws GvSigServiceNotFoundException{
		/*
		 * TODO:Ac haurem d'afegir la resta de GvSIGMobile_XXXX_MANAGER
		 */
		if(myClasses.length>0){
			if(myClasses[0].equals(Envelope.class) || myClasses[0].equals(ITool.class)){
				return GvSigConstants.GvSIGMobile_TOOLBAR_MANAGER;
			}
		}
		throw new GvSigServiceNotFoundException("Manager for this classes");
	}
	
	public String constructFilter(){
		
		if(myClasses.length>0){
			if(myClasses.length==1){
			return	StringConstructor.Construct(new String[]{"(objectclass=",myClasses[0].getName(),")"});
			}
			else{
				String res="(|";
				for (int i = 0; i < myClasses.length; i++) {
					res=StringConstructor.Construct(new String[]{res,"(objectclass=",myClasses[i].getName(),")"});
				}
				return StringConstructor.Construct(new String[]{res,")"});
			}
		}
		return "";
	}
	
	public String getFilter(){
		
		String fil= StringConstructor.Construct(new String[]{"(",WireConstants.WIREADMIN_PRODUCER_PID,"=",this.myPID,")"});
		return fil;
	}
	
	
	public void consumersConnected(Wire[] wires) {
		if (wires != null) {
		      for (int i = 0; i < wires.length; i++) {
		        wires[i].update(polled(wires[i]));
		      }
		 }    
	}
	
	public void closeConection() {
		WireAdmin wa = null;
		try {
			wa = OSGiMethods.getWireAdmin(bc);
		} catch (GvSigServiceNotFoundException e) {
			ShowMessage.Show(e.toString());
		}
		ShowMessage.Show(StringConstructor.Construct(new String[]{this.getMessage()," stopping..."}), true); 
		if (wire!=null){
			if(wa!=null){
				wa.deleteWire(wire);
			}
		}
		reg.unregister();
		bc = null;
	}
	
	public Object polled(Wire wire){
		return event;
//		return objectChanged;
	}
	
	private String nombreClase(Class c){
		String[] aux=org.gvsig.compat.CompatLocator.getStringUtils().split(c.getName(), "I");
		if(aux.length==0){return c.getName();}
		return aux[aux.length-1];
	}
	private String getMessage(){
		String res="";
		nombreClase(myClasses[0]);
		if(myClasses.length>0){
			if(myClasses.length==1){
			String aux=
			res=StringConstructor.Construct(new String[]{"(",nombreClase(myClasses[0])," Extension Manager)"});
			}
			else{
				res=StringConstructor.Construct(new String[]{"(",nombreClase(myClasses[0])});
				for (int i = 1; i < myClasses.length; i++) {

					res=StringConstructor.Construct(new String[]{res," & ",nombreClase(myClasses[i]),""});
				}
				res=StringConstructor.Construct(new String[]{res," Extension Manager)"});
			}
		}
		return res;
	}
	
	public void openConnection(){
		WireAdmin wa = null;
		try {
			wa = OSGiMethods.getWireAdmin(bc);
		} catch (GvSigServiceNotFoundException e) {
			ShowMessage.Show(e.toString());
		}
		ShowMessage.Show(StringConstructor.Construct(new String[]{this.getMessage()," starting..."}), true); 
		Hashtable props = new Hashtable();
		Class[] flavors=myClasses;
		props.put(WireConstants.WIREADMIN_PRODUCER_FLAVORS, flavors);
		props.put(GvSigConstants.SERVICE_PID, this.myPID);
//		props.put(WireConstants.WIREADMIN_PRODUCER_SCOPE, Envelope.class.getName());
		reg=bc.registerService(Producer.class.getName(), this,props);		
		if(wa!=null){
			try{
				Hashtable props2= null;
				ShowMessage.Show("Creating WIRE",true);
				wire =wa.createWire(this.myPID, this.gvSigEMPID, props2);
				ShowMessage.Show("Wire Created",true);
				
			}
			catch(Exception e){
				ShowMessage.Show(StringConstructor.Construct(new String[]{this.getMessage(),"Error during the wire creation. Aborting Start."}), true); 
				e.printStackTrace();
				return;
			}
		}
		else{
			ShowMessage.Show(StringConstructor.Construct(new String[]{this.getMessage(),"WA==null Error during the wire creation. Aborting Start."}), true);
			return;
		}
				
		String filter=this.constructFilter();
				
		try {
			bc.addServiceListener(this, filter);
		} catch (InvalidSyntaxException e1) {
			e1.printStackTrace();
		}
		ServiceReference references[]=null;
		try {
			references = bc.getServiceReferences(null, filter);
		} catch (InvalidSyntaxException e) {
			e.printStackTrace();
		} 
		
		

		for (int i = 0; references != null && i < references.length; i++) 
		{ 
		    this.serviceChanged(new ServiceEvent(ServiceEvent.REGISTERED,references[i])); 
		}
		
		
	}
}
