package org.gvsig.mobile.core.extensions.interfaces;

import java.util.HashSet;
import java.util.Hashtable;

import org.gvsig.mobile.core.interfaces.IGvSigMouseEvent;
import org.osgi.framework.BundleContext;
import org.osgi.framework.ServiceReference;
import org.osgi.framework.ServiceRegistration;

/**
 * This Interface provides the methods that must implement every Functionality plugin of GvSIG Mobile</br>
 * The method execution(), onNewMouseEventListened(IGvSigMouseEvent event) & configure() must be overrided with the implementation of the functionality
 * @author ProS
 */
public interface IFunctionality extends ITask {
	/**
	 * 	Executes the functionality.
	 * @param gvSIGMobileRef A reference to gvSigMobile to be used where necessary to obtain data from the current context of the application
	 */
	public boolean execute(ServiceReference gvSIGMobileRef);

	
	/**
	 * This method contains the execution procedure of a functionality and must be overrided.
	 * @param gvSIGMobileRef A reference to gvSigMobile to be used where necessary to obtain data from the current context of the application
	 * @return True if everything is allright, false if something's wrong
	 */
	public boolean execution(ServiceReference gvSIGMobileRef);
	
	
	/**
	 * 
	 * @return The ID of the Functionality
	 */
	public String getID();
	
	/**
	 * 
	 * @return The Name of the Functionality (if it has been attached)
	 */	
	public String getName();
	
	/**
	 * 
	 * @return The Version of the Functionality (if it has been attached)
	 */	
	public String getVersion();
	
	/**
	 * 
	 * @return The Provider of the Functionality (if it has been attached)
	 */	
	public String getProvider();
	
	/**
	 * 	
	 * @return The ID of the extension associated with the functionality
	 */
	public String getExtensionID();
	
	
	//TODO: JOAN--> MIRAR DE COMPLETAR EL JAVADOC DE AQUESTOS MTODES
	public String getInterfaceID();
	public String getInterfaceName();
	public String getInterfaceVersion();
	
	/**
	 * 
	 * @return The Description of the Functionality (if it has been attached)
	 */
	public String getDescription();
	
	/**
	 * 
	 * @return The Comments of the Functionality (if it has been attached)
	 */
	public String getComments();
	
	/**
	 * 
	 * @return The Bundle Context of the Functionality (This parameter is Mandatory)
	 */
	public BundleContext getBundleContext();

	
	
	
	/**
	 * Set the ID of the Functionality</br>This parameter is Mandatory
	 * @param ID The ID to be attached
	 */
	public void setID(String ID);
	
	/**
	 * Set the NAME of the Functionality
	 * @param name The Name to be attached
	 */
	public void setName(String name);
	
	/**
	 * Set the VERSION of the Functionality
	 * @param version The version number to be attached
	 */
	public void setVersion(String version);
	
	/**
	 * Set the PROVIDER of the Functionality
	 * @param provider The provider to be attached 
	 */
	public void setProvider(String provider);
	
	/**
	 * Set the ID of the extension associated with the functionality
	 * @param extensionID The Extension Identifier to be attached
	 */
	public void setExtensionID(String extensionID);
	
	
	//TODO: JOAN-> PLENAR EL JAVADOC
	
	/**
	 * 
	 * @param interfaceID
	 */
	public void setInterfaceID(String interfaceID);
	
	/**
	 * 
	 * @param interfaceName
	 */
	public void setInterfaceName(String interfaceName);
	
	/**
	 * 
	 * @param interfaceVersion
	 */
	public void setInterfaceVersion(String interfaceVersion);
	
	/**
	 * Set the DESCRIPTION of the Functionality
	 * @param description Description of the functionality
	 */
	public void setDescription(String description);
	
	/**
	 * Set the COMMENTS of the Functionality
	 * @param comments The comments to be attached
	 */
	public void setComments(String comments);	
	
	/**
	 * Set the System Bundle Context to the Functionality
	 * @param bc the Bundle Context to be attached
	 */
	public void setBundleContext(BundleContext bc);
	
	/**
	 * Returns The Functionality OSGi Properties HashTable &ltKEY:String,VALUE:Object&gt
	 * @return The OSGi Properties of the functionality 
	 */
	public Hashtable getProps();
	
	
	/**
	 * 
	 * @return True if this functionality attends to mouse events and  false if not.
	 */
	public boolean getListenToMouseEvents();
	
	
	/**
	 * This method must be overrided and will be executed each time a new event is detected if event type is one of the event types the functionality has been subscribed 
	 * @param event The Mouse event listened
	 */
	public void onNewMouseEventListened(IGvSigMouseEvent event);
	
	/**
	 * 
	 * @return The OSGi service registration of the fnctionality
	 */
	public ServiceRegistration getServiceRegistration();
	
	/**
	 * Sets the OSGi Service Registration of the functionality
	 * @param sr The service registration of the functionality
	 */
	public void setServiceRegistration(ServiceRegistration sr);
	
	
	/**
	 * Register the functionality as consumer of events which classes has been attached as parameter
	 * @param classes The Classes which functionality must subscribe
	 */
	public void registerAsEventConsumer(Object[] classes);
	
	/**
	 * This method is for configure the parameters of the functionality
	 */
	public void configure();
	/*
	/**
	 * Sets the events which the functionality will attend to
	 * @param events HashSet of <strong>GvSigMouseEventType</strong>'s
	 */
	//public void setMouseEventsSubscriptions(HashSet<Integer> events);
	
	/**
	 * Subscribes the functionality to a GvSig Mouse Event
	 * @param gvSigMouseEventType The GvSig Mouse Event type to suscribe
	 */
	public void addMouseEventSubscription(int gvSigMouseEventType);
	
	/**
	 * 
	 * @return the <strong>GvSigMouseEventType</strong>'s has been subscribed
	 */
	public HashSet getMouseEventsSubscriptions();
	
	/**
	 * 
	 * @return true if the functionality is correct to apply
	 */
	public boolean isApplicable();
}
