package org.gvsig.mobile.core.extensions.interfaces;

import java.util.Hashtable;

import org.gvsig.mobile.core.exceptions.GvSigPropertyNotAttachedException;
import org.osgi.framework.BundleContext;



/**
 * 
 */

/**
 * It extends a task adding an icon and some text information.
 * 
 * It's very important when developing a tool to send inited
 * and finalized messages as DefaultTask does. The easiest way 
 * to write a tool is extending DefaultTool and executing 
 * super.run() and super.stop() inside run() and stop() methods.
 * 
 * @author JCARRAS
 *
 */
public interface ITool {
	public static int EXCLUSIVE_TOOL = 0;
	public static int STATE_TOOL = 1;
	public static int RUN_INMEDIATELY_TOOL = 2;
	public static int COMBO_TOOL = 3;
	public static int HELP_TOOL = 4;
	
	/**
	 * Every tool has a position inside the toolbar starting from 0.
	 * According to the toolbar size and the icon size of the tool
	 * a toolbar can contain a limited number of tools.
	 * 
	 * Two or more tools with the same position and the same toolbar key
	 * will be added automatically to a <code>org.gvsig.mobile.ui.awt.tools.ToolCombo</code>
	 *
	 * @param position The position of the current ITool
	 */
	public void setPosition(int position);
	
	/**
	 * Every tool has a position inside the toolbar starting from 0.
	 * According to the toolbar size and the icon size of the tool
	 * a toolbar can contain a limited number of tools.
	 * 
	 * Two or more tools with the same position and the same toolbar key
	 * will be added automatically to a <code>org.gvsig.mobile.ui.awt.tools.ToolCombo</code>
	 *
	 * @return The position of the current ITool
	 */
	public int getPosition();
	
	/**
	 * 
	 * @return The Key of the tool 
	 * @throws GvSigPropertyNotAttachedException If Key Property has not been attached
	 */
	public String getKey() throws GvSigPropertyNotAttachedException;

	/**
	 * Set the Key of the Tool
	 * @param Key ID of the tool
	 */
	public void setKey(String Key);

	/**
	 * 
	 * @return The path of the image of the tool
	 * @throws GvSigPropertyNotAttachedException if you try to recover this property and this has not been attached
	 * 
	 */
	public String getIcon() throws GvSigPropertyNotAttachedException; 
	
	/**
	 * Set the Path to the Image of the Tool in .png format
	 * @param icon Path to the image of the tool
	 */
	public void setIcon(String icon);

	/**
	 * 
	 * @return The TEXT of the tool 
	 * @throws GvSigPropertyNotAttachedException if you try to recover this property and this has not been attached
	 */
	public String getText() throws GvSigPropertyNotAttachedException;

	/**
	 * Set the Text of the Tool
	 * @param text Text of the tool
	 */
	public void setText(String text);

	/**
	 * 
	 * @return The Short description of the tool purpose
	 * @throws GvSigPropertyNotAttachedException if you try to recover this property and this has not been attached
	 */
	public String getShortDescription() throws GvSigPropertyNotAttachedException;
	
	/**
	 * Set the Description of the Tool purpose
	 * @param description Description of the tool
	 */
	public void setShortDescription(String description);

	/**
	 * 
	 * @return The Key of the toolbar where this tool should be hold in
	 * @throws GvSigPropertyNotAttachedException if you try to recover this property and this has not been attached
	 */
	public String getToolbarKey() throws GvSigPropertyNotAttachedException;
	
	/**
	 * Set the ID of the toolbar where will be included the tool
	 * </br>If the toolbar does not exist, the tool will be invisible
	 * @param toolbarKey Identifier of the toolbar
	 */
	public void setToolbarKey(String toolbarKey);

	/**
	 * 
	 * It could be
	 * 	EXCLUSIVE_TOOL(0) Only one exclusive tool will be active at the same time 
	 *	STATE_TOOL(1) This tool should be deactivated manually.
	 *	RUN_INMEDIATELY_TOOL(2) This tool executes and then stops automatically
	 * 
	 * @return The type of the tool
	 * @throws GvSigPropertyNotAttachedException if you try to recover this property and this has not been attached
	 */
	public int getToolType() throws GvSigPropertyNotAttachedException;
	
		
	/**
	 * @param type Type of the tool</br></br>
	 *  It could be</br>
	 * 	EXCLUSIVE_TOOL(0): Only one exclusive tool will be active at the same time</br> 
	 *	STATE_TOOL(1) This tool should be deactivated manually.</br>
	 *	RUN_INMEDIATELY_TOOL(2) This tool executes and then stops automatically</br>
	 */
	public void setToolType(int type);

	/**
	 * 
	 * @return The NAME of the tool 
	 * @throws GvSigPropertyNotAttachedException if you try to recover this property and this has not been attached
	 */
	public String getName() throws GvSigPropertyNotAttachedException;
	
	/**
	 * Set the NAME of the Tool
	 * @param name Name of the tool
	 */
	public void setName(String name);

	/**
	 * 
	 * @return The Alternative Text of the tool 
	 * @throws GvSigPropertyNotAttachedException if you try to recover this property and this has not been attached
	 */
	public String getAltText() throws GvSigPropertyNotAttachedException;
		
	//TODO: JOAN--> QUINA ES LA DIFERNCIA ENTRE TEXT I ALTTEXT?
	public void setAltText(String value);

	/**
	 * 
	 * @return The COMMENTS of the tool 
	 * @throws GvSigPropertyNotAttachedException if you try to recover this property and this has not been attached
	 */
	public String getComments() throws GvSigPropertyNotAttachedException;
	
	/**
	 * Set the Comments of the Tool
	 * @param comments Comments of the tool
	 */
	public void setComments(String comments);

	/**
	 * 	
	 * @return The ID of the extension associated with the tool
	 * @throws GvSigPropertyNotAttachedException if you try to recover this property and this has not been attached
	 */
	public String getExtensionID() throws GvSigPropertyNotAttachedException;
	
	/**
	 * Set the ID of the extension associated with the extension
	 * @param extensionID The Extension Identifier to be attached
	 */
	public void setExtensionID(String extensionID);

	/**
	 * 	
	 * @return The ID of the functionality associated with the tool
	 * @throws GvSigPropertyNotAttachedException if you try to recover this property and this has not been attached
	 */
	public String getFunctionalityID() throws GvSigPropertyNotAttachedException;
	
	/**
	 * Set the ID of the Functionality associated with the tool
	 * </br>This parameter could be used by the functionality Resolutor to find the suitable functionality
	 * @param functionalityID The Functionality ID to be attached
	 */
	public void setFunctionalityID(String functionalityID);

	/**
	 * 	
	 * @return The Name of the functionality associated with the tool
	 * @throws GvSigPropertyNotAttachedException if the propperty has not been attached
	 */
	public String getFunctionalityName() throws GvSigPropertyNotAttachedException;
	
	/**
	 * Set the Name of the Functionality associated with the tool
	 * </br>This parameter could be used by the functionality Resolutor to find the suitable functionality
	 * @param fuctionalityName The Functionality Name to be attached
	 */
	public void setFunctionalityName(String fuctionalityName);

	/**
	 * 	
	 * @return The version of the functionality associated with the tool
	 * @throws GvSigPropertyNotAttachedException if you try to recover this property and this has not been attached
	 */
	public String getFunctionalityVersion() throws GvSigPropertyNotAttachedException;
	
	/**
	 * Set the Version of the Functionality associated with the tool
	 * </br>This parameter could be used by the functionality Resolutor to find the suitable functionality
	 * @param functionalityVersion The Functionality Version to be attached
	 */
	public void setFunctionalityVersion(String functionalityVersion);

	/**
	 * 	
	 * @return The provider of the functionality associated with the tool
	 * @throws GvSigPropertyNotAttachedException if you try to recover this property and this has not been attached
	 */
	public String getFunctionalityProvider() throws GvSigPropertyNotAttachedException;
	
	/**
	 * Set the Provider of the Functionality associated with the tool
	 * </br>This parameter could be used by the functionality Resolutor to find the suitable functionality
	 * @param functionalityProvider The Functionality Provider to be attached
	 */
	public void setFunctionalityProvider(String functionalityProvider);

	/**
	 * 	
	 * @return The ID of the extension were was included the functionality associated with the tool
	 * @throws GvSigPropertyNotAttachedException if you try to recover this property and this has not been attached
	 */
	public String getFunctionalityExtensionID() throws GvSigPropertyNotAttachedException;
	
	/**
	 * Set the Extension ID of the Functionality associated with the tool
	 * </br>This parameter could be used by the functionality Resolutor to find the suitable functionality
	 * @param functionalityExtensionID The Functionality Extension ID to be attached
	 */
	public void setFunctionalityExtensionID(String functionalityExtensionID);

	/**
	 * 	
	 * @return The ID of the interface of the functionality associated with the tool
	 * @throws GvSigPropertyNotAttachedException if you try to recover this property and this has not been attached
	 */
	public String getFunctionalityInterfaceID() throws GvSigPropertyNotAttachedException;
	
	/**
	 * Set the Interface ID of the Functionality associated with the tool
	 * </br>This parameter could be used by the functionality Resolutor to find the suitable functionality
	 * @param functionalityInterfaceID The Functionality Interface ID to be attached
	 */
	public void setFunctionalityInterfaceID(String functionalityInterfaceID);

	/**
	 * 	
	 * @return The Name of the interface of the functionality associated with the tool
	 * @throws GvSigPropertyNotAttachedException if you try to recover this property and this has not been attached
	 */
	public String getFunctionalityInterfaceName() throws GvSigPropertyNotAttachedException;
	
	/**
	 * Set the Interface Name of the Functionality associated with the tool
	 * </br>This parameter could be used by the functionality Resolutor to find the suitable functionality
	 * @param functionalityInterfaceName The Functionality Interface Name to be attached
	 */
	public void setFunctionalityInterfaceName(String functionalityInterfaceName);

	/**
	 * 	
	 * @return The Version of the interface of the functionality associated with the tool
	 * @throws GvSigPropertyNotAttachedException if you try to recover this property and this has not been attached
	 */
	public String getFunctionalityInterfaceVersion() throws GvSigPropertyNotAttachedException;
	
	/**
	 * Set the Interface Version of the Functionality associated with the tool
	 * </br>This parameter could be used by the functionality Resolutor to find the suitable functionality
	 * @param functionalityInterfaceVersion The Functionality Interface Version to be attached
	 */
	public void setFunctionalityInterfaceVersion(String functionalityInterfaceVersion);
	
	
	/**
	 * Returns The Tool OSGi Properties Hashtable &ltKEY:String,VALUE:Object&gt
	 * @return The OSGi Properties of the tool 
	 */
	public Hashtable getProps();
	
	/**
	 * Sets the OSGi bundle context
	 * @param bc the OSGi bundle context
	 */
	public void setBundleContext(BundleContext bc);
	
	/**
	 * 	
	 * @return The OSGi Bundle Context
	 */
	public BundleContext getBundleContext();
	
	/**
	 * sets the functionality solved by the functionality resolutor based on the parameters relating to the functionality that has been attached
	 * @param funcSolved The functionality solved
	 */
	public void setFuncSolved(IFunctionality funcSolved);
	
	/**
	 * 
	 * @return The functionality solved by the functionality resolutor based on the parameters relating to the functionality that has been attached
	 */
	public IFunctionality getFuncSolved();


}
