package org.gvsig.mobile.extension.base;

import java.net.MalformedURLException;
import java.util.Iterator;
import java.util.List;

import org.gvsig.fmap.mapcontext.layers.FLayer;
import org.gvsig.mobile.app.Application;
import org.gvsig.mobile.core.concepts.BaseActivator;
import org.gvsig.mobile.core.libraries.LibrariesManager;
import org.gvsig.mobile.extension.base.forms.view.CustomForm;
import org.gvsig.mobile.extension.base.forms.view.GenericForm;
import org.gvsig.mobile.extension.base.layer.raster.FLayerGenericRaster;
import org.gvsig.mobile.ui.awt.ImageUtils;
import org.gvsig.mobile.ui.awt.mapcontrol.MapControl;
import org.gvsig.mobile.ui.awt.mapcontrol.impl.Snapping;
import org.gvsig.mobile.ui.lw.GvsigMobileGUIConstants;
import org.gvsig.mobile.ui.lw.IconResourceReader;
import org.gvsig.tools.ToolsLocator;
import org.gvsig.tools.dynobject.DynStruct;
import org.gvsig.tools.dynobject.impl.DefaultDynClass;
import org.osgi.framework.BundleContext;

public class Activator extends BaseActivator {

	Application app;
	private DefaultDynClass ddclass;

	/*
	 * (non-Javadoc)
	 * @see org.osgi.framework.BundleActivator#start(org.osgi.framework.BundleContext)
	 */
	public void start(BundleContext context) throws Exception {
		super.start(context, context.getBundle());
		
		// init form template path:
		GenericForm.TEMPLATE_FILE_NAME =
			ImageUtils.getBundleFile(context.getBundle(), "template.xml").getAbsolutePath();

		
		// do all libraries
		LibrariesManager.getInstance().initialize();
		 if(ToolsLocator.getPersistenceManager().getDefinition(FLayerGenericRaster.DYNCLASS_NAME)==null ) {
			 FLayerGenericRaster.registerPersistent();
		 }
		
		
		// add property to FLayer (infoable in TOC)
        ddclass = (DefaultDynClass) ToolsLocator.getDynObjectManager().get("Metadata:Layer"); 
		 
		 if (ddclass!=null){
			ddclass.addDynFieldBoolean(MapControl.INFO_ACTIVATED_DYN_PROPERTY_NAME).setMandatory(true);
			ddclass.addDynFieldBoolean(CustomForm.PROPERTY_KEY_USES_CUSTOM_FORM).setMandatory(true);
			ddclass.addDynFieldString(CustomForm.PROPERTY_KEY_CUSTOM_FORM_PATH).setMandatory(true);
		}
		
		// start app
//		Application app=new Application("C:\\Proyectos\\gvSIG Mobile 1.0"+File.separator, context);
		String path = context.getBundle().getLocation().substring(context.getBundle().getLocation().lastIndexOf(":")+1);
		path = path.substring(0, path.lastIndexOf("/"));
		
		if (path.endsWith(".jar")) {			
			path = path.substring(0, path.lastIndexOf("/"));			
		}		
		
		path = ImageUtils.parseBundleLocation(path);
		// System.out.println("PATH:" + path);		
		app=new Application(path, context);

		// Application app=new Application(File.separator + "jars"+File.separator, context);
		//((MapControl)app.getMapControl()).initAcetateLayer();
		Snapping.register();
		GvsigMobileGUIConstants.configureLocales();		
		
		
		// configure locale of base bundle explicitly:
		try {			
			String name = context.getBundle().getSymbolicName();
			org.gvsig.i18n.Messages.addResourceFamily("text", IconResourceReader.getLanguageFolder(name, "languages"));		
		} catch (MalformedURLException e) {
			System.err.println("Language files not found in resources directory.");
		}
		
	}

	/*
	 * (non-Javadoc)
	 * @see org.osgi.framework.BundleActivator#stop(org.osgi.framework.BundleContext)
	 */
	public void stop(BundleContext context) throws Exception {
	}
}
