/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2006 Prodevelop and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *   Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *   +34 963862235
 *   gvsig@gva.es
 *   http://www.gvsig.gva.es
 *
 *    or
 *
 *   Prodevelop Integracin de Tecnologas SL
 *   Conde Salvatierra de lava , 34-10
 *   46004 Valencia
 *   Spain
 *
 *   +34 963 510 612
 *   +34 963 510 968
 *   gis@prodevelop.es
 *   http://www.prodevelop.es
 *
 *    or
 *
 *   Instituto de Robtica
 *   Apartado de correos 2085
 *   46071 Valencia
 *   (Spain)
 *   
 *   +34 963 543 577
 *   jjordan@robotica.uv.es
 *   http://robotica.uv.es
 *   
 */

package org.gvsig.mobile.extension.base.forms.view;

import org.gvsig.mobile.extension.base.forms.model.FeatureTemp;
import org.gvsig.mobile.extension.base.util.TaskPerformer;


/**
 * It defines the methods a form implementation must implement
 * to be correctly called by {@link AlphanumericalData}
 * @author jcarras
 *
 */
public interface FeatureForm {
	  
	/**
	 * Binds the feature with the form
	 * @param feature
	 */
	  public void setFeature(FeatureTemp feature);

	  /**
	   * Returns the feature with the modifications done by the user
	   * @return the new feature
	   */
	  public FeatureTemp getFeature();

	  /**
	   * Puts the fields enabled to edit
	   */
	  public void startEdition();

	  /**
	   * Validates the form and advises the user if the fields are not valid
	   * @return true if the values are correct, false in other case
	   */
	  public boolean isValid();

	  /**
	   * Resets the field values and puts the fields not enabled
	   */
	  public void cancelEdition();

	  /**
	   * It destroys the form
	   */
	  public void close();

	  /**
	   * Checks if user modified any value
	   * @return true if any value has been modified, false in other case
	   */
	  public boolean isModified();

	  /**
	   * Checks if the user clicked on cancel button
	   * @return true if user clicked in cancel, false in other case
	   */
	  public boolean isCanceled();

	  /**
	   * Sets the form visible or not visible
	   * @param b 
	   */
	  public void setVisible(boolean b);
	  
	  /**
	   * Checks if the form is visible to the user
	   * @return true if the form is visible false in other case
	   */
	  public boolean isOpen();

	  
	  /**
	   * Object to be notified whena task has to be done
	   */
	  public void setTaskPerformer(TaskPerformer performer);
	  
	  /**
	   * Update listeners using parent components
	   */
	  public void updateListeners();
}
