package org.gvsig.mobile.extension.base.info.functionality;

import java.awt.Color;
import java.awt.geom.Point2D;
import java.util.ArrayList;

import org.cresques.cts.IProjection;
import org.gvsig.fmap.geom.Geometry;
import org.gvsig.fmap.geom.GeometryLocator;
import org.gvsig.fmap.geom.Geometry.SUBTYPES;
import org.gvsig.fmap.geom.primitive.GeneralPathX;
import org.gvsig.fmap.mapcontext.ViewPort;
import org.gvsig.mobile.core.concepts.Functionality;
import org.gvsig.mobile.core.interfaces.IGvSigContext;
import org.gvsig.mobile.core.interfaces.IGvSigMouseEvent;
import org.gvsig.mobile.extension.base.layer.raster.Utils;
import org.gvsig.mobile.libProjection.cartoutil.ConversionCoords;
import org.gvsig.mobile.libProjection.interfaces.IExtendedDatum;
import org.gvsig.mobile.symbology.basic.DefaultSymbol;
import org.gvsig.mobile.symbology.legend.AbstractVectorLegend;
import org.gvsig.mobile.ui.awt.mapcontrol.MapControl;
import org.gvsig.mobile.ui.awt.statusbar.StatusBarDescription;
import org.gvsig.mobile.ui.awt.statusbar.StatusBarLabel;
import org.gvsig.mobile.ui.awt.statusbar.StatusBarManager;
import org.osgi.framework.ServiceReference;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class MeasureDistanceFunc extends Functionality {
	
	protected static final Logger logger = LoggerFactory.getLogger(MeasureDistanceFunc.class);
	protected IGvSigContext gvc;
	protected MapControl mc;
	
	protected StatusBarDescription barDescription;
	protected StatusBarLabel lastLabel;
	protected StatusBarLabel totLabel;
	
	protected ArrayList verticesCoords = new ArrayList();
	
	public static DefaultSymbol measureSymbol = new DefaultSymbol();
	public static int measureSymbolID = 0;
	public static final String MEASURE_GRAPHICS_ID = "MEASURE_ID";
	
	protected boolean useStarted = false;
	protected static final String BAR_ID = "MEASURE_DISTANCE_BAR_ID";

	public MeasureDistanceFunc() {
		super();
		
		measureSymbol.setColor(new Color(255, 174, 0));
		measureSymbol.setBorderColor(new Color(255, 174, 0));
		measureSymbol.setFilled(false);
		measureSymbol.setWidth(1);
	}

	public void configure() {
		// TODO Auto-generated method stub
		
	}
	
	public void stop() {
		super.stop();
		
		// mc.getMapContext().getGraphicsLayer().setVisible(false);
		mc.getMapContext().getGraphicsLayer().clearAllGraphics();
		mc.getMapContext().getGraphicsLayer().clearAllSymbols();
		verticesCoords.clear();
		useStarted = false;
		

		mc.refreshAcetate();
		if (gvc != null) {
			if (! ((StatusBarManager) gvc.getStatusBarManager()).removeStatusBar(barDescription)) {
				logger.warn("Could not remove status bar MeasureDistanceFunc (not found?)");
			}
			
		} else {
			logger.error("Could not remove status bar MeasureDistanceFunc: IGvSigContext is NULL");
		}
		barDescription = null;
	}

	public boolean execution(ServiceReference gvSIGMobileRef) {
		try {
			gvc = (IGvSigContext)bc.getService(gvSIGMobileRef);
			mc = (MapControl) gvc.getMapControl();
			
			if (!useStarted) {
				ViewPort vp = mc.getViewPort();
				double v = 1;
				try {
					v = vp.getAdjustedEnvelope().getLength(0) / vp.getImageWidth();
					AbstractVectorLegend.setMetricsInSymbol(vp);
				} catch (Exception ex) {
					logger.warn("Using DIST tool before adding a map: Assumed PIXEL DIST = 1");
					DefaultSymbol.MAP_MIN_WX = 0;
					DefaultSymbol.MAP_MAX_WY = 0;
					DefaultSymbol.setMapDistOfPixel(1);
				}

				mc.getMapContext().getGraphicsLayer().clearAllGraphics();
				mc.getMapContext().getGraphicsLayer().clearAllSymbols();
				verticesCoords.clear();
				useStarted = true;
			}
			barDescription =
				new StatusBarDescription(StatusBarDescription.PREFERENCE_HIGH, BAR_ID);
			measureSymbolID = mc.getMapContext().getGraphicsLayer().addSymbol(measureSymbol);
			mc.getMapContext().getGraphicsLayer().setVisible(true);
			return true;
		} catch (Exception e) {
			logger.error("While executing MeasureCoordinateFunc: " + e.getMessage());
			return false;
		}
	}

	public void onNewMouseEventListened(IGvSigMouseEvent event) {
		
		Geometry aux = null;
		AbstractVectorLegend.setMetricsInSymbol(mc.getViewPort());

		java.awt.geom.Point2D mp = mc.getViewPort().toMapPoint(event.getX(), event.getY());
		verticesCoords.add(mp);

		mc.getMapContext().getGraphicsLayer().removeGraphics(
				MEASURE_GRAPHICS_ID);

		try {
			aux = GeometryLocator.getGeometryManager().createPoint(mp.getX(), mp.getY(), SUBTYPES.GEOM2D);
			mc.getMapContext().getGraphicsLayer().addGraphic(
					MEASURE_GRAPHICS_ID,
					aux, measureSymbolID);
			if (verticesCoords.size() > 1) {
				aux = getGeometry(verticesCoords);
				mc.getMapContext().getGraphicsLayer().addGraphic(
						MEASURE_GRAPHICS_ID,
						aux, measureSymbolID);
			}
		} catch (Exception e) {
			logger.error("While creating vertex geometry: " + e.getMessage());
			return;
		}

		mc.refreshAcetate();
		
		if (verticesCoords.size() < 2) {
			return;
		}
		
		// ============ last, tot in status bar
		
		setStatusBar();
		StatusBarManager sbm = (StatusBarManager) gvc.getStatusBarManager(); 
		sbm.addStatusBar(barDescription);
	}

	protected void setStatusBar() {
		
		double[] tot_last = getTotLast(
				verticesCoords,
				mc.getViewPort().getProjection(),
				mc.getViewPort().getDist1pixel());
		
		String[] str = new String[2];
		str[0] = Utils.getFormattedDouble(tot_last[0], 2, false) + " m";
		str[1] = Utils.getFormattedDouble(tot_last[1], 2, false) + " m";
		
		if (totLabel == null) {
			totLabel = new StatusBarLabel(str[0]);
		} else {
			totLabel.setText(str[0]);
		}
		
		if (lastLabel == null) {
			lastLabel = new StatusBarLabel(str[1]);
		} else {
			lastLabel.setText(str[1]);
		}

		
		barDescription.add(new Integer(50), totLabel);
		barDescription.add(new Integer(50), lastLabel);
	}

	private double[] getTotLast(ArrayList varray, IProjection pro, double pix_w) {
		
		int sz = varray.size();
		double[] resp = new double[2];
		for (int i=0; i<(sz-1); i++) {
			resp[1] = getDist((Point2D) varray.get(i), (Point2D) varray.get(i+1), pro, pix_w);
			resp[0] = resp[0] + resp[1];
		}
		return resp;
	}

	protected Geometry getGeometry(ArrayList vv) {
		
		int sz = vv.size();
		Geometry resp = null;
		if (sz < 2) {
			try {
				resp = GeometryLocator.getGeometryManager().createNullGeometry(SUBTYPES.GEOM2D);
			} catch (Exception e) {
				logger.error("While creating null geom: " + e.getMessage());
			}
			return resp;
		}
		
		Point2D fromp = (Point2D) vv.get(0);
		Point2D top = null;
		
		GeneralPathX gpx = new GeneralPathX();
		gpx.moveTo(fromp.getX(), fromp.getY());
		for (int i=1; i<sz; i++) {
			top = (Point2D) vv.get(i);
			gpx.lineTo(top.getX(), top.getY());
		}
		
		try {
			resp = GeometryLocator.getGeometryManager().createCurve(gpx, SUBTYPES.GEOM2D);
		} catch (Exception e) {
			logger.error("While creating null geom: " + e.getMessage());
		}
		return resp;
	}
	
	private double getDist(Point2D a, Point2D b, IProjection p, double pixel_w) {
		
		double dist = 0;
		if (p.isProjected()) {
			dist = a.distance(b);
			// dist non projected 
		} else {
			
			try {
				dist = ConversionCoords.PIGbsl(
						a.getX(), a.getY(),
						b.getX(), b.getY(),
						((IExtendedDatum) p.getDatum()).getEllipsoid(), pixel_w/2)[2];
			} catch (Exception e) {
				logger.error("While computing distance: " + e.getMessage());
				dist = 0;
			}
		}
		return dist;
	}
	
	
	
	/*
		String[] resp = new String[2];
		resp[0] = Utils.getFormattedDouble(x, decimals, false) + " " + units;
		resp[1] = Utils.getFormattedDouble(y, decimals, false) + " " + units;
	 * 
	 */

}
