package org.gvsig.mobile.extension.base.info.functionality;

import java.awt.Color;
import java.awt.geom.Point2D;
import java.util.ArrayList;

import org.gvsig.fmap.dal.exception.DataException;
import org.gvsig.fmap.dal.feature.FeatureQuery;
import org.gvsig.fmap.dal.feature.FeatureSet;
import org.gvsig.fmap.dal.feature.FeatureStore;
import org.gvsig.fmap.dal.feature.FeatureType;
import org.gvsig.fmap.geom.Geometry;
import org.gvsig.fmap.geom.GeometryManager;
import org.gvsig.fmap.geom.Geometry.SUBTYPES;
import org.gvsig.fmap.geom.primitive.Curve;
import org.gvsig.fmap.geom.primitive.GeneralPathX;
import org.gvsig.fmap.geom.primitive.Surface;
import org.gvsig.fmap.mapcontext.ViewPort;
import org.gvsig.fmap.mapcontext.layers.vectorial.FLyrVect;
import org.gvsig.fmap.mapcontext.layers.vectorial.GraphicLayer;
import org.gvsig.fmap.mapcontext.rendering.symbols.ISymbol;
import org.gvsig.mobile.core.concepts.Functionality;
import org.gvsig.mobile.core.constants.GvSigMouseEventType;
import org.gvsig.mobile.core.interfaces.IGvSigContext;
import org.gvsig.mobile.core.interfaces.IGvSigMouseEvent;
import org.gvsig.mobile.graphicLayer.IntersectsGeometryEvaluatorCdc;
import org.gvsig.mobile.symbology.legend.AbstractVectorLegend;
import org.gvsig.mobile.ui.awt.mapcontrol.MapControl;
import org.gvsig.tools.evaluator.Evaluator;
import org.osgi.framework.ServiceReference;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;



public class RectangleSelectionFunc extends Functionality {
	
	private static final Logger logger = LoggerFactory.getLogger(RectangleSelectionFunc.class);
	
	private IGvSigContext gvc;
	private FLyrVect layer;
	
	private ViewPort viewPort;
	private MapControl canvas;
	private int symbolId = 0;

	private Point2D startp = null;
	private Point2D endp = null;

	double minx = 0;
	double maxx = 0;
	double miny = 0;
	double maxy = 0;

	ServiceReference gvSIGMobileRef;
	GraphicLayer grf;
	GeometryManager gm = null;

	public static final String RECT_SEL_TOOL = "RECT_SEL_TOOL";

	
	public RectangleSelectionFunc() {
		super();
	}

	public void configure() {
		// TODO Auto-generated method stub
		
	}

	public boolean execution(ServiceReference gvSIGMobileRef) {
		
		try {
			gm = org.gvsig.fmap.geom.GeometryLocator.getGeometryManager();
			
			gvc = (IGvSigContext)bc.getService(gvSIGMobileRef);
			canvas = (MapControl) gvc.getMapControl();
			grf = canvas.getMapContext().getGraphicsLayer();
		} catch (Exception ex) {
			logger.error("While executing rect sel: " + ex.getMessage());
			return false;
		}
		
		ArrayList infolyrs = canvas.getLayersForInfo();
		
		if (infolyrs == null || infolyrs.size() != 1) {
			logger.warn("There must be exactly one layer for selection.");
			stop();
			return false;
		} else {
            ISymbol sym = org.gvsig.fmap.mapcontext.MapContextLocator
			.getSymbolManager().createSymbol(Geometry.TYPES.SURFACE, Color.red);
			if(sym!=null){
				symbolId = grf.addSymbol(sym);			
			}
			return true;
		}
	}

	public void onNewMouseEventListened(IGvSigMouseEvent event) {
		
		ArrayList infolyrs = canvas.getLayersForInfo();
		
		if (infolyrs == null || infolyrs.size() != 1) {
			logger.error("There must be exactly one layer for selection.");
			return;
		}
		
		layer = (FLyrVect) infolyrs.get(0);

		int type = event.getID();

		ViewPort vp = canvas.getViewPort();
		if (type == GvSigMouseEventType.MOUSE_PRESSED) {
			grf.removeGraphics(RECT_SEL_TOOL);
			startp = vp.toMapPoint(event.getX(), event.getY());
			AbstractVectorLegend.setMetricsInSymbol(canvas.getViewPort());
		} else

		if (type == GvSigMouseEventType.MOUSE_DRAGGED) {	
			Point2D tmpp = null;
			grf.removeGraphics(RECT_SEL_TOOL);		
			tmpp = vp.toMapPoint(event.getX(), event.getY());

			minx = Math.min(startp.getX(), tmpp.getX());
			maxx = Math.max(startp.getX(), tmpp.getX());
			miny = Math.min(startp.getY(), tmpp.getY());
			maxy = Math.max(startp.getY(), tmpp.getY());

			GeneralPathX gpx = new GeneralPathX();
			gpx.moveTo(minx, miny);
			gpx.lineTo(minx, maxy);
			gpx.lineTo(maxx, maxy);
			gpx.lineTo(maxx, miny);
			gpx.lineTo(minx, miny);

			Curve cur;
			try {
				cur = gm.createCurve(gpx,
						org.gvsig.fmap.geom.Geometry.SUBTYPES.GEOM2D);
				grf.addGraphic(RECT_SEL_TOOL, cur, symbolId);		
				canvas.refreshAcetate();
				
			} catch (Exception e) {
				logger.error("While updating rect in drag: " + e.getMessage());
			}
		} else

		if (type == GvSigMouseEventType.MOUSE_RELEASED && startp != null) {

			grf.removeGraphics(RECT_SEL_TOOL);

			endp = vp.toMapPoint(event.getX(), event.getY());
			minx = Math.min(startp.getX(), endp.getX());
			maxx = Math.max(startp.getX(), endp.getX());
			miny = Math.min(startp.getY(), endp.getY());
			maxy = Math.max(startp.getY(), endp.getY());
			
			GeneralPathX gpx = new GeneralPathX();
			gpx.moveTo(minx, miny);
			gpx.lineTo(minx, maxy);
			gpx.lineTo(maxx, maxy);
			gpx.lineTo(maxx, miny);
			gpx.lineTo(minx, miny);

			Surface surf = null;
			try {
				surf = gm.createSurface(gpx, SUBTYPES.GEOM2D);
			} catch (Exception ex) {
				logger.error("While creating intersect rect: " + ex.getMessage());
			}

			FeatureStore featureStore = layer.getFeatureStore();
			FeatureType ft = null;
			try {
				ft = featureStore.getDefaultFeatureType();
			} catch (DataException e1) {
				logger.error("While getting featuretype to get  geometry name: " + e1.getMessage());
				return;
			}
			
			String geomname = ft.getDefaultGeometryAttributeName();
			
			FeatureQuery fq = featureStore.createFeatureQuery();

			Evaluator evaluator=new IntersectsGeometryEvaluatorCdc(
					surf, vp.getProjection(),
					ft, geomname);
			
			fq.setFilter(evaluator);
			FeatureSet set;
			try {
				set = featureStore.getFeatureSet(fq);
				featureStore.getFeatureSelection().deselectAll();
				featureStore.getFeatureSelection().select(set);		
			} catch (DataException e) {
				logger.error("While setting selection: " + e.getMessage());
			}
			canvas.setDirty();
		}		
	}
}
