/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2006 Prodevelop and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *   Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *   +34 963862235
 *   gvsig@gva.es
 *   http://www.gvsig.gva.es
 *
 *    or
 *
 *   Prodevelop Integracin de Tecnologas SL
 *   Conde Salvatierra de lava , 34-10
 *   46004 Valencia
 *   Spain
 *
 *   +34 963 510 612
 *   +34 963 510 968
 *   gis@prodevelop.es
 *   http://www.prodevelop.es
 *
 *    or
 *
 *   Instituto de Robtica
 *   Apartado de correos 2085
 *   46071 Valencia
 *   (Spain)
 *   
 *   +34 963 543 577
 *   jjordan@robotica.uv.es
 *   http://robotica.uv.es
 *   
 */

package org.gvsig.mobile.extension.base.layer.dialog;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Insets;
import java.util.ArrayList;

import org.gvsig.fmap.mapcontext.layers.vectorial.FLyrVect;
import org.gvsig.i18n.Messages;
import org.gvsig.mobile.core.auxfunctions.FileTypeResolutor;
import org.gvsig.mobile.core.exceptions.GvSigServiceNotFoundException;
import org.gvsig.mobile.core.extensions.interfaces.IFileProviderFunctionality;
import org.gvsig.mobile.core.extensions.interfaces.IFileType;
import org.gvsig.mobile.extension.base.util.GraphicTextItem;
import org.gvsig.mobile.ui.awt.IWindowInfo;
import org.gvsig.mobile.ui.awt.WindowManager;
import org.gvsig.mobile.ui.lw.DialogConstants;
import org.gvsig.mobile.ui.lw.FileChooserDialog;
import org.gvsig.mobile.ui.lw.FileChooserListener;
import org.gvsig.mobile.ui.lw.LongTaskDialog;
import org.gvsig.mobile.ui.lw.OkCancelDialog;
import org.osgi.framework.BundleContext;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.zaval.lw.LwAList;
import org.zaval.lw.LwBorderLayout;
import org.zaval.lw.LwButton;
import org.zaval.lw.LwCombo;
import org.zaval.lw.LwCompList;
import org.zaval.lw.LwComponent;
import org.zaval.lw.LwComposite;
import org.zaval.lw.LwConstraints;
import org.zaval.lw.LwContainer;
import org.zaval.lw.LwGridLayout;
import org.zaval.lw.LwLabel;
import org.zaval.lw.LwPanel;
import org.zaval.lw.LwToolkit;
import org.zaval.lw.LwWrappedText;
import org.zaval.lw.event.LwActionListener;

public class ExportLayerDialog extends OkCancelDialog implements LwComposite,
		IWindowInfo, LwActionListener, FileChooserListener {

	private static Logger logger = LoggerFactory.getLogger(ExportLayerDialog.class);
	private LwButton pathButton;
	private String saveDirectory = null;
	
	private LwLabel fileNameField;	
	private LwCombo formats;
	private LwCombo layers;
	private ArrayList vectorLayers = new ArrayList();

	public ExportLayerDialog(BundleContext bc, ArrayList vlayers) {
		super(false, bc, false);
		vectorLayers = vlayers;
		initialize(false);
	}	

	private LwCombo getLyrCombo() {
		
		LwCombo resp = new LwCombo();		
		LwCompList l = new LwCompList();

		for (int i = 0; i < vectorLayers.size(); i++) {
			FLyrVect lyr = (FLyrVect) vectorLayers.get(i);
			GraphicTextItem item = new  GraphicTextItem(lyr.getName(), lyr, i);
			l.addItem(item);
		}

		if (l.count() > 0) {
			l.select(0);
		}

		l.addSelectionListener(this);
		resp.setList(l);
		return resp;
	}

	private LwCombo getFormatsCombo() {
		LwCombo resp = new LwCombo();		
		LwCompList l = new LwCompList();	
		
		IFileType[] avtypes = null;
		try {
			String filter = "(" +
				FileTypeResolutor.ACTION_WRITE_PARAMETER_KEY +
				"=" + 
				FileTypeResolutor.ACTION_YES + ")"; 
			avtypes = FileTypeResolutor.getRegisteredFileTypes(filter, bc);
		} catch (GvSigServiceNotFoundException e1) {
			logger.warn("No file types found: " + e1.getMessage());
			return resp;
		}
		
		for (int i=0; i<avtypes.length; i++) {
			GraphicTextItem item = new GraphicTextItem(
					avtypes[i].getFileTypeDescription(),
					avtypes[i], i);
			l.addItem(item);
		}
		
		if (l.count() > 0) {
			l.select(0);
		}
		resp.setList(l);
		return resp;
	}

	private FLyrVect getSelectedLayer() {
		
		LwAList list = layers.getList(); 
		int sel = list.getSelectedIndex();
		if (sel >= 0) {
			return (FLyrVect) ((GraphicTextItem) list.getSelected()).getId();
		} else {
			return null;
		}
	}

	private IFileType getSelectedFileType() {
		
		Object selitemobj = formats.getList().getSelected();
		
		if (selitemobj == null) {
			logger.warn("No item selected.");
			return null;
		}
		GraphicTextItem selitem = (GraphicTextItem) selitemobj; 
		IFileType resp = null;
		
		try {
			resp = (IFileType) selitem.getId();
		} catch (Exception e) {
			logger.error("While getting functionality: " + e.getMessage());
		} 
		return resp;
	}
	
	public void actionPerformed(Object src, Object arg1) {

		if (src == pathButton) {

			FileChooserDialog dlg =
				new FileChooserDialog(
						this, null, null, // new String[][]{UIConstants.FILTER_ALL},
						FileChooserDialog.DIALOG_TYPE_CREATE_NEW); //$NON-NLS-1$

			dlg.setTitle(Messages.getText("ProjectPropertiesDialog.22"));
			// if (saveDirectory == null) {
			// dlg.setDirectory(AppSettings.defaultFormsSavePath.getPath());
			// } else {
			// dlg.setDirectory(saveDirectory);
			// }

			dlg.showDialog();
		} else if (src == this.getOkButton()) {
			// TODO Aqu y en otras operaciones que duren varios segundos,
			// vendra bien ejecutarlas en segundo plano y mostrar
			// un icono o barra de progreso
			WindowManager.getInstance().closeActivePanel();
			// toc.getMapControl().repaint();
			executeExport();
		} else

		if (src == this.getCancelButton()) {
//			this.getOkButton().setEnabled(!this.getOkButton().isEnabled());
			WindowManager.getInstance().closeActivePanel();
		} else if (src instanceof LwAList) {
			this.getOkButton().setEnabled(this.getSelectedLayer() != null);
		}

	}

	private void executeExport() {

		FLyrVect exp_lyr = getSelectedLayer();
		
		if (exp_lyr == null) {
			return;
		}
		
		IFileType fity = this.getSelectedFileType();
		IFileProviderFunctionality dothis = null;
		
		try {
			dothis = (IFileProviderFunctionality) fity.getFunctionality();
		} catch (GvSigServiceNotFoundException e) {
			logger.error("Error: " + e.getMessage());
			return;
		}

		// dothis.get
		dothis.setFilePath(saveDirectory);
		dothis.getProps().put(FLyrVect.class.getName(), exp_lyr);
		dothis.setFileType(fity);
		
		LongTaskDialog dlg = new LongTaskDialog(
				"Silly task", dothis, this.getWidth(), LongTaskDialog.START_TASK_WITH_EXECUTE, true);
		//			
		// if (getSelectedLayer() == null) return;
		// if (saveDirectory!=null){
		// String writer = formatChoice.getSelectedItem();
		// ExportGpeWriter gpeWriter = new ExportGpeWriter();
		// try {
		// String aux = fileNameField.getText();
		// gpeWriter.write((FLyrVect) vectLayer, saveDirectory + File.separator
		// + aux, writer);
		// }catch (Exception e) {
		// logger.error("While exporting vector layer: " + e.getMessage());
		// }
		// }
		// return;
	}

	public boolean catchInput(LwComponent arg0) {
		// TODO Auto-generated method stub
		return false;
	}

	public Dimension getDesiredSize() {
		// TODO Auto-generated method stub
		return new Dimension(240, 300);
	}

	public boolean isMaximized() {
		// TODO Auto-generated method stub
		return true;
	}

	public void fileSelected(String _path) {

		if (_path == null)
			return;

		String ext = null; // "txt";
		if (formats != null && formats.getList() != null && formats.getList().getSelected() != null) {
			ext = formats.getList().getSelected().toString().toLowerCase();;
		}
		String path = _path;
		if (ext != null && ext.length() > 0) {
			path = path + "." + ext;
		}
		saveDirectory = path;		
		fileNameField.setText(path);
	}

	public LwPanel buildMainPanel() {
		LwPanel resp = new LwPanel();
		
		resp.setLwLayout(new LwBorderLayout());

		LwConstraints l = DialogConstants.getTextFieldConstraints();

		LwPanel left = new LwPanel();
		
		left.setLwLayout(new LwGridLayout(3, 1));
		LwLabel lyrLabel = DialogConstants.getDefaultLabel(Messages.getText("ExportDialog.1"));
		LwLabel frmtLabel = DialogConstants.getDefaultLabel(Messages.getText("ExportDialog.2"));
		LwLabel projectLabel = DialogConstants.getDefaultLabel(Messages.getText("ExportDialog.3"));
		
		LwConstraints l1 = DialogConstants.getLabelConstraints();
		l1.insets = new Insets(3,3,3,3);
		
		left.add(l1, lyrLabel);
		left.add(l1, frmtLabel);
		left.add(l1, projectLabel);		
		
		LwPanel center = new LwPanel();
		
		center.setLwLayout(new LwGridLayout(3, 1, LwToolkit.HORIZONTAL));
		
		LwContainer c = new LwPanel();
		c.setLwLayout(new LwBorderLayout());

		
		LwWrappedText wtext  = new LwWrappedText("");
		fileNameField = new LwLabel(wtext);
		fileNameField.setBackground(Color.WHITE);
		
		
		Dimension  dime = WindowManager.getInstance().getMainSize();
		if (dime != null) {
			fileNameField.setPSSize(-1, dime.height / 8);
		} else {
			fileNameField.setPSSize(-1, 45);
		}
		
		
		
		
		c.add(LwBorderLayout.CENTER, fileNameField);

		pathButton = getPathButton();
		c.add(LwBorderLayout.EAST, pathButton);
		
		this.layers = this.getLyrCombo();
		this.formats = this.getFormatsCombo();		
		
		center.add(l, layers);
		center.add(l, formats);
		center.add(l, c);
		
		resp.add(LwBorderLayout.WEST, left);
		resp.add(LwBorderLayout.CENTER, center);
		
		return resp;
	}
	
	private LwButton getPathButton() {
		return DialogConstants.getButton("carpeta", "carpeta", this);		
	}

	public String getTitle() {
		return Messages.getText("ExportDialog.0");
	}
}
