/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2006 Prodevelop and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *   Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *   +34 963862235
 *   gvsig@gva.es
 *   http://www.gvsig.gva.es
 *
 *    or
 *
 *   Prodevelop Integracin de Tecnologas SL
 *   Conde Salvatierra de lava , 34-10
 *   46004 Valencia
 *   Spain
 *
 *   +34 963 510 612
 *   +34 963 510 968
 *   gis@prodevelop.es
 *   http://www.prodevelop.es
 *
 *    or
 *
 *   Instituto de Robtica
 *   Apartado de correos 2085
 *   46071 Valencia
 *   (Spain)
 *   
 *   +34 963 543 577
 *   jjordan@robotica.uv.es
 *   http://robotica.uv.es
 *   
 */
package org.gvsig.mobile.extension.base.layer.dialog;

import java.awt.Insets;

import org.gvsig.fmap.mapcontext.layers.FLayer;
import org.gvsig.fmap.mapcontext.layers.vectorial.FLyrVect;
import org.gvsig.i18n.Messages;
import org.gvsig.mobile.extension.base.forms.view.CustomForm;
import org.gvsig.mobile.ui.awt.mapcontrol.MapControl;
import org.gvsig.mobile.ui.lw.DialogConstants;
import org.gvsig.mobile.ui.lw.FileChooserDialog;
import org.gvsig.mobile.ui.lw.FileChooserListener;
import org.gvsig.mobile.ui.lw.OkCancelDialog;
import org.gvsig.mobile.ui.lw.UIConstants;
import org.osgi.framework.BundleContext;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.zaval.lw.LwBorderLayout;
import org.zaval.lw.LwButton;
import org.zaval.lw.LwCheckbox;
import org.zaval.lw.LwConstraints;
import org.zaval.lw.LwGridLayout;
import org.zaval.lw.LwPanel;
import org.zaval.lw.LwTextField;
import org.zaval.lw.LwToolkit;

public class LayerPropertiesDialog extends OkCancelDialog implements FileChooserListener {

	private LwTextField nameText;
	private LwTextField fileText;
	private LwCheckbox visibleCheck;
	private LwCheckbox selectCheck;
	private LwCheckbox maxScaleCheck;
	private LwCheckbox minScaleCheck;
	private LwTextField maxScaleText;
	private LwTextField minScaleText;
	private LwTextField formText;

	private LwTextField commentText;
	private LwButton pathButton;
	private LwCheckbox formCheck;
	
	private FLayer layer;

	private final Logger logger = LoggerFactory.getLogger(LayerPropertiesDialog.class);
	private LayerManagerDialog managerDialog;
	
	public LayerPropertiesDialog(boolean scrollable, BundleContext bc, FLayer layer, LayerManagerDialog d) {
		super(scrollable, bc, true);
		
		this.layer = layer;
		this.managerDialog = d;
		
		//fill the data after having built the main panel
		this.fillData();
		
	}

	public void actionPerformed(Object arg0, Object arg1) {
		if (arg0 == this.getOkButton()) {
			saveLayer();
			managerDialog.rebuildTOC();
			close();
		} else if (arg0 == this.getCancelButton()) {
			close();
		} else if (arg0 == pathButton) {
			FileChooserDialog d = new FileChooserDialog(this, null, new String[][]{UIConstants.FILTER_ALL}, FileChooserDialog.DIALOG_TYPE_SELECT);
			d.setTitle(Messages.getText("ProjectPropertiesDialog.22"));
			d.showDialog();
		}
	}

	public LwPanel buildMainPanel() {
		LwPanel resp = new LwPanel();		
		resp.setLwLayout(new LwBorderLayout());

		LwConstraints l = DialogConstants.getTextFieldConstraints();
		
		LwConstraints l2 = DialogConstants.getLabelConstraints();
		if(!DialogConstants.isResolution640())
			l2.insets = new Insets(2,2,2,2);
		else
			l2.insets = new Insets(4,4,4,4);
		
		l2.fill = LwToolkit.NONE;
		l2.ax = LwToolkit.LEFT;

		LwPanel left = new LwPanel();
		left.setLwLayout(new LwGridLayout(7, 1));		
		
		left.add(l2, DialogConstants.getDefaultLabel(Messages.getText("LayerPropertiesDialog.1")));
		left.add(l2, DialogConstants.getDefaultLabel(Messages.getText("LayerPropertiesDialog.2")));

		visibleCheck = new LwCheckbox(DialogConstants.getDefaultLabel(Messages.getText("LayerPropertiesDialog.4")), LwCheckbox.CHECK);
		left.add(l2, visibleCheck);		
		
		selectCheck = new LwCheckbox(DialogConstants.getDefaultLabel(Messages.getText("LayerPropertiesDialog.5")), LwCheckbox.CHECK);
		left.add(l2, selectCheck);		
		
		formCheck = new LwCheckbox(DialogConstants.getDefaultLabel(Messages.getText("LayerPropertiesDialog.8")), LwCheckbox.CHECK);
		left.add(l2, formCheck);
		maxScaleCheck = new LwCheckbox(DialogConstants.getDefaultLabel(Messages.getText("LayerPropertiesDialog.6")), LwCheckbox.CHECK);
		left.add(l2, maxScaleCheck);	
		
		minScaleCheck = new LwCheckbox(DialogConstants.getDefaultLabel(Messages.getText("LayerPropertiesDialog.7")), LwCheckbox.CHECK);
		left.add(l2, minScaleCheck);

		LwPanel right = new LwPanel();
		right.setLwLayout(new LwGridLayout(7, 1, LwToolkit.HORIZONTAL));

		nameText = getTextField();
		right.add(l, nameText);

		fileText = getTextField();
		right.add(l, fileText);
		
		right.add(l, DialogConstants.getDefaultLabel(""));
		right.add(l, DialogConstants.getDefaultLabel(""));		

		LwPanel formPanel = new LwPanel();
		formPanel.setLwLayout(new LwBorderLayout());

		formText = getTextField();
		pathButton = getPathButton();		
				
		formPanel.add(LwBorderLayout.CENTER, formText);
		formPanel.add(LwBorderLayout.EAST, pathButton);		

		right.add(l, formPanel);

		LwPanel maxScalePanel = new LwPanel();
		maxScalePanel.setLwLayout(new LwBorderLayout());

		maxScaleText = getTextField();
		
		maxScalePanel.add(LwBorderLayout.CENTER, maxScaleText);
		
		right.add(l, maxScalePanel);
		
		LwPanel minScalePanel = new LwPanel();
		minScalePanel.setLwLayout(new LwBorderLayout());

		minScaleText = getTextField();
		
		minScalePanel.add(LwBorderLayout.CENTER, minScaleText);
		
		right.add(l, minScalePanel);		
		
		LwPanel top = new LwPanel();
		top.setLwLayout(new LwBorderLayout());

		top.add(LwBorderLayout.WEST, left);
		top.add(LwBorderLayout.CENTER, right);
		
		LwPanel center = new LwPanel();
		center.setLwLayout(new LwBorderLayout());
		
		center.add(LwBorderLayout.WEST, DialogConstants.getDefaultLabel(Messages.getText("LayerPropertiesDialog.3")));
		commentText = getTextField();
		center.add(LwBorderLayout.CENTER, commentText);
		
		resp.add(LwBorderLayout.NORTH, top);
		resp.add(LwBorderLayout.CENTER, center);		
		
		return resp;
	}
	
	private void saveLayer() {
		layer.setName(this.nameText.getText());
		
		try {
			if (this.maxScaleCheck.getState()) {
				layer.setMaxScale(Double.parseDouble(this.maxScaleText.getText()));
			} else {
				layer.setMaxScale(-1);
			}
		} catch (Exception e) {
			
		}
		
		try {
			if (this.minScaleCheck.getState()) {
				layer.setMinScale(Double.parseDouble(this.minScaleText.getText()));
			} else {
				layer.setMinScale(-1);
			}
		} catch (Exception e) {
			
		}
		
		layer.setVisible(this.visibleCheck.getState());
		
		Boolean infoed = new Boolean(selectCheck.getState());
		if (layer.hasDynValue(MapControl.INFO_ACTIVATED_DYN_PROPERTY_NAME)) {
			layer.setDynValue(MapControl.INFO_ACTIVATED_DYN_PROPERTY_NAME, infoed);
		}

		// ==================================================
		Boolean usesform = new Boolean(formCheck.getState());
		layer.setDynValue(CustomForm.PROPERTY_KEY_USES_CUSTOM_FORM, usesform);
		// ==================================================
		String formpath = formText.getText();
		layer.setDynValue(CustomForm.PROPERTY_KEY_CUSTOM_FORM_PATH, formpath);
		// ==================================================

	}
	
	private void fillData() {
		
		if (layer.isVisible())
			visibleCheck.setState(true);
		
		Boolean infoed = new Boolean(false);
		if (layer.hasDynValue(MapControl.INFO_ACTIVATED_DYN_PROPERTY_NAME)) {
			infoed = (Boolean) layer.getDynValue(MapControl.INFO_ACTIVATED_DYN_PROPERTY_NAME);
		}
		this.selectCheck.setState(infoed.booleanValue());
		
		
		
		this.setMaxScaleFromLayer();
		this.setMinScaleFromLayer();
		
		try {
			if(layer instanceof FLyrVect) {
				String name = ((FLyrVect) layer).getFeatureStore().getParameters().getDynValue("shpfilename").toString();
				this.fileText.setText(name);
			}
		} catch (Exception e) {
			logger.error(e.getMessage());
		}
		
		String name = layer.getName();
		if (name != null) {
			this.nameText.setText(name);
		} 
		
		// ==================================================
		Boolean usesform = new Boolean(false);
		if (layer.hasDynValue(CustomForm.PROPERTY_KEY_USES_CUSTOM_FORM)) {
			usesform = (Boolean) layer.getDynValue(CustomForm.PROPERTY_KEY_USES_CUSTOM_FORM);
		}
		this.formCheck.setState(usesform.booleanValue());
		// ==================================================
		String formpath = "";
		if (layer.hasDynValue(CustomForm.PROPERTY_KEY_CUSTOM_FORM_PATH)) {
			formpath = (String) layer.getDynValue(CustomForm.PROPERTY_KEY_CUSTOM_FORM_PATH);
		}
		this.formText.setText(formpath);
		formText.setEditable(false);
		// ==================================================
		
		
		this.fileText.setEditable(false);
	}
	
	private void setMaxScaleFromLayer() {
		final double maxScale = this.layer.getMaxScale();
		
		if (maxScale == -1) return;
		
		this.maxScaleCheck.setState(true);
		this.maxScaleText.setText(String.valueOf(maxScale));
	}
	
	private void setMinScaleFromLayer() {
		final double minScale = this.layer.getMinScale();
		
		if (minScale == -1) return;
		
		this.minScaleCheck.setState(true);
		this.minScaleText.setText(String.valueOf(minScale));		
	}

	private LwButton getPathButton() {
		return DialogConstants.getButton("carpeta", "carpeta", this);		
	}

	private LwTextField getTextField() {
		return DialogConstants.getDefaultTextField();
	}

	public void fileSelected(String path) {
		this.formText.setText(path);
	}

	public String getTitle() {
		return Messages.getText("LayerPropertiesDialog.0");
	}
}
