package org.gvsig.mobile.extension.base.layer.functionality;

import java.awt.image.BufferedImage;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.lang.reflect.Method;

import org.gvsig.i18n.Messages;
import org.gvsig.mobile.core.concepts.Functionality;
import org.gvsig.mobile.core.interfaces.IGvSigContext;
import org.gvsig.mobile.core.interfaces.IGvSigMouseEvent;
import org.gvsig.mobile.ui.awt.mapcontrol.MapControl;
import org.gvsig.mobile.ui.lw.MessageBox;
import org.osgi.framework.ServiceReference;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import es.prodevelop.gvsig.mobile.fmap.driver.raster.ecw.EcwReader;

public class CaptureScreenFunc extends Functionality {
	
	private static Logger logger = LoggerFactory.getLogger(CaptureScreenFunc.class);

	public CaptureScreenFunc() {
		super();
	}

	public void configure() {
		// TODO Auto-generated method stub
		
	}

	public boolean execution(ServiceReference gvSIGMobileRef) {
		try {
			
			IGvSigContext gvc = (IGvSigContext)bc.getService(gvSIGMobileRef);
			MapControl mc = (MapControl) gvc.getMapControl();
			BufferedImage img = mc.getMapImage();
			int w = img.getWidth();
			int h = img.getHeight();
			
			String img_name = ("IM_" + System.currentTimeMillis() % 100000) + ".png"; //$NON-NLS-1$ //$NON-NLS-2$
			img_name = System.getProperty("user.home") + File.separator + img_name;
			File fout = new File(img_name);
			if (fout.exists()) {
				logger.warn("PNG file existed (too bad!) and was deleted: " + img_name);
				fout.delete();
			}
			int[] pix = new int[w*h];
			img.getRGB(0,0,w,h,pix,0,w);
			
			if (USING_PDA) {
				
				byte[] png = EcwReader.paintArray(pix, w, h, 0, false);
				writeBytesToFile(png, img_name);
				
			} else {
				
				String a = "javax";
				String b = "imageio";
				String c = "ImageIO";
				Class imageioclass = null; 
				imageioclass = Class.forName(a + "." + b + "." + c);
				Class[] params = new Class[3];
				Class renderedimageclass = Class.forName("java.awt.image.RenderedImage");
				params[0] = renderedimageclass;
				params[1] = String.class;
				params[2] = File.class;
				Method writem = imageioclass.getMethod("write", params);
				Object[] param_values = new Object[3];
				param_values[0] = img;
				param_values[1] = "png";
				param_values[2] = fout;
				writem.invoke(null, param_values);
				
			}
			
			MessageBox.showDialog(true, bc,
					Messages.getText("image_stored") + ": " + img_name,
					null, MessageBox.MESSAGE_TYPE_INFO, MessageBox.MESSAGE_ONLY_OK);

			super.stop();
			return true;
		} catch (Exception e) {
			logger.error("While capturing screen: " + e.getMessage());
			return false;
		}
	}

	public void onNewMouseEventListened(IGvSigMouseEvent event) {
		// TODO Auto-generated method stub
		
	}
	
	
	/**
	 * Writes byte array to file
	 * @param data 
	 * @param path file path
	 * @return whether the writing was successful
	 */
	public static boolean writeBytesToFile(byte[] data, String path) {
		try {
			File f = new File(path);
			if (f.exists())
				f.delete();
			FileOutputStream fos = new FileOutputStream(f);
			fos.write(data);
			fos.flush();
			fos.close();
			return true;
		} catch (IOException ex) {
			logger.error("While writing data to file: " + ex.getMessage());
			return false;
		}
	}
	
	public static boolean USING_PDA = false;
	static {
		if (System.getProperty("os.name").equalsIgnoreCase("WINDOWS CE"))
			USING_PDA = true;
	}
}
