/*
 * Created on 20-dic-2004
 */
/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
/************************************************
 *												*
 *   Modfied By:								*
 *   Prodevelop Integracin de Tecnologas SL	*
 *   Conde Salvatierra de lava , 34-10			*
 *   46004 Valencia								*
 *   Spain										*
 *												*
 *   +34 963 510 612							*
 *   +34 963 510 968							*
 *   gis@prodevelop.es							*
 *   http://www.prodevelop.es					*
 *												*
 *   gvSIG Mobile Team 2006 					*
 *  											*	 
 ************************************************/
package org.gvsig.mobile.extension.base.layer.raster;

import java.awt.geom.Rectangle2D;
import java.io.File;
import java.io.IOException;

import org.apache.log4j.Logger;


/**
 * Adapta un driver de fichero vectorial a la interfaz vectorial, manteniendo
 * adems el estado necesario por una capa vectorial de fichero (el nombre del
 * fichero)
 */
public class RasterFileAdapter extends RasterAdapter {
	
	private static Logger logger = Logger.getLogger(RasterFileAdapter.class);
	private boolean driverInitialized = false;
	private File file;

	//private FileWriterDriver writeDriver;

	/**
	 * Crea un nuevo VectorialFileAdapter.
	 *
	 * @param file Fichero.
	 */
	public RasterFileAdapter(File file) {
		this.file = file;
	}

	/**
	 * Devuelve el Driver Raster.
	 *
	 * @return RasterDriver.
	 */
	private RasterDriver getFileDriver() {
		return (RasterDriver) getDriver();
	}

	/**
	 * incrementa el contador de las veces que se ha abierto el fichero.
	 * Solamente cuando el contador est a cero pide al driver que abra el
	 * fichero
	 *
	 * @throws DriverIOException
	 */
	public void start()  {

		try {
			getFileDriver().open(file);
			if (!driverInitialized) {
				getFileDriver().initialize();
				driverInitialized = true;
			}
		} catch (IOException e) {
			
		}
	}

	/**
	 * decrementa el contador de nmero de aperturas y cuando llega a cero pide
	 * al driver que cierre el fichero
	 *
	 * @throws DriverIOException
	 */
	public void stop()  {

		try {
			driverInitialized = false;
			getFileDriver().close();
		} catch (IOException e) {
			
		}
	}

	/**
	 * Devuelve el nmero de bandas de raster.
	 *
	 * @return Nmero de bandas.
	 */
	public int getNumBands() {
		return getFileDriver().getNumBands();
	}

	/**
	 * Devuelve la extensin total de la capa.
	 *
	 * @return Extensin total.
	 */
	public Rectangle2D getFullExtent() {
		return getFileDriver().getFullExtent();
	}

	/**
	 * @see com.iver.cit.gvsig.fmap.layers.ReadableVectorial#getShapeType()
	 */
	public int getRasterDataType() {
		return getFileDriver().getRasterDataType();
	}

	/**
	 * Obtiene una muestra del pixel que se pasa como parmetro en la banda que
	 * se pasa como parmetro
	 *
	 * @param x Coordenada X
	 * @param y Coordenada Y
	 * @param band Nmero de banda.
	 *
	 * @return Objeto.
	 */
	public Object getData(int x, int y, int band) {
		return getFileDriver().getData(x, y, band);
	}


	/**
	 * Devuelve el fichero.
	 *
	 * @return Fichero.
	 */
	public File getFile() {
		return file;
	}
		
}
