package org.gvsig.mobile.extension.base.layer.raster;

import java.awt.geom.Rectangle2D;

/**
 * This utility class keeps the basic information about a viewport (image size and map extent.)
 * 
 * @author Juan Lucas Dominguez Rubio, jldominguez@prodevelop.es
 *
 */
public class SimpleVP {
	
	private Rectangle2D extent = new Rectangle2D.Double(0, 0, 1, 1);
	private int imageWidth = 1;
	private int imageHeight = 1;

	public SimpleVP(Rectangle2D ext, int w, int h) {
		extent = ext;
		imageWidth = w;
		imageHeight = h;
	}

	/**
	 * @return the extent
	 */
	public Rectangle2D getExtent() {
		return extent;
	}

	/**
	 * @param extent the extent to set
	 */
	public void setExtent(Rectangle2D extent) {
		this.extent = extent;
	}

	/**
	 * @return the imageWidth
	 */
	public int getImageWidth() {
		return imageWidth;
	}

	/**
	 * @param imageWidth the imageWidth to set
	 */
	public void setImageWidth(int imageWidth) {
		this.imageWidth = imageWidth;
	}

	/**
	 * @return the imageHeight
	 */
	public int getImageHeight() {
		return imageHeight;
	}

	/**
	 * @param imageHeight the imageHeight to set
	 */
	public void setImageHeight(int imageHeight) {
		this.imageHeight = imageHeight;
	}
	
	public static SimpleVP[] getQuad(SimpleVP svp) {
		
		SimpleVP[] resp = new SimpleVP[4];
		
		int w = svp.getImageWidth();
		int h = svp.getImageHeight();
		Rectangle2D rect = svp.getExtent();
		
		double map_units_per_pixel = rect.getWidth() / w;
		
		int w1 = 0;
		int w2 = 0;
		int h1 = 0;
		int h2 = 0;
		
		// width
		if ((w % 2) == 0) { // even
			w1 = w / 2;
			w2 = w1;
		} else { // odd
			w1 = 1 + (w / 2);
			w2 = w - w1;
		}
		
		// height
		if ((h % 2) == 0) { // even
			h1 = h / 2;
			h2 = h1;
		} else { // odd
			h1 = 1 + (h / 2);
			h2 = h - h1;
		}
		
		Rectangle2D.Double itemr = new Rectangle2D.Double(
				rect.getMinX(),
				rect.getMaxY() - h1 * map_units_per_pixel,
				map_units_per_pixel * w1,
				map_units_per_pixel * h1);
		resp[0] = new SimpleVP(itemr, w1, h1);

		itemr = new Rectangle2D.Double(
				rect.getMinX() + w1 * map_units_per_pixel,
				rect.getMaxY() - h1 * map_units_per_pixel,
				map_units_per_pixel * w2,
				map_units_per_pixel * h1);
		resp[1] = new SimpleVP(itemr, w2, h1);

		itemr = new Rectangle2D.Double(
				rect.getMinX(),
				rect.getMinY(),
				map_units_per_pixel * w1,
				map_units_per_pixel * h2);
		resp[2] = new SimpleVP(itemr, w1, h2);

		itemr = new Rectangle2D.Double(
				rect.getMinX() + w1 * map_units_per_pixel,
				rect.getMinY(),
				map_units_per_pixel * w2,
				map_units_per_pixel * h2);
		resp[3] = new SimpleVP(itemr, w2, h2);

		return resp;
	}
	
	/**
	 * Returns half (or almost) widths and heights. Used in quad draw.
	 * 
	 * @param svp
	 * @return [0] = w1, [1] = w2, [3] = h1, [4] = h2
	 */
	public static int[] getWidthsHeights(SimpleVP svp) {
		
		int w = svp.getImageWidth();
		int h = svp.getImageHeight();

		int w1 = 0;
		int w2 = 0;
		int h1 = 0;
		int h2 = 0;
		
		// width
		if ((w % 2) == 0) { // even
			w1 = w / 2;
			w2 = w1;
		} else { // odd
			w1 = 1 + (w / 2);
			w2 = w - w1;
		}
		
		// height
		if ((h % 2) == 0) { // even
			h1 = h / 2;
			h2 = h1;
		} else { // odd
			h1 = 1 + (h / 2);
			h2 = h - h1;
		}
		
		int[] resp = new int[4];
		resp[0] = w1;
		resp[1] = w2;
		resp[2] = h1;
		resp[3] = h2;
		return resp;
	}
	
	
}
