/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2006 Prodevelop and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *   Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *   +34 963862235
 *   gvsig@gva.es
 *   http://www.gvsig.gva.es
 *
 *    or
 *
 *   Prodevelop Integracin de Tecnologas SL
 *   Conde Salvatierra de lava , 34-10
 *   46004 Valencia
 *   Spain
 *
 *   +34 963 510 612
 *   +34 963 510 968
 *   gis@prodevelop.es
 *   http://www.prodevelop.es
 *
 *    or
 *
 *   Instituto de Robtica
 *   Apartado de correos 2085
 *   46071 Valencia
 *   (Spain)
 *   
 *   +34 963 543 577
 *   jjordan@robotica.uv.es
 *   http://robotica.uv.es
 *   
 */

package org.gvsig.mobile.extension.base.navigation.functionality;

import java.util.TimerTask;

import org.gvsig.mobile.ui.awt.mapcontrol.MapControl;


/**
 * This is a utility class used to describe the refresh task performed
 * on a regular basis while the pan process is happening.
 * 
 * This object decides when the refresh has to be done, but the amount of the
 * offset used to paint again the image is set in another place.
 * 
 * @author jldominguez
 *
 */
public class PanRefreshTimerTask extends TimerTask {
	
//	private static Logger logger = Logger.getLogger(PanRefreshTimerTask.class);
	
	private int offsetx = 0;
	private int offsety = 0;
	private MapControl mc;
//	public Point2D p;
	
	/**
	 * Constructor.
	 * @param _mc The object that will be notified when a new refreshment
	 * needs to be done.
	 * 
	 */
	public PanRefreshTimerTask(MapControl _mc) {
		mc = _mc;
	}

	public void run() {
//		logger.debug("Running pan refresh task...");
		mc.setOffsetAndRefresh(offsetx, offsety, true);
//		if (p != null)
//		mc.draw(p);
//		mc.repaint();
//		mc.setOffsetAndRefresh(offsetx, offsety, true);
	}

	/**
	 * This method sets a new horizontal offset for the image.
	 *
	 * This object decides when the refresh has to be done, but the amount of the
	 * offset used to paint again the image is set in another place.
	 * 
	 * @param x the new horizontal offset
	 */
	public void setOffSetX(int x) {
		offsetx = x;
	}

	/**
	 * This method sets a new vertical offset for the image.
	 *
	 * This object decides when the refresh has to be done, but the amount of the
	 * offset used to paint again the image is set in another place.
	 * 
	 * @param y the new vertical offset
	 */
	public void setOffSetY(int y) {
		offsety = y;
	}

}
