package org.gvsig.mobile.extension.base.navigation.functionality;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.geom.AffineTransform;
import java.awt.geom.NoninvertibleTransformException;
import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;

import org.gvsig.fmap.dal.feature.FeatureStoreNotification;
import org.gvsig.fmap.geom.Geometry;
import org.gvsig.fmap.geom.GeometryManager;
import org.gvsig.fmap.geom.primitive.Curve;
import org.gvsig.fmap.geom.primitive.GeneralPathX;
import org.gvsig.fmap.geom.primitive.impl.Envelope2D;
import org.gvsig.fmap.mapcontext.ViewPort;
import org.gvsig.fmap.mapcontext.layers.vectorial.GraphicLayer;
import org.gvsig.fmap.mapcontext.rendering.symbols.ISymbol;
import org.gvsig.mobile.core.concepts.Functionality;
import org.gvsig.mobile.core.constants.GvSigMouseEventType;
import org.gvsig.mobile.core.interfaces.IGvSigContext;
import org.gvsig.mobile.core.interfaces.IGvSigMouseEvent;
import org.gvsig.mobile.symbology.legend.AbstractVectorLegend;
import org.gvsig.mobile.ui.awt.mapcontrol.MapControl;
import org.gvsig.tools.observer.Observable;
import org.gvsig.tools.observer.Observer;
import org.osgi.framework.ServiceReference;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class ZoomRectFunc extends Functionality implements Observer {

	protected static final Logger logger = LoggerFactory.getLogger(ZoomRectFunc.class);
	
	private ViewPort viewPort;
	private MapControl canvas;
	private int symbolId = 0;

	private Point2D startp = null;
	private Point2D endp = null;

	double minx = 0;
	double maxx = 0;
	double miny = 0;
	double maxy = 0;

	ServiceReference gvSIGMobileRef;
	GraphicLayer grf;
	GeometryManager gm = null;

	public static final String DRAG_TOOL_RECT = "DragToolRect";

	public ZoomRectFunc() {
		super();
	}

	public void configure() {
		// TODO Auto-generated method stub

	}

	public boolean execution(ServiceReference gvSIGMobileRef) {
		try {
			// TODO: Implement this
			
			gm = org.gvsig.fmap.geom.GeometryLocator.getGeometryManager();

			// super.stop();			
			if (this.canvas == null) {
				IGvSigContext gv = (IGvSigContext) bc
						.getService(gvSIGMobileRef);
				this.canvas = (MapControl) gv.getMapControl();
				this.viewPort = canvas.getViewPort();
				grf = canvas.getMapContext().getGraphicsLayer();
//				grf.getFeatureStore().addObserver(this);
			}
			

            ISymbol sym = org.gvsig.fmap.mapcontext.MapContextLocator
			.getSymbolManager().createSymbol(Geometry.TYPES.SURFACE, Color.red);
			if(sym!=null){
				symbolId = grf.addSymbol(sym);			
			}
			return true;
		} catch (Exception e) {
			logger.error("While executing zoom rect exe: " + e.getMessage());
			e.printStackTrace();
			return false;
		}
	}

	private Point2D getMapPoint(int x, int y, AffineTransform at) {

		Point2D screenp = new Point2D.Double(x, y);
		Point2D mapp = new Point2D.Double(0, 0);
		try {
			at.inverseTransform(screenp, mapp);
		} catch (NoninvertibleTransformException e) {
		}
		return mapp;
	}

	public void onNewMouseEventListened(IGvSigMouseEvent event) {
		int type = event.getID();

		if (type == GvSigMouseEventType.MOUSE_PRESSED) {
			AbstractVectorLegend.setMetricsInSymbol(canvas.getViewPort());
			grf.removeGraphics(DRAG_TOOL_RECT);
			startp = getMapPoint(event.getX(), event.getY(), viewPort
					.getAffineTransform());
		} else

		if (type == GvSigMouseEventType.MOUSE_DRAGGED) {	
			Point2D tmpp = null;
//			//Esto tarda 2 ms
			grf.removeGraphics(DRAG_TOOL_RECT);		
			try {
				tmpp = getMapPoint(event.getX(), event.getY(), viewPort
						.getAffineTransform());
			} catch (Throwable th) {
				th.printStackTrace();
			}
						
			
			// System.out.print(".");
			minx = Math.min(startp.getX(), tmpp.getX());
			maxx = Math.max(startp.getX(), tmpp.getX());
			miny = Math.min(startp.getY(), tmpp.getY());
			maxy = Math.max(startp.getY(), tmpp.getY());

			// System.out.println("(" + minx + ", " + miny + "), (" + maxx +
			// ", "
			// + maxy + ")");

			GeneralPathX gpx = new GeneralPathX();
			gpx.moveTo(minx, miny);
			gpx.lineTo(minx, maxy);
			gpx.lineTo(maxx, maxy);
			gpx.lineTo(maxx, miny);
			gpx.lineTo(minx, miny);

			Curve cur;
			try {
				cur = gm.createCurve(gpx,
						org.gvsig.fmap.geom.Geometry.SUBTYPES.GEOM2D);
				
//				t1 = System.currentTimeMillis();
				grf.addGraphic(DRAG_TOOL_RECT, cur, symbolId);		
//				t2 = System.currentTimeMillis();
//				System.out.println("add graphic"+(t2-t1));
				//Hasta aqu otros 3 ms

//				t1 = System.currentTimeMillis();
				
				// System.out.println(grf.toString());
				
				canvas.refreshAcetate();
				//Unos 60 ms
				
//				t2 = System.currentTimeMillis();
//				System.out.println("refresh acetate "+(t2-t1));
				
			} catch (Exception e) {
				logger.error("While managing mouse event: " + e.getMessage());
				e.printStackTrace();
			}

		} else

		if (type == GvSigMouseEventType.MOUSE_RELEASED && startp != null) {

			grf.removeGraphics(DRAG_TOOL_RECT);
			
//
			endp = getMapPoint(event.getX(), event.getY(), viewPort
					.getAffineTransform());
			minx = Math.min(startp.getX(), endp.getX());
			maxx = Math.max(startp.getX(), endp.getX());
			miny = Math.min(startp.getY(), endp.getY());
			maxy = Math.max(startp.getY(), endp.getY());
			Rectangle2D userr = new Rectangle2D.Double(minx, miny, maxx - minx,
					maxy - miny);

			Dimension dim = viewPort.getImageSize();
			Envelope2D newext = getExtentFromFullExtentAndImageSize(userr, dim);
			ZoomNextEnvelopeHistory.clearNextEnvelopes();
			viewPort.setEnvelope(newext);
			startp = null;
		}
	}

	/**
	 * 
	 * Computes the minimum rectangle with the width/height ratio that contains
	 * the provided extent.
	 * 
	 * @param fext
	 *            the extent to be covered
	 * @param imgdim
	 *            the size of the screen
	 * @return the minimum rectangle with the width/height ratio that contains
	 *         the provided extent.
	 */
	public Envelope2D getExtentFromFullExtentAndImageSize(Rectangle2D fext,
			Dimension imgdim) {
		double ext_ratio = fext.getWidth() / fext.getHeight();
		double img_ratio = (1.0 * imgdim.width) / (1.0 * imgdim.height);

		if (ext_ratio == img_ratio) {
			// same ratio
			return new Envelope2D(fext.getMinX(), fext.getMinY(), fext
					.getMaxX(), fext.getMaxY());
		} else {
			if (ext_ratio > img_ratio) {
				// cinemascope
				double newexth = fext.getWidth() / img_ratio;
				double yoff = 0.5 * (newexth - fext.getHeight());

				return new Envelope2D(fext.getMinX(), fext.getMinY() - yoff,
						fext.getMinX() + fext.getWidth(), fext.getMinY() - yoff
								+ newexth);
			} else {
				// canadian flag
				double newextw = fext.getHeight() * img_ratio;
				double xoff = 0.5 * (newextw - fext.getWidth());

				return new Envelope2D(fext.getMinX() - xoff, fext.getMinY(),
						fext.getMinX() - xoff + newextw, fext.getMinY()
								+ fext.getHeight());
			}
		}
	}

	public void update(Observable observable, Object notification) {
		FeatureStoreNotification d = (FeatureStoreNotification) notification;
		if (d.getType().compareTo(FeatureStoreNotification.BEFORE_STARTEDITING) == 0) {

		} else if (d.getType().compareTo(
				FeatureStoreNotification.AFTER_FINISHEDITING) == 0) {
//			System.out.println("AFTER_FINISHEDITING");
			
		} else if (d.getType().compareTo(FeatureStoreNotification.AFTER_INSERT) == 0) {
//			System.out.println("AFTER_INSERT");
//			System.out.println("Refresh acetate");
			canvas.refreshAcetate();
		} else if (d.getType().compareTo(FeatureStoreNotification.AFTER_DELETE) == 0) {
//			System.out.println("AFTER_DELETE");
//			canvas.repaint();
//			canvas.refreshAcetate();
		}
	}

}
