/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2006 Prodevelop and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *   Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *   +34 963862235
 *   gvsig@gva.es
 *   http://www.gvsig.gva.es
 *
 *    or
 *
 *   Prodevelop Integracin de Tecnologas SL
 *   Conde Salvatierra de lava , 34-10
 *   46004 Valencia
 *   Spain
 *
 *   +34 963 510 612
 *   +34 963 510 968
 *   gis@prodevelop.es
 *   http://www.prodevelop.es
 *
 *    or
 *
 *   Instituto de Robtica
 *   Apartado de correos 2085
 *   46071 Valencia
 *   (Spain)
 *   
 *   +34 963 543 577
 *   jjordan@robotica.uv.es
 *   http://robotica.uv.es
 *   
 */

package org.gvsig.mobile.extension.base.project.dialog;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Insets;
import java.awt.geom.Rectangle2D;
import java.io.File;
import java.util.ArrayList;

import org.gvsig.i18n.Messages;
import org.gvsig.mobile.libProjection.impl.CRSResolutor;
import org.gvsig.mobile.libProjection.interfaces.CRSConstants;
import org.gvsig.mobile.libProjection.interfaces.ICRS;
import org.gvsig.mobile.ui.awt.IWindowInfo;
import org.gvsig.mobile.ui.awt.WindowManager;
import org.gvsig.mobile.ui.awt.mapcontrol.MapControl;
import org.gvsig.mobile.ui.lw.AppSettings;
import org.gvsig.mobile.ui.lw.ColorChooserDialog;
import org.gvsig.mobile.ui.lw.ColorChooserListener;
import org.gvsig.mobile.ui.lw.DefaultFileChooserListener;
import org.gvsig.mobile.ui.lw.DialogConstants;
import org.gvsig.mobile.ui.lw.FileChooserDialog;
import org.gvsig.mobile.ui.lw.FileChooserListener;
import org.gvsig.mobile.ui.lw.OkCancelComposite;
import org.gvsig.mobile.ui.lw.UIConstants;
import org.osgi.framework.Bundle;
import org.slf4j.LoggerFactory;
import org.zaval.lw.LwBorderLayout;
import org.zaval.lw.LwBorderPan;
import org.zaval.lw.LwButton;
import org.zaval.lw.LwCheckbox;
import org.zaval.lw.LwCombo;
import org.zaval.lw.LwCompList;
import org.zaval.lw.LwComponent;
import org.zaval.lw.LwComposite;
import org.zaval.lw.LwConstraints;
import org.zaval.lw.LwContainer;
import org.zaval.lw.LwFlowLayout;
import org.zaval.lw.LwGridLayout;
import org.zaval.lw.LwLabel;
import org.zaval.lw.LwNotebook;
import org.zaval.lw.LwPanel;
import org.zaval.lw.LwSwitchManImpl;
import org.zaval.lw.LwTextField;
import org.zaval.lw.LwTextRender;
import org.zaval.lw.LwToolkit;
import org.zaval.lw.event.LwActionListener;

/**
 * This dialog shows app current settings and allows changes in them: default
 * view settings, startup project and language.
 * 
 * @see SettingsSerializer.prodevelop.gvsig.mobile.settings.AppSettings
 * 
 * @author jldominguez
 * 
 */

public class AppSettingsDialog extends LwPanel implements LwComposite,
		IWindowInfo, LwActionListener, FileChooserListener,
		ColorChooserListener {

	final static private org.slf4j.Logger logger = LoggerFactory.getLogger(AppSettingsDialog.class);
	private LwNotebook mainTabbedPane;

	// tab general ==================================================
	private LwCombo langCombo, sizeCombo;
	// tab view ==================================================
	private LwCombo srsCombo;
	private LwTextField projectPathText;
	private LwButton projectPathButton;
	private LwButton colorChooserButton;
	private LwTextField minXText, maxXText, minYText, maxYText;
	// tab init project ==================================================
	private LwCheckbox initProjectActiveCheck;
	private LwTextField initProjectPathText;
	private LwButton initProjectPathSearchButton, initProjectPathUpdateButton;
	// ==================================================

	private OkCancelComposite okCancelComp;
	private Bundle bundle;

	private MapControl mapControl;

	private ColorChooserDialog colorChoser;

	public boolean catchInput(LwComponent arg0) {
		return false;
	}

	public AppSettingsDialog(Bundle bundle, MapControl mapControl, int w, int h) {
		this.mapControl=mapControl;
		setLwLayout(new LwBorderLayout());
		setSize(w, h);
		this.bundle = bundle;
		
		mainTabbedPane = new LwNotebook(LwToolkit.TOP);
		final LwTextRender tab1 = new LwTextRender("");
		tab1.setFont(DialogConstants.getDefaultFont());
		tab1.getTextModel().setText(Messages.getText("General"));
		
		final LwTextRender tab2 = new LwTextRender("");
		tab2.setFont(DialogConstants.getDefaultFont());
		tab2.getTextModel().setText(Messages.getText("AppSettingsDialog.8"));
		
		final LwTextRender tab3 = new LwTextRender("");
		tab3.setFont(DialogConstants.getDefaultFont());
		tab3.getTextModel().setText(Messages.getText("AppSettingsDialog.13"));		
		
		mainTabbedPane.addPage(tab1,getGeneralSettingsTab());
		mainTabbedPane.addPage(tab2,getViewSettingsTab());
		mainTabbedPane.addPage(tab3,getStartupProjectSettingsTab());
		mainTabbedPane.setBackground(Color.white);
		mainTabbedPane.addSelectionListener(this);
		
		add(LwBorderLayout.CENTER, mainTabbedPane);
		add(LwBorderLayout.SOUTH, getOkCancelComposite());
		
		loadValues();
	}
	
	private void loadValues() {
		
		colorSelected(this.getColorChooserButton(), AppSettings.defaultBgColor);
		
	}

	public String getTitle() {
		return Messages.getText("AppSettingsDialog.0");
	}

	private LwPanel getOkCancelComposite() {
		okCancelComp = new OkCancelComposite(this);
		return okCancelComp;
	}

	private LwComponent getStartupProjectSettingsTab() {
		LwPanel panel = new LwPanel();
		panel.setLwLayout(new LwBorderLayout());
		
		final Insets t = DialogConstants.getTextFieldInsets();		
		
		LwConstraints l = DialogConstants.getTextFieldConstraints();
		
		LwConstraints l1 = new LwConstraints();
		l1.insets = t;
		l1.fill = LwToolkit.NONE;
		l1.ax = LwToolkit.LEFT;	
		
		LwConstraints l2 = new LwConstraints();
		l2.insets = new Insets(1,1,1,1);
		l2.fill = LwToolkit.NONE;
		l2.ax = LwToolkit.LEFT;
		
		/**
		 * Left panel with labels
		 */
		LwPanel left = new LwPanel();
		left.setLwLayout(new LwGridLayout(2, 1));

		LwSwitchManImpl tt = new LwSwitchManImpl();
		tt.install(getInitProjectActiveCheck());
		getInitProjectActiveCheck().setSwitchManager(tt);
		tt.addActionListener(this);

		left.add(l2, getInitProjectActiveCheck());
		left.add(l2, DialogConstants.getDefaultLabel(Messages.getText("AppSettingsDialog.10")));
		
		/**
		 * Center panel with textfields
		 */
		LwPanel center = new LwPanel();
		center.setLwLayout(new LwGridLayout(2, 1, LwToolkit.HORIZONTAL));
		
				
		center.add(l1, new LwLabel(""));		
		
		initProjectPathSearchButton = DialogConstants.getButton("carpeta", "carpeta", this);
		initProjectPathUpdateButton = DialogConstants.getButton("pro_set_default", "pro_set_default", this);
		
		LwPanel centerComposite = new LwPanel();
		centerComposite.setLwLayout(new LwBorderLayout());
		
		LwPanel rightComposite = new LwPanel();
		rightComposite.setLwLayout(new LwFlowLayout());
		rightComposite.add(this.initProjectPathSearchButton);
		rightComposite.add(this.initProjectPathUpdateButton);
		
		centerComposite.add(LwBorderLayout.CENTER, getInitProjectPathText());		
		centerComposite.add(LwBorderLayout.EAST, rightComposite);
		
		center.add(l, centerComposite);
		
		panel.add(LwBorderLayout.WEST, left);
		panel.add(LwBorderLayout.CENTER, center);

		return panel;
	}
	
	private LwComponent getInitProjectPathText() {

		if (initProjectPathText == null) {
			initProjectPathText = DialogConstants.getDefaultTextField();
			initProjectPathText.setEditable(false);
		}
		return initProjectPathText;
	}

	private LwCheckbox getInitProjectActiveCheck() {

		if (initProjectActiveCheck == null) {
			initProjectActiveCheck = new LwCheckbox(DialogConstants.getDefaultLabel(Messages.getText("AppSettingsDialog.9")),LwCheckbox.CHECK);
		}
		return initProjectActiveCheck;
	}

	private LwComponent getViewSettingsTab() {
		LwPanel resp = new LwPanel();
		resp.setLwLayout(new LwBorderLayout());
		
		LwConstraints l = DialogConstants.getTextFieldConstraints();		
		
		LwConstraints l1 = new LwConstraints();
		l1.fill = LwToolkit.NONE;
		l1.ax = LwToolkit.LEFT;
		
		LwConstraints l2 = DialogConstants.getLabelConstraints();
		
		LwPanel topLeft = new LwPanel();
		topLeft.setLwLayout(new LwGridLayout(3, 1));
		
		LwPanel topRight = new LwPanel();
		topRight.setLwLayout(new LwGridLayout(3, 1, LwToolkit.HORIZONTAL));
		
		topLeft.add(l2, DialogConstants.getDefaultLabel(Messages.getText("AppSettingsDialog.1")));
		srsCombo = getSrsCombo();
		topRight.add(l, srsCombo);
		
		LwContainer c = new LwPanel();
		c.setLwLayout(new LwBorderLayout());
		
		topLeft.add(l2, DialogConstants.getDefaultLabel(Messages.getText("AppSettingsDialog.6")));

		c.add(LwBorderLayout.EAST, getSearchProjectPathButton());
		
		getProjectPathtext().setText(AppSettings.defaultSavePath.toString());
		c.add(LwBorderLayout.CENTER, getProjectPathtext());

		topRight.add(l, c);
		
		topLeft.add(l2, DialogConstants.getDefaultLabel(Messages.getText("AppSettingsDialog.5")));

		colorSelected(getColorChooserButton(), AppSettings.defaultBgColor);
		topRight.add(l1, getColorChooserButton());
		
		LwBorderPan center = new LwBorderPan();
		center.add(LwBorderPan.TITLE, DialogConstants.getDefaultLabel(Messages.getText("AppSettingsDialog.2")));

		LwPanel centerLeft = new LwPanel();
		centerLeft.setLwLayout(new LwGridLayout(2, 1));
		
		LwPanel centerRight = new LwPanel();
		centerRight.setLwLayout(new LwGridLayout(2, 1, LwToolkit.HORIZONTAL));
		
		centerLeft.add(l2, DialogConstants.getDefaultLabel(Messages.getText("AppSettingsDialog.3")));
		LwPanel e1 = new LwPanel();
		e1.setLwLayout(new LwGridLayout(1, 2, LwToolkit.HORIZONTAL));
		
		minXText = getExtentText();
		minXText.setText(new Double(AppSettings.defaultBBox.getMinX()).toString());
		e1.add(l, minXText);
		
		maxXText = getExtentText();
		maxXText.setText(new Double(AppSettings.defaultBBox.getMaxX()).toString());
		e1.add(l, maxXText);
		
		centerRight.add(e1);
		
		centerLeft.add(l2, DialogConstants.getDefaultLabel(Messages.getText("AppSettingsDialog.4")));
		LwPanel e2 = new LwPanel();
		e2.setLwLayout(new LwGridLayout(1, 2, LwToolkit.HORIZONTAL));
		
		minYText = getExtentText();
		minYText.setText(new Double(AppSettings.defaultBBox.getMinY()).toString());
		e2.add(l, minYText);
		
		maxYText = getExtentText();
		maxYText.setText(new Double(AppSettings.defaultBBox.getMaxY()).toString());
		e2.add(l, maxYText);

		centerRight.add(l, e2);
		
		LwPanel top = new LwPanel();
		top.setLwLayout(new LwBorderLayout());
		top.add(LwBorderLayout.WEST, topLeft);
		top.add(LwBorderLayout.CENTER, topRight);
		
		LwPanel cent = new LwPanel();
		cent.setLwLayout(new LwBorderLayout());
		cent.add(LwBorderLayout.WEST, centerLeft);
		cent.add(LwBorderLayout.CENTER, centerRight);
		
		center.add(LwBorderPan.CENTER, cent);
		
		resp.add(LwBorderLayout.NORTH, top);
		resp.add(LwBorderLayout.CENTER, center);

		return resp;
	}	

	private LwTextField getExtentText() {
		LwTextField resp = DialogConstants.getDefaultTextField();		
		resp.setPSSize(20, -1);
		return resp;
	}	

	private LwButton getColorChooserButton() {
		
		if (colorChooserButton == null) {
			colorChooserButton = DialogConstants.getButton("", "", this);
			colorChooserButton.addActionListener(this);
			colorChooserButton.getViewMan(true).setBorder("br.plain");
			colorChooserButton.setOpaque(true);
			int h = getSrsCombo().getPreferredSize().height;
			colorChooserButton.setSize(h,h);
			colorChooserButton.setPSSize(h,h);
		}
		return colorChooserButton;
	}
	
	private LwButton getSearchProjectPathButton() {
		
		if (projectPathButton == null) {
			projectPathButton = DialogConstants.getButton("carpeta", "carpeta", this);		
		}
		return projectPathButton;
	}

	private LwTextField getProjectPathtext() {
		if (projectPathText == null) {
			projectPathText = DialogConstants.getDefaultTextField();
			projectPathText.setEditable(false);
		}
		return projectPathText;
	}

	
	private LwCombo getSrsCombo() {
		if (srsCombo == null) {
			srsCombo = new LwCombo();
			LwCompList l = new LwCompList();
			int actualProjection=0;
			
			ArrayList crsl = CRSResolutor.getCRSs(bundle.getBundleContext());
			ICRS icr = null;
			String abrev;
			for (int i=0; i<crsl.size(); i++) {
				icr = (ICRS) crsl.get(i);
				abrev = (String) icr.getProps().get(CRSConstants.CRS_ABREV);
				l.addItem(DialogConstants.getDefaultLabel(abrev));
				if (AppSettings.defaultSRS.compareToIgnoreCase(abrev)==0){
					actualProjection=i;
				}
			}
			l.select(actualProjection);
			srsCombo.setList(l);
		}
		
		return srsCombo;
	}

	private LwComponent getGeneralSettingsTab() {
		LwPanel resp = new LwPanel();

		resp.setLwLayout(new LwBorderLayout());

		LwConstraints cons = DialogConstants.getTextFieldConstraints();
		LwConstraints l = DialogConstants.getLabelConstraints();

		LwPanel left = new LwPanel();
		left.setLwLayout(new LwGridLayout(2, 1));
		LwLabel langLabel = DialogConstants.getDefaultLabel(Messages.getText("Language"));
		left.add(l, langLabel);
		LwLabel sizeLabel = DialogConstants.getDefaultLabel(Messages.getText("Iconos"));
		left.add(l, sizeLabel);

		LwPanel right = new LwPanel();
		right.setLwLayout(new LwGridLayout(2, 1, LwToolkit.HORIZONTAL));
		
		right.add(cons, getLanguageCombo());
		right.add(cons, getSizeCombo());
		
		resp.add(LwBorderLayout.WEST, left);
		resp.add(LwBorderLayout.CENTER, right);
		return resp;
	}

	private LwCombo getSizeCombo() {

		if (sizeCombo == null) {
			sizeCombo = new LwCombo();
			LwCompList list = new LwCompList();
			int actualIconsSize=0;
			list.addItem(DialogConstants.getDefaultLabel(Messages.getText("24x24")));
			list.addItem(DialogConstants.getDefaultLabel(Messages.getText("48x48")));
			if (AppSettings.size_Factor.compareTo("")==0)
				AppSettings.size_Factor=AppSettings.defSizeFactor;
			if (AppSettings.size_Factor.compareTo("24x24")==0){
				actualIconsSize=0;	
			}else{
				actualIconsSize=1;
			}
			list.select(actualIconsSize);
			sizeCombo.setList(list);
		}
		return sizeCombo;
	}

	private LwCombo getLanguageCombo() {

		if (langCombo == null) {
			langCombo = new LwCombo();
			LwCompList list = new LwCompList();
			int actualLanguage = 0;
			try {
				String[] available_lang = AppSettings.getAvailableLanguages();
				for (int i = 0; i < available_lang.length; i++) {
					list.addItem(DialogConstants.getDefaultLabel(Messages.getText(available_lang[i])));
					if (AppSettings.localeLanguage.compareTo(available_lang[i])==0){
						actualLanguage=i;	
					}
				}
			} catch (Exception ex) {
				ex.printStackTrace();
			}			
			if(list.count() == 0){
				list.addItem(DialogConstants.getDefaultLabel(""));
			}
			// list.select(0);
			list.select(actualLanguage);
			langCombo.setList(list);
		}
		return langCombo;
	}

	public boolean isMaximized() {

		return true;
	}

	public Dimension getDesiredSize() {

		return null;
	}

	public void actionPerformed(Object src, Object data) {

		if (src == projectPathButton) {
			final FileChooserDialog f =
				new FileChooserDialog(this, null, new String[][]{UIConstants.FILTER_ALL, UIConstants.FILTER_GVM}, FileChooserDialog.DIALOG_TYPE_SELECT);
			f.setTitle(Messages.getText("AppSettingsDialog.19"));
			f.showDialog();
		} else if (src == getColorChooserButton()) {
			colorChoser = new ColorChooserDialog((LwComponent) src, this);
			colorChoser.showDialog();
		} else if (src == okCancelComp.getCancelButton()) {
			WindowManager.getInstance().closeActivePanel();
		} else if (src instanceof LwSwitchManImpl) {
			this.initProjectPathText.setEnabled(((LwSwitchManImpl) src)
					.getState(this.initProjectActiveCheck));
		} else if (src == this.initProjectPathSearchButton) {
			final FileChooserDialog f =
				new FileChooserDialog(new DefaultFileChooserListener(this.initProjectPathText), null, null, FileChooserDialog.DIALOG_TYPE_SELECT);
			f.setTitle(Messages.getText("AppSettingsDialog.20"));
			f.showDialog();
		}else if (src == okCancelComp.getOkButton()) {
			//Get Combo values
			AppSettings.localeLanguage = ((LwLabel) langCombo.getList().getSelected()).getText();
			AppSettings.size_Factor = ((LwLabel) sizeCombo.getList().getSelected()).getText();
			AppSettings.defaultSRS = ((LwLabel) srsCombo.getList().getSelected()).getText();
			AppSettings.defaultSavePath = new File(projectPathText.getText());
			Color c = getColorChooserButton().getBackground();
			AppSettings.defaultBgColor = c;
			Rectangle2D newDefaultExtent = new Rectangle2D.Double();
			
			try {
				double minx = Double.parseDouble(minXText.getText());
				double miny = Double.parseDouble(minYText.getText());
				double maxx = Double.parseDouble(maxXText.getText());
				double maxy = Double.parseDouble(maxYText.getText());
				newDefaultExtent.setFrameFromDiagonal(
						minx,miny,maxx,maxy);
				newDefaultExtent = new Rectangle2D.Double(minx,miny,maxx-minx,maxy-miny);
				AppSettings.defaultBBox=newDefaultExtent;
			} catch (Exception ex) {
				logger.warn("Bad extent. Set to default");
				newDefaultExtent.setFrameFromDiagonal(-180,-90,180,90);
				newDefaultExtent = new Rectangle2D.Double(-180,-90,360,180);
				AppSettings.defaultBBox=newDefaultExtent;
			}
			// setting Map Control Changes
			mapControl.setBackColor(AppSettings.defaultBgColor);
			mapControl.repaint();
			
			AppSettings.store(new AppSettings());
			
			WindowManager.getInstance().closeActivePanel();
		}else if (src == okCancelComp.getCancelButton()) {
			WindowManager.getInstance().closeActivePanel();
		}
	}

	public void fileSelected(String path) {
		this.projectPathText.setText(path);
	}

	public void colorSelected(LwComponent ref_comp, Color c) {
		ref_comp.setBackground(c);		
	}
}
