package org.gvsig.mobile.extension.base.util;

import java.text.SimpleDateFormat;
import java.util.Date;

import org.gvsig.fmap.dal.exception.DataException;
import org.gvsig.fmap.dal.feature.Feature;
import org.gvsig.fmap.dal.feature.FeatureStore;
import org.gvsig.fmap.dal.feature.FeatureType;
import org.gvsig.fmap.geom.Geometry;
import org.gvsig.fmap.geom.GeometryLocator;
import org.gvsig.fmap.geom.GeometryManager;
import org.gvsig.fmap.geom.primitive.Curve;
import org.gvsig.fmap.geom.primitive.GeneralPathX;
import org.gvsig.fmap.mapcontext.layers.vectorial.FLyrVect;
import org.gvsig.tools.dataTypes.DataTypes;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.zaval.lw.LwAList;
import org.zaval.lw.LwLabel;

public class DataUtils {
	
	private static Logger logger = LoggerFactory.getLogger(DataUtils.class);
	
	private static SimpleDateFormat dateFormat = new SimpleDateFormat();
	private static GeometryManager gm = null;

	private static GeometryManager getGM() {
		if (gm == null) {
			gm = GeometryLocator.getGeometryManager();
			if (gm == null) {
				logger.error("Unable to get Geometry Manager!!");
			}
		}
		return gm;
	}

	public static boolean isNumericType(int t) {
		return t == DataTypes.DOUBLE || t == DataTypes.FLOAT
		 || t == DataTypes.INT || t == DataTypes.LONG
		 || t == DataTypes.BYTE;
	}
	
	public static String getNameOfNthAttribute(FLyrVect lyr, int n) {
		
		FeatureType ft = null;
		FeatureStore fs = null;
		try {
			fs = lyr.getFeatureStore();
			ft = fs.getDefaultFeatureType();
		} catch (DataException e) {
			logger.error("While getting FeatureType: " + e.getMessage());
			return "";
		}
		String att_name = ft.getAttributeDescriptor(n).getName();
		return att_name;
	}
	
	
	public static double getNumericValue(Feature feat, int fieldInd, int fieldType) throws Exception {
		
		if (fieldType == DataTypes.INT) {
			return feat.getInt(fieldInd);
		} else {
			if (fieldType == DataTypes.DOUBLE) {
				return feat.getDouble(fieldInd);
			} else {
				if (fieldType == DataTypes.BYTE) {
					return feat.getByte(fieldInd);
				} else {
					if (fieldType == DataTypes.FLOAT) {
						return feat.getFloat(fieldInd);
					} else {
						if (fieldType == DataTypes.LONG) {
							return feat.getLong(fieldInd);
						} else {
							throw new Exception("Unknown (numeric?) type: " + fieldType);
						}
					}
				}
			}
		}
	}
	
	
	public static void setSelected(LwAList list, String str) {
		
		int len = list.count();
		for (int i=0; i<len; i++) {
			LwLabel lbl = (LwLabel) list.getItem(i); 
			if (lbl.getText().compareToIgnoreCase(str) == 0) {
				list.select(i);
				return;
			}
		}
		logger.error("Did not find item in list: " + str);
	}
	
	public static String fieldValueToString(Object v) {
		
		if (v instanceof String) {
			return (String) v;
		} else {
			if (v instanceof Integer) {
				return Integer.toString(((Integer) v).intValue());
			} else {
				if (v instanceof Double) {
					return Double.toString(((Double) v).doubleValue());
				} else {
					if (v instanceof Byte) {
						return "" + ((Byte) v).intValue(); 
					} else {
						if (v instanceof Float) {
							return Float.toString(((Float) v).floatValue());
						} else {
							if (v instanceof Long) {
								return "" + ((Long) v).longValue();
							} else {
								if (v instanceof Short) {
									return Short.toString(((Short) v).shortValue());
								} else {
									if (v instanceof Boolean) {
										return ((Boolean) v).booleanValue() ? "true" : "false";
									} else {
										if (v instanceof Date) {
											return dateFormat.format((Date) v);
										} else {
											logger.warn("Unexpected class: " + v.getClass().getName());
											logger.warn("Returned toString() = " + v.toString());
											return v.toString();
										}
									}
								}
							}
						}
					}
				}
			}
		}
	}
	
	public static Geometry getSurroundingRect(double x, double y, double tol) {
		
		if (getGM() == null) {
			return null;
		}
		
		double minx = x-tol;
		double miny = y-tol;
		double maxx = x+tol;
		double maxy = y+tol;
		
		GeneralPathX gpx = new GeneralPathX();
		gpx.moveTo(minx, miny);
		gpx.lineTo(minx, maxy);
		gpx.lineTo(maxx, maxy);
		gpx.lineTo(maxx, miny);
		gpx.lineTo(minx, miny);

		Curve cur = null;
		try {
			cur = gm.createCurve(gpx,org.gvsig.fmap.geom.Geometry.SUBTYPES.GEOM2D);
		} catch (Exception ex) {
			logger.error("While creating surr. rect: " + ex.getMessage());
		}
		return cur;
		
	}

}
