package org.gvsig.mobile.extension.editing.dialog;

import java.awt.Color;
import java.util.ArrayList;

import org.gvsig.i18n.Messages;
import org.gvsig.mobile.ui.awt.WindowManager;
import org.gvsig.mobile.ui.awt.mapcontrol.MapControlLocator;
import org.gvsig.mobile.ui.awt.mapcontrol.MapControlManager;
import org.gvsig.mobile.ui.awt.mapcontrol.snapping.ISnapper;
import org.gvsig.mobile.ui.lw.OkCancelDialog;
import org.osgi.framework.BundleContext;
import org.zaval.data.Matrix;
import org.zaval.lw.LwBorderLayout;
import org.zaval.lw.LwCanvas;
import org.zaval.lw.LwCheckbox;
import org.zaval.lw.LwCombo;
import org.zaval.lw.LwContainer;
import org.zaval.lw.LwFlowLayout;
import org.zaval.lw.LwLabel;
import org.zaval.lw.LwList;
import org.zaval.lw.LwPanel;
import org.zaval.lw.LwScrollPan;
import org.zaval.lw.LwTextField;
import org.zaval.lw.LwToolkit;
import org.zaval.lw.grid.LwGrid;
import org.zaval.lw.grid.LwGridCaption;
import org.zaval.lw.misc.LwGridCompEditor;
import org.zaval.lw.misc.LwGridCompView;

/**
 * @author Vicente Caballero Navarro
 */
public class SnappingSelectorDialog extends OkCancelDialog {
    private LwGrid      gSnapping;
    private LwTextField tolerance;
    private ArrayList properties=new ArrayList();
    
    public SnappingSelectorDialog(boolean scrollable, BundleContext bc) {
        super(scrollable, bc);
        this.getCancelButton().setVisible(false);
        this.getCancelButton().setVisible(true);
        super.add(LwBorderLayout.CENTER, this.buildMainPanel());
    }
    
    public void actionPerformed(Object src, Object data) {
        if (src == this.getOkButton()) {
        	MapControlManager mapControlManager=MapControlLocator.getMapControlManager();
            try{
        		int tol=Integer.parseInt(tolerance.getText());
        		mapControlManager.setTolerance(tol);
            }catch (Exception e) {
				e.printStackTrace();
			}
            for (int i = 0; i < properties.size(); i++) {
				PropertySnapping ps=(PropertySnapping)properties.get(i);
				ISnapper snapper=mapControlManager.getSnapperAt(i);
				snapper.setEnabled(ps.check.getState());
				snapper.setPriority(ps.combo.getList().getSelectedIndex());
            }
        	WindowManager.getInstance().closeActivePanel();
        }else if (src == this.getCancelButton()){
        	WindowManager.getInstance().closeActivePanel();
        }
    }
    
    private LwGrid getGridSnnapings() {
        properties=new ArrayList();
    	Matrix m = new Matrix(2, 2);
        
        gSnapping = new LwGrid(m);
        // if (layers == null)
        // return g;
        LwGridCompEditor editors = new LwGridCompEditor();
        gSnapping.setEditorProvider(editors);
        gSnapping.setViewProvider(new LwGridCompView());
        
        LwGridCaption cap2 = new LwGridCaption(gSnapping);
        cap2.putTitle(0, Messages.getText("Type"));
        cap2.putTitle(1, Messages.getText("Sel."));
        cap2.putTitle(2, Messages.getText("Priority"));
        
        gSnapping.setBackground(Color.white);
        gSnapping.add(LwGrid.TOP_CAPTION_EL, cap2);
        gSnapping.usePsMetric(true);
        // g.getPosController().setOffset(10);
        gSnapping.setColor(LwGrid.SEL2_COLOR, Color.blue);
        gSnapping.setColor(LwGrid.SEL1_COLOR, Color.WHITE);
        gSnapping.setColor(LwGrid.NET_COLOR, Color.WHITE);
        
        gSnapping.setBackground(Color.white);
        MapControlManager mapControlManager=MapControlLocator.getMapControlManager();
        int count=mapControlManager.getSnapperCount();
        for (int i = 0; i < count; i++) {
            ISnapper snapper=mapControlManager.getSnapperAt(i);
            LwLabel c1 = new LwLabel();
            c1.setText(Messages.getText(snapper.getToolTipText()));
            c1.setPSSize(150, 30);
            c1.setBackground(Color.white);
            
            LwCheckbox c2 = new LwCheckbox();
            c2.setPSSize(40, 30);
            c2.setBackground(Color.white);
            c2.setState(snapper.isEnabled());
            
            
            LwCombo c3=new LwCombo();
            LwList list=new LwList();
            for (int j = 0; j < 10; j++) {
            	list.addItem(String.valueOf(j));
            }
            c3.setList(list);
            c3.setPSSize(100, 30);
            c3.getList().select(snapper.getPriority());
            properties.add(new PropertySnapping(c2,c3));
            
            m.put(i, 0, c1);
            m.put(i, 1, c2);
            m.put(i, 2, c3);
            
        }
        return gSnapping;
    }
    
    public LwPanel buildMainPanel() {
    	MapControlManager mapControlManager=MapControlLocator.getMapControlManager();
        LwContainer toolbar1 = new LwPanel();
        toolbar1.setBackground(Color.white);
        toolbar1.setLocation(20, 50);
        LwFlowLayout toolbar1Layout = new LwFlowLayout(LwToolkit.LEFT,
                LwToolkit.TOP, LwToolkit.HORIZONTAL, 1);
        toolbar1.setLwLayout(toolbar1Layout);
        
        LwLabel numTol = new LwLabel(Messages.getText("Tolerance"));
        tolerance = new LwTextField();
        tolerance.setPSSize(100, 30);
        tolerance.setText(String.valueOf(mapControlManager.getTolerance()));
        toolbar1.add(numTol);
        toolbar1.add(tolerance);
        
        LwPanel resp = new LwPanel();
        resp.setLwLayout(new LwBorderLayout());
        gSnapping = getGridSnnapings();
        
        LwGridCaption verSnapping = new LwGridCaption(gSnapping,
                LwToolkit.VERTICAL);
        gSnapping.add(LwGrid.LEFT_CAPTION_EL, verSnapping);
        LwCanvas stubSnapping = new LwCanvas();
        stubSnapping.getViewMan(true).setBorder("br.raised");
        gSnapping.add(LwGrid.STUB_EL, stubSnapping);
        for (int i = 0; i < gSnapping.getModel().getRows(); i++)
            verSnapping.putTitle(i, "" + i);
        
        LwScrollPan sc1 = new LwScrollPan(gSnapping);
        sc1.setIncrements(30, 60, 30, 60);
        resp.add(LwBorderLayout.CENTER, sc1);
        resp.add(LwBorderLayout.SOUTH, toolbar1);
        
        return resp;
    }
    
    public String getTitle() {
        return Messages.getText("SnappingSelectorDialog.0");
    }
    class PropertySnapping{
    	private LwCheckbox check;
    	private LwCombo combo;
    	public PropertySnapping(LwCheckbox check, LwCombo combo) {
			this.check=check;
			this.combo=combo;
		}
    }
}
