package org.gvsig.mobile.extension.editing.functionality;

import java.awt.geom.Point2D;

import org.gvsig.fmap.dal.exception.ReadException;
import org.gvsig.fmap.geom.Geometry;
import org.gvsig.fmap.geom.exception.CreateGeometryException;
import org.gvsig.fmap.geom.primitive.Ellipse;
import org.gvsig.fmap.geom.primitive.GeneralPathX;
import org.gvsig.fmap.geom.primitive.Point;
import org.gvsig.fmap.mapcontext.layers.vectorial.FLyrVect;
import org.gvsig.mobile.core.constants.GvSigMouseEventType;
import org.gvsig.mobile.core.interfaces.IGvSigMouseEvent;

/**
 * @author Vicente Caballero Navarro
 */
public class AddEllipseFunc extends EditingFunctionality implements InsertablePoint{
    protected Point2D startAxis;
    protected Point2D endAxis;
    
    public AddEllipseFunc() {
        super();
    }
    
    public void configure() {
        // TODO Auto-generated method stub
        
    }
    
    public void onNewMouseEventListened(IGvSigMouseEvent event) {
        int type = event.getID();
        
        if (type == GvSigMouseEventType.MOUSE_PRESSED) {
            Point2D point = getHandlerAdjustedPoint();
            addPoint(point);
        }
    }
    
    protected Ellipse createEllipse(Point p1, Point p2,double distance) {
        Ellipse ellipse=null;
        try {
            ellipse = (Ellipse)geomManager.create(Geometry.TYPES.ELLIPSE, Geometry.SUBTYPES.GEOM2D);
            ellipse.setPoints(p1,p2, distance);
            ellipse.getGeneralPath().closePath();
        } catch (CreateGeometryException e) {
            e.printStackTrace();
        }
        return ellipse;
    }
    
    public boolean isApplicable() {
        FLyrVect lv = getEditingLayer();
        try {
            int type = lv.getShapeType();
            if (
            		type == Geometry.TYPES.CURVE ||
            		type == Geometry.TYPES.SURFACE ||
            		type == Geometry.TYPES.MULTICURVE ||
            		type == Geometry.TYPES.MULTISURFACE ||
            		type == Geometry.TYPES.GEOMETRY) {
                return true;
            }
        } catch (ReadException e) {
            e.printStackTrace();
        }
        return false;
    }

	public void addPoint(Point2D point) {
		if (startAxis == null) {
            startAxis = point;
            drawGeometry(createPoint(startAxis.getX(), startAxis.getY()));
        } else if (endAxis == null) {
            endAxis = point;
            GeneralPathX gpx=new GeneralPathX();
            gpx.moveTo(startAxis.getX(), startAxis.getY());
            gpx.lineTo(endAxis.getX(), endAxis.getY());
            try {
				drawGeometry(geomManager.createCurve(gpx, Geometry.SUBTYPES.GEOM2D));
			} catch (CreateGeometryException e) {
				e.printStackTrace();
			}
        }else{
            Point2D middle = new Point2D.Double((startAxis.getX() +
                    endAxis.getX()) / 2, (startAxis.getY() + endAxis.getY()) / 2);
            Point2D third = point;
            double distance = middle.distance(third);
            Ellipse ellipse =
            	createEllipse(
            			createPoint(startAxis.getX(), startAxis.getY()),
            			createPoint(endAxis.getX(), endAxis.getY()),
            			distance);
            
            insertGeometry(ellipse);
            startAxis=null;
            endAxis=null;
        }
	}
}
