package org.gvsig.mobile.extension.editing.functionality;

import java.awt.geom.PathIterator;
import java.awt.geom.Point2D;
import java.util.Iterator;

import org.gvsig.fmap.dal.exception.DataException;
import org.gvsig.fmap.dal.feature.Feature;
import org.gvsig.fmap.dal.feature.FeatureSelection;
import org.gvsig.fmap.geom.Geometry;
import org.gvsig.fmap.geom.exception.CreateGeometryException;
import org.gvsig.fmap.geom.primitive.GeneralPathX;
import org.gvsig.mobile.core.constants.GvSigMouseEventType;
import org.gvsig.mobile.core.interfaces.IGvSigMouseEvent;

/**
 * @author Vicente Caballero Navarro
 */
public class AddFinalVertexFunc extends EditingFunctionality implements InsertablePoint{
    private Feature feature = null;
    
    public AddFinalVertexFunc() {
        super();
    }
    
    public void configure() {
        // TODO Auto-generated method stub
        
    }
    
    public void onNewMouseEventListened(IGvSigMouseEvent event) {
        int type = event.getID();
        if (type == GvSigMouseEventType.MOUSE_PRESSED) {
            Point2D point = getHandlerAdjustedPoint();
            addPoint(point);
        }
    }
    
    public boolean isApplicable() {
        try {
            FeatureSelection selection = getEditingLayer().getFeatureStore()
                    .getFeatureSelection();
            if (selection.getSelectedCount() == 1) {
                Iterator features = selection.iterator();
                feature = (Feature) features.next();
                Geometry geometry = feature.getDefaultGeometry();
                return
                geometry.getType() == Geometry.TYPES.CURVE ||
                geometry.getType() == Geometry.TYPES.MULTICURVE
                ;
            }
        } catch (DataException e) {
            e.printStackTrace();
        }
        return false;
    }
    
    private Geometry addFinalVertex(Geometry geom, Point2D p) {
        GeneralPathX newGp = new GeneralPathX();
        double[] theData = new double[6];
        
        PathIterator theIterator;
        int theType;
        int numParts = 0;
        Point2D firstPoint = null;
        theIterator = geom.getPathIterator(null, getFlatNess()); // ,
        // flatness);
        int numSegmentsAdded = 0;
        while (!theIterator.isDone()) {
            theType = theIterator.currentSegment(theData);
            switch (theType) {
                case PathIterator.SEG_MOVETO:
                    if (numParts == 0) {
                        firstPoint = new Point2D.Double(theData[0], theData[1]);
                    }
                    numParts++;
                    newGp.moveTo(theData[0], theData[1]);
                    numSegmentsAdded++;
                    break;
                
                case PathIterator.SEG_LINETO:
                    newGp.lineTo(theData[0], theData[1]);
                    numSegmentsAdded++;
                    break;
                
                case PathIterator.SEG_QUADTO:
                    newGp
                            .quadTo(theData[0], theData[1], theData[2],
                                    theData[3]);
                    numSegmentsAdded++;
                    break;
                
                case PathIterator.SEG_CUBICTO:
                    newGp.curveTo(theData[0], theData[1], theData[2],
                            theData[3], theData[4], theData[5]);
                    numSegmentsAdded++;
                    break;
                
                case PathIterator.SEG_CLOSE:
                    newGp.closePath();
                    break;
            } // end switch
            
            theIterator.next();
        } // end while loop
        newGp.lineTo(p.getX(), p.getY());
        
        Geometry shp = null;
        switch (geom.getType()) {
            case Geometry.TYPES.POINT: // Tipo punto
                shp = createPoint(firstPoint.getX(), firstPoint.getY());
                break;
            
            case Geometry.TYPES.CURVE:
                try {
                    shp = geomManager.createCurve(newGp,
                            Geometry.SUBTYPES.GEOM2D);
                } catch (CreateGeometryException e) {
                    e.printStackTrace();
                }
                break;
            case Geometry.TYPES.SURFACE:
            case Geometry.TYPES.CIRCLE:
            case Geometry.TYPES.ELLIPSE:
                try {
                    shp = geomManager.createSurface(newGp,
                            Geometry.SUBTYPES.GEOM2D);
                } catch (CreateGeometryException e) {
                    e.printStackTrace();
                }
                break;
        }
        return shp;
    }

	public void addPoint(Point2D point) {
		Geometry geometry = feature.getDefaultGeometry();
        Geometry newGeometry = addFinalVertex(geometry, point);
        updateGeometry(feature, newGeometry);
	}
}
