package org.gvsig.mobile.extension.editing.functionality;

import java.awt.geom.Point2D;

import org.gvsig.fmap.dal.exception.ReadException;
import org.gvsig.fmap.geom.Geometry;
import org.gvsig.fmap.geom.exception.CreateGeometryException;
import org.gvsig.fmap.geom.primitive.GeneralPathX;
import org.gvsig.fmap.mapcontext.layers.vectorial.FLyrVect;
import org.gvsig.mobile.core.constants.GvSigMouseEventType;
import org.gvsig.mobile.core.interfaces.IGvSigMouseEvent;
import org.gvsig.tools.locator.LocatorException;

/**
 * @author Vicente Caballero Navarro
 */
public class AddFreeTraceFunc extends AddPolilineFunc {
    private static double TOL = 4;
    private Point2D       lastPoint;
    
    public AddFreeTraceFunc() {
        super();
    }
    
    public void onNewMouseEventListened(IGvSigMouseEvent event) {
        int type = event.getID();
        if (type == GvSigMouseEventType.MOUSE_DRAGGED) {
        	Point2D point = getHandlerAdjustedPoint();
            addPoint(point);
        }
    }
    
    public void stop() {
        super.stop();
        lastPoint = null;
    }
    
    public boolean isApplicable() {
        FLyrVect lv = getEditingLayer();
        try {
            int type = lv.getShapeType();
            if (
            		type == Geometry.TYPES.CURVE ||
            		type == Geometry.TYPES.MULTICURVE ||
            		type == Geometry.TYPES.GEOMETRY) {
                return true;
            }
        } catch (ReadException e) {
            e.printStackTrace();
        }
        return false;
    }
    
    public void addPoint(Point2D point){
    	if (lastPoint == null || point.distance(lastPoint) > TOL) {
            lastPoint = new Point2D.Double(point.getX(), point.getY());
             points.add(point);
            GeneralPathX gpx = createGPXLine(points);
            try {
            	if (getEditingLayer().getShapeType()==Geometry.TYPES.MULTISURFACE){
            		Point2D p=(Point2D)points.get(0);
            		gpx.lineTo(p.getX(), p.getY());
            		gpx.closePath();
            		geom = geomManager.createSurface(gpx,
            				Geometry.SUBTYPES.GEOM2D);
            	}else{
            		geom = geomManager.createCurve(gpx,
                				Geometry.SUBTYPES.GEOM2D);
               }
            	drawGeometry(geom);
            } catch (LocatorException e) {
            	e.printStackTrace();
            } catch (CreateGeometryException e) {
            	e.printStackTrace();
            } catch (ReadException e) {
            	e.printStackTrace();
            }
        }
    }
}
