package org.gvsig.mobile.extension.editing.functionality;

import java.awt.geom.Point2D;

import org.gvsig.fmap.dal.exception.ReadException;
import org.gvsig.fmap.geom.Geometry;
import org.gvsig.fmap.geom.exception.CreateGeometryException;
import org.gvsig.fmap.geom.primitive.GeneralPathX;
import org.gvsig.fmap.mapcontext.layers.vectorial.FLyrVect;
import org.gvsig.mobile.core.constants.GvSigMouseEventType;
import org.gvsig.mobile.core.interfaces.IGvSigMouseEvent;
import org.gvsig.tools.locator.LocatorException;

/**
 * @author Vicente Caballero Navarro
 */
public class AddRectangleFunc extends EditingFunctionality implements InsertablePoint{
    private Point2D firstPoint = null;
    
    public AddRectangleFunc() {
        super();
        configure();
    }
    
    public void configure() {
    }
    
    public void onNewMouseEventListened(IGvSigMouseEvent event) {
        int type = event.getID();
        
        if (type == GvSigMouseEventType.MOUSE_PRESSED) {
            Point2D point = getHandlerAdjustedPoint();
            addPoint(point);
        }
    }
    
    public boolean isApplicable() {
        FLyrVect lv = getEditingLayer();
        try {
            int type = lv.getShapeType();
            if (
            		type == Geometry.TYPES.CURVE ||
            		type == Geometry.TYPES.SURFACE ||
            		type == Geometry.TYPES.MULTICURVE ||
            		type == Geometry.TYPES.MULTISURFACE ||
            		type == Geometry.TYPES.GEOMETRY) {
                return true;
            }
        } catch (ReadException e) {
            e.printStackTrace();
        }
        return false;
    }

	public void addPoint(Point2D point) {
		 if (firstPoint == null) {
             firstPoint = point;
         } else {
             Point2D secondPoint = point;
             GeneralPathX gpx = new GeneralPathX();
             gpx.moveTo(firstPoint.getX(), firstPoint.getY());
             gpx.lineTo(secondPoint.getX(), firstPoint.getY());
             gpx.lineTo(secondPoint.getX(), secondPoint.getY());
             gpx.lineTo(firstPoint.getX(), secondPoint.getY());
             gpx.lineTo(firstPoint.getX(), firstPoint.getY());
             gpx.closePath();
             try {
                 Geometry geom = geomManager.createSurface(gpx,
                         Geometry.SUBTYPES.GEOM2D);
                 insertGeometry(geom);
             } catch (LocatorException e) {
                 e.printStackTrace();
             } catch (CreateGeometryException e) {
                 e.printStackTrace();
             }
             firstPoint = null;
         }
	}
}
