package org.gvsig.mobile.extension.editing.functionality;

import java.awt.Color;
import java.awt.geom.NoninvertibleTransformException;
import java.awt.geom.Point2D;

import org.gvsig.fmap.dal.exception.DataException;
import org.gvsig.tools.dispose.DisposableIterator; 
import org.gvsig.fmap.dal.feature.EditableFeature;
import org.gvsig.fmap.dal.feature.Feature;
import org.gvsig.fmap.dal.feature.FeatureSet;
import org.gvsig.fmap.dal.feature.FeatureStore;
import org.gvsig.fmap.geom.Geometry;
import org.gvsig.fmap.geom.GeometryLocator;
import org.gvsig.fmap.geom.GeometryManager;
import org.gvsig.fmap.geom.Geometry.SUBTYPES;
import org.gvsig.fmap.geom.exception.CreateEnvelopeException;
import org.gvsig.fmap.geom.exception.CreateGeometryException;
import org.gvsig.fmap.geom.handler.Handler;
import org.gvsig.fmap.geom.primitive.Envelope;
import org.gvsig.fmap.geom.primitive.GeneralPathX;
import org.gvsig.fmap.geom.primitive.Point;
import org.gvsig.fmap.mapcontext.ViewPort;
import org.gvsig.fmap.mapcontext.layers.FLayer;
import org.gvsig.fmap.mapcontext.layers.FLayers;
import org.gvsig.fmap.mapcontext.layers.SpatialCache;
import org.gvsig.fmap.mapcontext.layers.vectorial.FLyrVect;
import org.gvsig.fmap.mapcontext.layers.vectorial.GraphicLayer;
import org.gvsig.fmap.mapcontext.rendering.symbols.ISymbol;
import org.gvsig.i18n.Messages;
import org.gvsig.mobile.core.concepts.Functionality;
import org.gvsig.mobile.core.interfaces.IGvSigContext;
import org.gvsig.mobile.ui.awt.mapcontrol.MapControl;
import org.gvsig.mobile.ui.lw.MessageBox;
import org.osgi.framework.ServiceReference;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * @author Vicente Caballero Navarro
 */
public abstract class EditingFunctionality extends Functionality {
	
	private static final Logger logger = LoggerFactory.getLogger(EditingFunctionality.class);
	
    public static GeometryManager geomManager = null;

    public static int             TOL         = 5;
    public static int             TOLHANDLER  = 10;
    
    private GraphicLayer grf;
	private int symbolId;
	private int symbolIdHandlers;
	private int symbolIdHandler;
	private static String DRAW_GEOMETRY_ID="draw_geometry_id";
	private static String DRAW_HANDLERS_ID="draw_handlers_id";
	private static String DRAW_HANDLER_ID="draw_handler_id";
    
	protected static Point2D selectedHandler;
	protected static Feature selectedFeatureHandler;
	
	
    protected double getFlatNess(){
        return geomManager.getFlatness();
    }
    
    protected Feature insertGeometry(Geometry geometry) {
        FLyrVect lv = getEditingLayer();
        try {
            FeatureStore fs = lv.getFeatureStore();
            EditableFeature eFeature = fs.createNewFeature(true);
            eFeature.setGeometry(fs.getDefaultFeatureType()
                    .getDefaultGeometryAttributeName(), geometry);
            fs.insert(eFeature);
            insertSpatialCache(geometry);
            repaintMap();
            // draw(geometry.cloneGeometry());
            return eFeature;
        } catch (DataException e) {
            e.printStackTrace();
            return null;
        } catch (CreateEnvelopeException e) {
            e.printStackTrace();
            return null;
        }
    }
    protected void repaintMap() {
    	getMapControl().getMapContext().getGraphicsLayer().clearAllGraphics();
    	getMapControl().setDirty(); // extentChanged(null);
    	
	}

	protected Feature insertGeometry(Geometry geometry, Feature feature) {
        FLyrVect lv = getEditingLayer();
        try {
            FeatureStore fs = lv.getFeatureStore();
            EditableFeature eFeature = fs.createNewFeature(fs.getDefaultFeatureType(),feature);
            eFeature.setGeometry(fs.getDefaultFeatureType()
                    .getDefaultGeometryAttributeName(), geometry);
            fs.insert(eFeature);
            insertSpatialCache(geometry);
            repaintMap();
            // draw(geometry.cloneGeometry());
            return eFeature;
        } catch (DataException e) {
            e.printStackTrace();
            return null;
        } catch (CreateEnvelopeException e) {
            e.printStackTrace();
            return null;
        }
    }
    
    protected Feature updateGeometry(Feature feature, Geometry geometry) {
        FLyrVect lv = getEditingLayer();
        try {
            FeatureStore fs = lv.getFeatureStore();
            EditableFeature eFeature = feature.getEditable();
            eFeature.setGeometry(fs.getDefaultFeatureType()
                    .getDefaultGeometryAttributeName(), geometry);
            fs.update(eFeature);
            removeSpatialCache(feature.getDefaultGeometry());
            insertSpatialCache(geometry);
            repaintMap();
            // draw(geometry.cloneGeometry());
            return eFeature;
        } catch (DataException e) {
            e.printStackTrace();
            return null;
        } catch (CreateEnvelopeException e) {
            e.printStackTrace();
            return null;
        }
    }
    
    public boolean execution(ServiceReference gvRef) {
        try {
        	
        	
        	if (!isApplicable()) {
        		MessageBox.showDialog(
        				true, bc, Messages.getText("Invalid_geometry_type"), null,
        				MessageBox.MESSAGE_TYPE_INFO,
        				MessageBox.MESSAGE_ONLY_OK);
        		stop();
        		return false;
        	}


        	geomManager = GeometryLocator.getGeometryManager();
        	
            gvSigMobileRef = gvRef;
            grf = getMapControl().getMapContext().getGraphicsLayer();
            
            ISymbol sym = org.gvsig.fmap.mapcontext.MapContextLocator
				.getSymbolManager().createSymbol(Geometry.TYPES.SURFACE, Color.gray);
            ISymbol symHandlers = org.gvsig.fmap.mapcontext.MapContextLocator
				.getSymbolManager().createSymbol(Geometry.TYPES.SURFACE, Color.orange);
            ISymbol symHandler = org.gvsig.fmap.mapcontext.MapContextLocator
				.getSymbolManager().createSymbol(Geometry.TYPES.SURFACE, Color.red);
            
            // TODO:we have to assign an identifier (but where?)
            symbolId = grf.addSymbol(sym);
            symbolIdHandlers = grf.addSymbol(symHandlers);	
            symbolIdHandler = grf.addSymbol(symHandler);
            return true;
        } catch (Exception e) {
            logger.error("While executing edit func: " + e.getMessage());
            e.printStackTrace();
            return false;
        }
    }
    
    protected ViewPort getViewPort() {
        IGvSigContext gv = (IGvSigContext) bc.getService(gvSigMobileRef);
        return (ViewPort) gv.getViewPort();
    }
    
//    public MapControl getMapControl() {
//        IGvSigContext gv = (IGvSigContext) bc.getService(gvSigMobileRef);
//        return (MapControl) gv.getMapControl();
//    }
    
    protected Point2D getMapPoint(int x, int y) {
        
        Point2D screenp = new Point2D.Double(x, y);
        Point2D mapp = new Point2D.Double(0, 0);
        try {
            getViewPort().getAffineTransform().inverseTransform(screenp, mapp);
        } catch (NoninvertibleTransformException e) {
            logger.error("While getting map point: " + e.getMessage());
        }
        return mapp;
    }
    
    protected FLyrVect getEditingLayer() {
        MapControl mapControl = getMapControl();
        FLayers lyrs = mapControl.getMapContext().getLayers();
        for (int i = 0; i < lyrs.getLayersCount(); i++) {
            FLayer layer = lyrs.getLayer(i);
            if (layer instanceof FLyrVect) {
                if (((FLyrVect) layer).isEditing()) {
                    return (FLyrVect) layer;
                }
            }
        }
        return null;
    }
    
    protected void drawGeometry(Geometry geom) {
    	grf.clearAllGraphics();
    	grf.addGraphic(DRAW_GEOMETRY_ID, geom, symbolId);
    	getMapControl().refreshAcetate();
    }
    
    private void insertSpatialCache(Geometry geom)
            throws CreateEnvelopeException {
        SpatialCache spatialCache = getEditingLayer().getSpatialCache();
        Envelope r = geom.getEnvelope();
        if (geom.getType() == Geometry.TYPES.POINT) {
            r = geomManager.createEnvelope(r.getMinimum(0)-0.5, r.getMinimum(1)-0.5, r
                    .getMinimum(0) + 0.5, r.getMinimum(1) + 0.5, SUBTYPES.GEOM2D);// Rectangle2D.Double(r.getX(),r.getY(),1,1);
        }
        spatialCache.insert(r, geom);
        
    }
    
    private void removeSpatialCache(Geometry geom)
            throws CreateEnvelopeException {
        SpatialCache spatialCache = getEditingLayer().getSpatialCache();
        Envelope r = geom.getEnvelope();
        if (geom.getType() == Geometry.TYPES.POINT) {
            r = geomManager.createEnvelope(r.getMinimum(0)-0.5, r.getMinimum(1)-0.5, r
                    .getMinimum(0) + 0.5, r.getMinimum(1) + 0.5, SUBTYPES.GEOM2D);// Rectangle2D.Double(r.getX(),r.getY(),1,1);
        }
        spatialCache.remove(r, geom);
        
    }
    
    protected Point createPoint(double x, double y) {
        Point point = null;
        try {
            point = (Point) geomManager.createPoint(x, y, Geometry.SUBTYPES.GEOM2D);
        } catch (org.gvsig.fmap.geom.exception.CreateGeometryException e) {
            e.printStackTrace();
        }
        return point;
    }
    
    protected void drawHandlers(FeatureSet set) {
    	try {
    		DisposableIterator iter=set.iterator();
    		grf.clearAllGraphics();
    		GeneralPathX gpx=new GeneralPathX();
    		while (iter.hasNext()) {
    			Feature feature = (Feature) iter.next();
    			Geometry geom=feature.getDefaultGeometry();
    			double middleTOL=getViewPort().toMapDistance(TOLHANDLER)*0.5;
    			Handler[] handlers = geom.getHandlers(Geometry.SELECTHANDLER);
    			for (int i = 0; i < handlers.length; i++) {
    				Point2D p = handlers[i].getPoint();
    				gpx.moveTo(p.getX()-middleTOL, p.getY()-middleTOL);
    				gpx.lineTo(p.getX()+middleTOL, p.getY()-middleTOL);
    				gpx.lineTo(p.getX()+middleTOL, p.getY()+middleTOL);
    				gpx.lineTo(p.getX()-middleTOL, p.getY()+middleTOL);
    				gpx.lineTo(p.getX()-middleTOL, p.getY()-middleTOL);
    				gpx.closePath();
    			}
    		}
    		iter.dispose();
    		Geometry rec=geomManager.createSurface(gpx,Geometry.SUBTYPES.GEOM2D);
    		grf.addGraphic(DRAW_HANDLERS_ID, rec, symbolIdHandlers);
    		getMapControl().refreshAcetate();
    	} catch (CreateGeometryException e) {
    		e.printStackTrace();
    	} catch (DataException e) {
    		e.printStackTrace();
    	}
    }
    public void clearAllGraphics(){
    	grf.clearAllGraphics();
    }
    protected void drawHandler(Point2D p) {
    	grf.removeGraphics(DRAW_HANDLER_ID);
    	try {
    		GeneralPathX gpx=new GeneralPathX();
    		double middleTOL=getViewPort().toMapDistance(TOLHANDLER+10)*0.5;
    		gpx.moveTo(p.getX()-middleTOL, p.getY()-middleTOL);
    		gpx.lineTo(p.getX()+middleTOL, p.getY()-middleTOL);
    		gpx.lineTo(p.getX()+middleTOL, p.getY()+middleTOL);
    		gpx.lineTo(p.getX()-middleTOL, p.getY()+middleTOL);
    		gpx.lineTo(p.getX()-middleTOL, p.getY()-middleTOL);
    		gpx.closePath();
    		Geometry rec=geomManager.createSurface(gpx,Geometry.SUBTYPES.GEOM2D);
    		grf.addGraphic(DRAW_HANDLER_ID, rec, symbolIdHandler);
    		getMapControl().refreshAcetate();
    	} catch (CreateGeometryException e) {
    		e.printStackTrace();
    	} 
    }
    public Point2D getHandlerAdjustedPoint(){
    	return getMapControl().getHandlerAdjustedPoint();
    }
	public Point2D adjustPointToHandler(Point2D p){
		return getMapControl().adjustPointToHandler(p);
	}
}
