package org.gvsig.mobile.extension.editing.functionality;

import java.awt.geom.Point2D;

import org.gvsig.fmap.dal.exception.DataException;
import org.gvsig.tools.dispose.DisposableIterator; 
import org.gvsig.fmap.dal.feature.EditableFeature;
import org.gvsig.fmap.dal.feature.Feature;
import org.gvsig.fmap.dal.feature.FeatureSelection;
import org.gvsig.fmap.dal.feature.FeatureStore;
import org.gvsig.fmap.geom.Geometry;
import org.gvsig.fmap.geom.util.UtilFunctions;
import org.gvsig.mobile.core.constants.GvSigMouseEventType;
import org.gvsig.mobile.core.interfaces.IGvSigMouseEvent;

/**
 * @author Vicente Caballero Navarro
 */
public class ModifyMoveFunc extends SelectionFunc implements InsertablePoint{
    private Point2D firstPoint;
    private Point2D lastPoint;

    public void configure() {
        // TODO Auto-generated method stub
        
    }
    
    public void onNewMouseEventListened(IGvSigMouseEvent event) {
    	int type = event.getID();
    	if (type == GvSigMouseEventType.MOUSE_PRESSED) {
    		Point2D point = getHandlerAdjustedPoint();
    		addPoint(point);
    	}
    }

    public void addPoint(Point2D point) {
    	try{
    		FeatureStore featureStore=getEditingLayer().getFeatureStore();
    		FeatureSelection selection=featureStore.getFeatureSelection();
    		if(selection.isEmpty()){
    			selectWithPoint(point);
    		}else{
    			if (firstPoint==null){
    				firstPoint = point;
    			} else {
    				lastPoint = point;
    				featureStore.beginEditingGroup(getName());
    				try {
    					DisposableIterator iterator = selection.iterator();
    					while (iterator.hasNext()) {
    						Feature feature = (Feature) iterator.next();
    						Geometry ig = (feature.getDefaultGeometry())
    						.cloneGeometry();
    						EditableFeature eFeature = feature.getEditable();
    						if (ig == null) {
    							continue;
    						}
    						UtilFunctions.moveGeom(ig, lastPoint.getX()
    								- firstPoint.getX(), lastPoint.getY()
    								- firstPoint.getY());

    						eFeature.setGeometry(featureStore
    								.getDefaultFeatureType()
    								.getDefaultGeometryAttributeName(), ig);
    						featureStore.update(eFeature);
    					}
    					iterator.dispose();
    					repaintMap();
    				} finally {
    					firstPoint=null;
    					featureStore.endEditingGroup();
    				}
    			}
    		}
    	} catch (DataException e) {
    		e.printStackTrace();
    	}
    }
}
