package org.gvsig.mobile.extension.editing.functionality;

import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;

import org.gvsig.fmap.dal.exception.DataException;
import org.gvsig.tools.dispose.DisposableIterator; 
import org.gvsig.fmap.dal.feature.Feature;
import org.gvsig.fmap.dal.feature.FeatureSelection;
import org.gvsig.fmap.geom.Geometry;
import org.gvsig.fmap.geom.handler.Handler;
import org.gvsig.mobile.core.constants.GvSigMouseEventType;
import org.gvsig.mobile.core.interfaces.IGvSigMouseEvent;
import org.osgi.framework.ServiceReference;

/**
 * @author Vicente Caballero Navarro
 */
public class SelectionVertexFunc extends EditingFunctionality implements SelectablePoint{
    private static FeatureSelection selection       = null;
   
    
    public boolean execution(ServiceReference gvSIGMobileRef) {
        boolean execute = super.execution(gvSIGMobileRef);
        try {
            selection = getEditingLayer().getFeatureStore()
                    .getFeatureSelection();
            drawHandlers(selection);
        } catch (DataException e) {
            e.printStackTrace();
        }
        return execute;
    }
    
    public void configure() {
        // TODO Auto-generated method stub
        
    }
    
    public void onNewMouseEventListened(IGvSigMouseEvent event) {
    	int type = event.getID();
    	if (type == GvSigMouseEventType.MOUSE_PRESSED) {
    		Point2D point = getMapPoint(event.getX(), event.getY());
    		selectWithPoint(point);
    	}
    }

    /**
     * @throws DataException 
     * 
     */
    public void selectWithPoint (Point2D point) {
    	try{
    		double tol=getViewPort().toMapDistance(TOLHANDLER);
    		Rectangle2D rec = new Rectangle2D.Double(point.getX() - tol
    				* 0.5, point.getY() - tol * 0.5, tol, tol);
    		DisposableIterator features = selection.iterator();
    		while (features.hasNext()) {
    			Feature feature = (Feature) features.next();
    			Geometry geometry = feature.getDefaultGeometry();
    			Handler[] handlers = geometry
    			.getHandlers(Geometry.SELECTHANDLER);
    			for (int j = 0; j < handlers.length; j++) {
    				if (rec.contains(handlers[j].getPoint())) {
    					selectedHandler = handlers[j].getPoint();
    					selectedFeatureHandler = feature;
    					drawHandler(selectedHandler);
    				}
    			}
    		}
    		features.dispose();
    		selection.dispose();
    	} catch (DataException e) {
    		e.printStackTrace();
    	}
    }
    
}
