package org.gvsig.mobile.extension.editing.gps.functionality;

import java.awt.geom.Point2D;
import java.util.Properties;

import org.cresques.cts.ICoordTrans;
import org.cresques.cts.IProjection;
import org.gvsig.fmap.crs.CRSFactory;
import org.gvsig.fmap.dal.exception.ReadException;
import org.gvsig.fmap.geom.Geometry;
import org.gvsig.fmap.mapcontext.layers.vectorial.FLyrVect;
import org.gvsig.location.Location;
import org.gvsig.location.LocationListener;
import org.gvsig.location.LocationProvider;
import org.gvsig.location.MovementListener;
import org.gvsig.location.QualifiedCoordinates;
import org.gvsig.mobile.app.MainFrame;
import org.gvsig.mobile.core.concepts.DefaultExclusiveTool;
import org.gvsig.mobile.core.extensions.interfaces.IFunctionality;
import org.gvsig.mobile.core.interfaces.IGvSigContext;
import org.gvsig.mobile.core.interfaces.IGvSigMouseEvent;
import org.gvsig.mobile.extension.editing.functionality.EditingFunctionality;
import org.gvsig.mobile.extension.editing.functionality.InsertablePoint;
import org.gvsig.mobile.gps.GPSConstants;
import org.gvsig.mobile.gps.GPSResolutor;
import org.osgi.framework.ServiceReference;

/**
 * @author Vicente Caballero Navarro
 */
public class AddGPSAutoVertexFunc extends EditingFunctionality implements LocationListener, MovementListener{
    
	public boolean execution(ServiceReference gvRef) {
		boolean b= super.execution(gvRef);
		// If there's a location provider available, subscribe to location updates
		LocationProvider provider = GPSResolutor.getLocationProvider();
		if (provider!=null){
			provider.addLocationListener(this, 1, LocationProvider.PROVIDER_DEFAULT, 
				LocationProvider.PROVIDER_DEFAULT);
			Properties properties=GPSResolutor.getGPSPrefs();
			Double dist=Double.valueOf((String)properties.get(GPSConstants.K_TRK_FILTER_DIST_M));
			Double speed=Double.valueOf((String)properties.get(GPSConstants.K_TRK_FILTER_SPEED_KM_H));
			Double course=Double.valueOf((String)properties.get(GPSConstants.K_TRK_FILTER_COURSE_DEG));
			provider.setMovementListener(this,dist.doubleValue(),speed.doubleValue()/3.6,course.doubleValue());// 100, 10/3.6, 15);
//			provider.setMovementListener(this, 1, LocationProvider.PROVIDER_DEFAULT, 
//				LocationProvider.PROVIDER_DEFAULT);
//			provider.startAveraging(this, LocationProvider.PROVIDER_DEFAULT, 
//				LocationProvider.PROVIDER_DEFAULT);
		
		
//			QualifiedCoordinates qcoord=provider.getLastLocation().getQualifiedCoordinates();
//			addQualifiedCoordinates(qcoord);
		}
		stop();
//		repaintMap();
		return b;
	}
	
	
	
	
    public AddGPSAutoVertexFunc() {
        super();
    }
    
    public void configure() {
        // TODO Auto-generated method stub
        
    }
    
    public void onNewMouseEventListened(IGvSigMouseEvent event) {
        // TODO Auto-generated method stub
        
    }
    
    public boolean isApplicable() {
        FLyrVect lv = getEditingLayer();
        try {
            int type = lv.getShapeType();
            if (type == Geometry.TYPES.CURVE || type == Geometry.TYPES.SURFACE
                    || type == Geometry.TYPES.GEOMETRY) {
                return true;
            }
        } catch (ReadException e) {
            e.printStackTrace();
        }
        return false;
    }




	public void locationUpdated(LocationProvider arg0, Location arg1) {
		// TODO Auto-generated method stub
		
	}




	public void providerStateChanged(LocationProvider arg0, int arg1) {
		// TODO Auto-generated method stub
		
	}




	public void discardedLocation(Location arg0, int arg1) {
		// TODO Auto-generated method stub
		
	}




	public void movingLocation(Location location) {
		QualifiedCoordinates qcoord=location.getQualifiedCoordinates();
		addQualifiedCoordinates(qcoord);
	}




	private void addQualifiedCoordinates(QualifiedCoordinates qcoord) {
		if (qcoord!=null){
			double latitude=qcoord.getLatitude();
			double longitude=qcoord.getLongitude();
//			double altitude=qcoord.getAltitude();

//			Debera devolver en que proyeccin est capturandose el punto desde el GPS, pero no est implementado 
//			y lo pondremos a pin en 4326.
//			String srs=qcoord.getSRS();
			String srs="EPSG:4326";
			IProjection gpsProj=CRSFactory.getCRS(srs);
			IProjection viewProj=getViewPort().getProjection();
			ICoordTrans ct=gpsProj.getCT(viewProj);
			//Hacer la reproyeccin adecuada desde el gps a la vista del punto a insertar
	
			Point2D pOriginal=new Point2D.Double(longitude, latitude);
			Point2D pDest=new Point2D.Double();
			pDest=ct.convert(pOriginal, pDest);
			
			IGvSigContext gv = (IGvSigContext)bc.getService(gvSigMobileRef);
			DefaultExclusiveTool det=(DefaultExclusiveTool)((MainFrame)gv.getMainFrame()).getActiveExclusiveTool();
			IFunctionality func=det.getFuncSolved();
			if (func!=null && func instanceof InsertablePoint )
				((InsertablePoint)func).addPoint(adjustPointToHandler(pDest));
		}
	}
	
}
