package org.gvsig.mobile.extension.gps;

import java.awt.Color;
import java.awt.geom.AffineTransform;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.ArrayList;
import java.util.Iterator;

import org.gvsig.fmap.geom.Geometry;
import org.gvsig.fmap.geom.GeometryManager;
import org.gvsig.fmap.geom.Geometry.SUBTYPES;
import org.gvsig.fmap.geom.exception.CreateGeometryException;
import org.gvsig.fmap.geom.primitive.GeneralPathX;
import org.gvsig.fmap.mapcontext.layers.vectorial.GraphicLayer;
import org.gvsig.fmap.mapcontext.rendering.symbols.ISymbol;
import org.gvsig.location.Location;
import org.gvsig.location.LocationListener;
import org.gvsig.location.LocationProperties;
import org.gvsig.location.LocationProvider;
import org.gvsig.location.QualifiedCoordinates;
import org.gvsig.location.utils.StringUtils;
import org.gvsig.mobile.ui.awt.ImageUtils;
import org.gvsig.mobile.ui.awt.mapcontrol.MapControl;
import org.osgi.framework.Bundle;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

// import com.vividsolutions.jts.io.WKTReader;

/**
 * This class holds information about the GPS cursor.
 * 
 * @author juangui
 * @author Juan Lucas Dominguez Rubio jldominguez at prodevelop.es
 *
 */
public class GPSCursor implements LocationListener, PropertyChangeListener {
//	private ViewPort viewPort;
	private MapControl canvas;

	private GraphicLayer grf;
	private QualifiedCoordinates lastQC = new QualifiedCoordinates(0.0d, 0.0d, 0.0f, Float.NaN, Float.NaN);

	/*
	 * Is the cursor visible? Default true.
	 */
	private boolean visible = true;
	
	private static Geometry[] gpsArrow = null;
	private static Color[] gpsArrowColor = null;
	private static int[] gpsArrowSymbolId = null;
	
	private long lastCourseTime = 0;
	private double currentCourse = 0;
	public static final long TIME_ALLOWED_WITHOUT_COURSE = 10 * 1000; // milisecs

	private static final String GPS_CURSOR_ID = "GPSCursor";
	private static final String GPS_TRACK = "LineTrack";
	private static Color GPS_TRACK_COLOR = Color.BLUE;
    
	private static final Logger logger = LoggerFactory.getLogger(GPSCursor.class);

	GeometryManager gm = org.gvsig.fmap.geom.GeometryLocator.getGeometryManager();
	
	private QualifiedCoordinates firstPoint = null;
	private QualifiedCoordinates secondPoint = null;
	
	private int symbolId;
	private ArrayList gpstrack;
	boolean fin = false;
	

	public GPSCursor(MapControl canvas, Bundle b) {
		this.canvas = canvas;
		readGpsArrow(b);
		this.grf = canvas.getMapContext().getGraphicsLayer();
		
		int len = gpsArrowColor.length;
		ISymbol cursorSym = null;
		ISymbol simbolo = null;
		
		for (int i=0; i<len; i++) {
			cursorSym = org.gvsig.fmap.mapcontext.MapContextLocator.getSymbolManager().
			createSymbol(Geometry.TYPES.SURFACE, gpsArrowColor[i]);
			if (cursorSym != null) {
				cursorSym.setColor(gpsArrowColor[i]);
				gpsArrowSymbolId[i] = grf.addSymbol(cursorSym);			
			}
		}

		simbolo = org.gvsig.fmap.mapcontext.MapContextLocator.getSymbolManager().createSymbol(Geometry.TYPES.CURVE, GPS_TRACK_COLOR);
		if (simbolo != null) {
			symbolId = grf.addSymbol(simbolo);
		}

		gpstrack = new ArrayList();
	}

	/**
	 * Set cursor visibility
	 */
	public void setVisible(boolean visible) {
		this.visible = visible;
		if (!visible) {
			// Remove cursor cursor from graphics layer
			int len = 3;
			if (gpsArrow != null) {
				len = gpsArrow.length;
			} else {
				logger.error("Unexpected NULL: gpsArrow");
			}
			for (int i=0; i<len; i++) {
				grf.removeGraphics(GPS_CURSOR_ID + i);
			}
			
			removeTrack();
			
			canvas.refreshAcetate();
		}else{
			Iterator it_gpstrack = gpstrack.iterator();
			while( it_gpstrack.hasNext() )
			{
		        grf.addGraphic(GPS_TRACK, (Geometry)it_gpstrack.next(), symbolId);				
			}
		}
	}

	/**
	 * Location listener callback for location updates
	 */
	public void locationUpdated(LocationProvider provider, Location location) {
		if (visible) {
			updateLocationInAcetate(location);
			drawTrack();
			
			canvas.refreshAcetate();		
		}
	}

	/**
	 * Location listener callback for location provider state changes
	 */
	public void providerStateChanged(LocationProvider provider, int newState) {
		String state;
		switch (newState) {
		case LocationProvider.AVAILABLE: state = "AVAILABLE"; break;
		case LocationProvider.OUT_OF_SERVICE: state = "OUT OF SERVICE"; break;
		default: state = "TEMPORARY UNAVAILABLE"; break;
		}
		logger.debug("New location provider state: " + state);
	}

	public void propertyChange(PropertyChangeEvent e) {
		
		// expected COURSE and COURSE_ESTIMATED only
		String pname = e.getPropertyName();
		if (
				pname.compareToIgnoreCase(LocationProperties.COURSE) != 0 &&
				pname.compareToIgnoreCase(LocationProperties.COURSE_ESTIMATED) != 0
				) {
			return;
		}
		// ===========================================
		
		Object v = e.getNewValue();
		Double vd = new Double(0);
		if (v instanceof Double) {
			vd = (Double) v;
		} else {
			logger.warn("Unexpected value type: " + v.getClass().getName());
			logger.warn("(property is " + e.getPropertyName() + ")");
			return;
		}
		
		if (e.getPropertyName().compareToIgnoreCase(LocationProperties.COURSE) == 0) {
			
			lastCourseTime = System.currentTimeMillis();
			setCourse(vd);
			
		} else {
			if (e.getPropertyName().compareToIgnoreCase(LocationProperties.COURSE_ESTIMATED) == 0) {
				if ((System.currentTimeMillis() - lastCourseTime) > TIME_ALLOWED_WITHOUT_COURSE) {
					setCourse(vd);
				}
			}
		}
	}
	
	private void setCourse(Double vd) {
		currentCourse = vd.doubleValue();
		if (visible) {
			updateRotationInAcetate();
			canvas.refreshAcetate();		
		}
	}

	private void updateLocationInAcetate(Location location) {
		
		// Remove old cursor
		int len = gpsArrow.length;
		for (int i=0; i<len; i++) {
			grf.removeGraphics(GPS_CURSOR_ID + i);
		}
		
		QualifiedCoordinates qc = location.getQualifiedCoordinates();
		if (qc != null) {
			Geometry[] tmpArrow = rotatedScaledGpsArrow(currentCourse, lastQC.getLongitude(), lastQC.getLatitude());
			for (int i=0; i<len; i++) {
				grf.addGraphic(GPS_CURSOR_ID + i, tmpArrow[i], gpsArrowSymbolId[i]);
			}
			lastQC = qc;
		} else {
			// Draw cursor on last coordinates in different color
		}
	}

	private void updateRotationInAcetate() {
		
		if (lastQC != null) {
			// Remove old cursor
			int len = gpsArrow.length;
			for (int i=0; i<len; i++) {
				grf.removeGraphics(GPS_CURSOR_ID + i);
			}
			
			Geometry[] tmpArrow = rotatedScaledGpsArrow(currentCourse, lastQC.getLongitude(), lastQC.getLatitude());
			for (int i=0; i<len; i++) {
				grf.addGraphic(GPS_CURSOR_ID + i, tmpArrow[i], gpsArrowSymbolId[i]);
			}
		}
	}
	
	private Geometry[] rotatedScaledGpsArrow(double course, double lon, double lat) {
		
		int len = gpsArrow.length;
		Geometry[] resp = new Geometry[len];
		
		AffineTransform rot = AffineTransform.getRotateInstance(-course*Math.PI/180.0, 0, 0);
		
		double vpsc = 1;
		
		try {
			vpsc = canvas.getViewPort().getAffineTransform().getScaleX();
		} catch (Exception ex) {
			logger.warn("While getting VP AT SC: " + ex.getMessage());
		}
		
		AffineTransform sca = AffineTransform.getScaleInstance(1.0/vpsc, 1.0/vpsc);
		AffineTransform mov = AffineTransform.getTranslateInstance(lon,lat);
		
		sca.preConcatenate(rot);
		sca.preConcatenate(mov);
		
		Geometry geom = null;
		for (int i=0; i<len; i++) {
			geom = gpsArrow[i].cloneGeometry();
			geom.transform(sca);
			resp[i] = geom; 
		}
		return resp;
	}
	
	private void readGpsArrow(Bundle bu) {
		
		if (gpsArrow != null) {
			// already done
			return;
		}
		
		byte[] arrowb = ImageUtils.getFileInIconsFile(bu, "icons", "gpsarrow/arrow_wkt.txt");
		String arrowstr = new String(arrowb);
		
		String[] desc = null;
		
		if (arrowstr.indexOf("\r\n") != -1) {
			// windows
			desc = StringUtils.splitString(arrowstr, "\r\n");
		} else {
			if (arrowstr.indexOf("\n\r") != -1) {
				// ?
				desc = StringUtils.splitString(arrowstr, "\n\r");
			} else {
				// linux
				desc = StringUtils.splitString(arrowstr, "\n");
			}
		}
		
		int len = desc.length;
		
		gpsArrow = new Geometry[len/2];
		gpsArrowColor = new Color[len/2];
		gpsArrowSymbolId = new int[len/2];
		
		for (int i=0; i<len; i=i+2) {
			try {
				gpsArrow[i/2] = gm.createFrom(desc[i],"EPSG:4326");
			} catch (Exception e) {
				logger.error("While parsing WKT for GPS arrow: " + e.getMessage());
			}
			gpsArrowColor[i/2] = strToColor(desc[i+1]);
		}
		
		int vpw = canvas.getViewPort().getImageWidth();
		int vph = canvas.getViewPort().getImageHeight();
		
		double maxw = 470.0;
		if (vpw < maxw || vph < maxw) {
			scaleGeoms(gpsArrow, 1.0*Math.min(vpw,vph)/maxw);
		}
	}

	private void scaleGeoms(Geometry[] gg, double d) {
		
		int len = gg.length;
		AffineTransform at = AffineTransform.getScaleInstance(d,d);
		for (int i=0; i<len; i++) {
			gg[i].transform(at);
		}
	}

	private Color strToColor(String str) {
		
		String[] cha = StringUtils.splitString(str, ",");
		if (cha.length < 3) {
			return Color.RED;
		} else {
			
			int r,b,g;
			try { r = Integer.parseInt(cha[0]); } catch (Exception ex) { r = 0; } 
			try { g = Integer.parseInt(cha[1]); } catch (Exception ex) { g = 0; } 
			try { b = Integer.parseInt(cha[2]); } catch (Exception ex) { b = 0; } 
			return new Color(r,g,b);
		}
	}

	/**
	 * Function that draws actual track
	 */
	private void drawTrack()
	{
		if(firstPoint == null)
		{
			firstPoint = new QualifiedCoordinates(0.0d, 0.0d, 0.0f, Float.NaN, Float.NaN);
			secondPoint = new QualifiedCoordinates(0.0d, 0.0d, 0.0f, Float.NaN, Float.NaN);
			
			firstPoint.setLatitude(lastQC.getLatitude());
			firstPoint.setLongitude(lastQC.getLongitude());
		}
		else 
		{
			firstPoint.setLatitude(secondPoint.getLatitude());
			firstPoint.setLongitude(secondPoint.getLongitude());
		}
		
		secondPoint.setLatitude(lastQC.getLatitude());
		secondPoint.setLongitude(lastQC.getLongitude());
		
        try 
        {	
        	if(!((firstPoint.getLatitude() == secondPoint.getLatitude()) && (firstPoint.getLongitude() == secondPoint.getLongitude())) )
        	{
				GeneralPathX gpx = new GeneralPathX();
		        gpx.moveTo(firstPoint.getLongitude(), firstPoint.getLatitude());
		        gpx.lineTo(secondPoint.getLongitude(), secondPoint.getLatitude());
		        Geometry aux = null;
	
				aux = gm.createCurve(gpx, SUBTYPES.GEOM2D);
				
				gpstrack.add(aux);
		        grf.addGraphic(GPS_TRACK, aux, symbolId);
        	}
        } catch (CreateGeometryException e) 
        {
        	logger.error("While creating geometry for track of gps: " + e.getMessage());
        	e.printStackTrace();
        }  
	}
	
	/**
	 * Function that removes the track generated by the cursor. 
	 */
	public void removeTrack() 
	{
		grf.removeGraphics(GPS_TRACK);
		canvas.refreshAcetate();
	}
}
