/* gvSIG. Sistema de Información Geográfica de la Generalitat Valenciana

 *
 * Copyright (C) 2006 Prodevelop and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *   Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibáñez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *   +34 963862235
 *   gvsig@gva.es
 *   http://www.gvsig.gva.es
 *
 *    or
 *
 *   Prodevelop Integración de Tecnologías SL
 *   Conde Salvatierra de Álava , 34-10
 *   46004 Valencia
 *   Spain
 *
 *   +34 963 510 612
 *   +34 963 510 968
 *   gis@prodevelop.es
 *   http://www.prodevelop.es
 *
 *    or
 *
 *   Instituto de Robótica
 *   Apartado de correos 2085
 *   46071 Valencia
 *   (Spain)
 *   
 *   +34 963 543 577
 *   jjordan@robotica.uv.es
 *   http://robotica.uv.es
 *   
 */

package org.gvsig.mobile.extension.gps.dialog;

import java.util.Properties;
import java.lang.String;

import org.gvsig.i18n.Messages;
import org.gvsig.location.comm.BluetoothDevice;
import org.gvsig.location.comm.BluetoothDeviceDiscover;
import org.gvsig.mobile.extension.gps.PrefsListener;
import org.gvsig.mobile.gps.GPSConstants;
import org.gvsig.mobile.ui.lw.DialogConstants;
import org.osgi.framework.BundleContext;
import org.zaval.lw.LwAList;
import org.zaval.lw.LwBorderLayout;
import org.zaval.lw.LwButton;
import org.zaval.lw.LwCombo;
import org.zaval.lw.LwCompList;
import org.zaval.lw.LwConstraints;
import org.zaval.lw.LwContainer;
import org.zaval.lw.LwGridLayout;
import org.zaval.lw.LwLabel;
import org.zaval.lw.LwPanel;
import org.zaval.lw.LwToolkit;


/**
 * This dialog enables the selection of the bluetooth connection
 * @author manu
 *
 */
public class GPSBluetoothPrefsDialog extends OkCancelPrefsDialog {

	public static final String BLUETOOTH_SEPARATION_NAME = "  ";
	private static final long BLUETOOTH_CONNECTION_TIMEOUT = 12000;
		
	// row 1
	private LwCombo addressCombo;
	private LwButton refreshButton;
	// row 2
	private LwCombo baudCombo;

	/**
	 * Constructor
	 */
	public GPSBluetoothPrefsDialog(BundleContext bc, Properties prefs, PrefsListener listener) {
			super(false, bc, prefs, listener);
	}

	public String getTitle() {
		return Messages.getText("GPSBluetoothPrefsDialog.title");
	}	
	
	/*
	 * (non-Javadoc)
	 * @see org.gvsig.mobile.ui.lw.OkCancelDialog#buildMainPanel()
	 */
	public LwPanel buildMainPanel() {

		// --------- create controls: ----------------

		// labels
		final LwLabel addressLabel = DialogConstants.getDefaultLabel(Messages.getText("GPSBluetoothPrefsDialog.address"));
		final LwLabel baudRateLabel = DialogConstants.getDefaultLabel(Messages.getText("GPSBluetoothPrefsDialog.baud_rate"));
		// resize labels to homogenize
		int width = Math.max((int)addressLabel.getPreferredSize().width,
				(int)baudRateLabel.getPreferredSize().width);
		int height = Math.max((int)addressLabel.getPreferredSize().height,
				(int)baudRateLabel.getPreferredSize().height);
		addressLabel.setPSSize(width, height);
		baudRateLabel.setPSSize(width, height);

		// combos
		addressCombo = getAddressCombo();
		baudCombo = getBaudCombo();
		
		// buttons
		refreshButton = DialogConstants.getButton("options", "options", this);

		// --------- create layouts and fill the panel: ----------------
		LwConstraints rightC = DialogConstants.getTextFieldConstraints();
		LwConstraints leftC = DialogConstants.getLabelConstraints();

		// the panel
		LwPanel panel = new LwPanel();
		panel.setLwLayout(new LwBorderLayout());

		// a left column
		LwPanel left = new LwPanel();
		left.setLwLayout(new LwGridLayout(2, 1));
		
		// a right column, horizontal fill
		LwPanel right = new LwPanel();
		right.setLwLayout(new LwGridLayout(2, 1, LwToolkit.HORIZONTAL));		

		// fill left column
		left.add(leftC, addressLabel);		
		left.add(leftC, baudRateLabel);				

		// fill right column
		LwContainer c = new LwPanel(); // a container for the combo + button
		c.setLwLayout(new LwBorderLayout());
		c.add(LwBorderLayout.CENTER, addressCombo); // the combo takes more space
		c.add(LwBorderLayout.EAST, refreshButton);
		right.add(rightC, c);
		right.add(rightC, baudCombo);

		// add columns to panel
		panel.add(LwBorderLayout.WEST,left);
		panel.add(LwBorderLayout.CENTER,right);

		return panel;		
	}	
	
	private LwCombo getAddressCombo() {
		final LwCombo ret = new LwCombo();
		final LwCompList list = new LwCompList();
	
		try{
			BluetoothDeviceDiscover gpsDiscover = new BluetoothDeviceDiscover();
			BluetoothDevice [] bt = gpsDiscover.discover(BLUETOOTH_CONNECTION_TIMEOUT);

			for(int i=0; i<bt.length; i++)
			{
				String name = bt[i].getFriendlyName();
				String address = bt[i].getBtAddress();

				if( name == null )
				{
					name = Messages.getText("GPSDiscover.unknoun");
				}
				list.addItem(address + BLUETOOTH_SEPARATION_NAME + name);
			}
		}catch(Exception e){
			logger.error("Error reading devices connected by bluetooth");
			e.printStackTrace();
		}
		
		//list.addItem("00:02:78:1a:17:0b");
		if( list.getLines() > 0 )
			list.select(0);
			
		ret.setList(list);
		
		return ret;
	}
	
	private LwCombo getBaudCombo() {
		final LwCombo ret = new LwCombo();
		final LwCompList list = new LwCompList();
		list.addItem(GPSConstants.V_BAUD_RATE_4800);
		list.addItem(GPSConstants.V_BAUD_RATE_9600);
		list.addItem(GPSConstants.V_BAUD_RATE_19200);
		list.addItem(GPSConstants.V_BAUD_RATE_38400);
		list.addItem(GPSConstants.V_BAUD_RATE_57600);
		list.addItem(GPSConstants.V_BAUD_RATE_115200);
		list.select(0);
		ret.setList(list);
		return ret;
	}

	/**
	 * Utility method to get the index of a combo list
	 * @param combo
	 * @param text
	 * @return
	 */
	public int comboIndexOf(LwCombo combo, String text) {
		LwAList list = combo.getList();
		for (int i=0;i<list.count();i++) {
			if (((LwLabel)list.getItem(i)).getText().equals(text))
				return i;
		}
		return -1;
	}

	/*
	 * (non-Javadoc)
	 * @see org.gvsig.mobile.extension.gps.dialog.OkCancelPrefsDialog#collectPrefs()
	 */
	public Properties collectPrefs(Properties initPrefs) {
		Properties prefs = new Properties();

		if( ((LwLabel)addressCombo.getList().getSelected()) != null )
		{
			String addr_name = ((LwLabel)addressCombo.getList().getSelected()).getText();
			String address = addr_name.substring(0, addr_name.indexOf(GPSBluetoothPrefsDialog.BLUETOOTH_SEPARATION_NAME));
			prefs.setProperty(GPSConstants.K_DEVICE_ADDR, address);
			prefs.setProperty(GPSConstants.K_BAUD_RATE, ((LwLabel)baudCombo.getList().getSelected()).getText());
		}
		return prefs;
	}

	/*
	 * (non-Javadoc)
	 * @see org.gvsig.mobile.extension.gps.dialog.OkCancelPrefsDialog#selectDefaults(java.util.Properties)
	 */
	public void selectDefaults(Properties initPrefs) {
		if (initPrefs.containsKey(GPSConstants.K_DEVICE_ADDR)) {
			String portName = initPrefs.getProperty(GPSConstants.K_DEVICE_ADDR);
			int i = comboIndexOf(addressCombo, portName);
			if (i != -1)
				addressCombo.getList().select(i);
		}
		if (initPrefs.containsKey(GPSConstants.K_BAUD_RATE)) {
			String baudRate = initPrefs.getProperty(GPSConstants.K_BAUD_RATE);
			int i = comboIndexOf(baudCombo, baudRate);
			if (i != -1)
				baudCombo.getList().select(i);
		}
	}

}
