/* gvSIG. Sistema de Información Geográfica de la Generalitat Valenciana

 *
 * Copyright (C) 2006 Prodevelop and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *   Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibáñez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *   +34 963862235
 *   gvsig@gva.es
 *   http://www.gvsig.gva.es
 *
 *    or
 *
 *   Prodevelop Integración de Tecnologías SL
 *   Conde Salvatierra de Álava , 34-10
 *   46004 Valencia
 *   Spain
 *
 *   +34 963 510 612
 *   +34 963 510 968
 *   gis@prodevelop.es
 *   http://www.prodevelop.es
 *
 *    or
 *
 *   Instituto de Robótica
 *   Apartado de correos 2085
 *   46071 Valencia
 *   (Spain)
 *   
 *   +34 963 543 577
 *   jjordan@robotica.uv.es
 *   http://robotica.uv.es
 *   
 */

package org.gvsig.mobile.extension.gps.dialog;

import org.gvsig.i18n.Messages;
import org.gvsig.mobile.ui.awt.WindowManager;
import org.gvsig.mobile.ui.lw.DialogConstants;
import org.gvsig.mobile.ui.lw.OkCancelDialog;
import org.osgi.framework.BundleContext;
import org.zaval.lw.LwBorderLayout;
import org.zaval.lw.LwCheckbox;
import org.zaval.lw.LwConstraints;
import org.zaval.lw.LwFlowLayout;
import org.zaval.lw.LwGridLayout;
import org.zaval.lw.LwPanel;
import org.zaval.lw.LwToolkit;


/**
 * This dialog enables the selection of the NMEA messages to be processed
 * @author manu
 *
 */
public class GPSDataActivityDialog extends OkCancelDialog {

	// fixed width for labels that are on the left side of the dialog
	//private static final int LABELS_WIDTH = 65;
	//private static final int BUTTONS_WIDTH = 22;
	
	//Default values before click the button and show the dialog
	static public final boolean defaultPosition = true;
	static public final boolean defaultAltitude = true;
	static public final boolean defaultSpeed = true;
	static public final boolean defaultHDOP = true;
	static public final boolean defaultVDOP = true;
	static public final boolean defaultHPrecision = true;
	static public final boolean defaultVPrecision = false;

	// row 1
	private LwCheckbox positionChkbox;
	private LwCheckbox VDOPChkbox;

	// row 2
	private LwCheckbox altitudeChkbox;
	private LwCheckbox hPrecisionChkbox;

	// row 3
	private LwCheckbox speedChkbox;
	private LwCheckbox vPrecisionChkbox;

	// row 4
	private LwCheckbox HDOPChkbox;

	/**
	 * Constructor
	 */
	public GPSDataActivityDialog(BundleContext bc) 
	{
		super(false, bc);
		
		positionChkbox.setState(defaultPosition);
		VDOPChkbox.setState(defaultVDOP);
		altitudeChkbox.setState(defaultAltitude);
		hPrecisionChkbox.setState(defaultHPrecision);
		speedChkbox.setState(defaultSpeed);
		vPrecisionChkbox.setState(defaultVPrecision);
		HDOPChkbox.setState(defaultHDOP);
	}

	/**
	 * Fills the dialog with its controls
	 *
	 */
	public LwPanel buildMainPanel() {

		// --------- create controls: ----------------
		
		LwConstraints l = DialogConstants.getTextFieldConstraints();
		LwConstraints l1= DialogConstants.getLabelConstraints();

		LwPanel resp = new LwPanel();
		resp.setLwLayout(new LwGridLayout(4, 1, LwToolkit.HORIZONTAL));
		
		
		positionChkbox = new LwCheckbox(DialogConstants.getDefaultLabel(Messages.getText("GPSDataActivityDialog.1")), LwCheckbox.CHECK);
//		positionChkbox.setState(true);
		VDOPChkbox = new LwCheckbox(DialogConstants.getDefaultLabel(Messages.getText("GPSDataActivityDialog.5")), LwCheckbox.CHECK);
//		VDOPChkbox.setState(true);
		
		altitudeChkbox = new LwCheckbox(DialogConstants.getDefaultLabel(Messages.getText("GPSDataActivityDialog.2")), LwCheckbox.CHECK);
//		altitudeChkbox.setState(true);
		hPrecisionChkbox = new LwCheckbox(DialogConstants.getDefaultLabel(Messages.getText("GPSDataActivityDialog.6")), LwCheckbox.CHECK);
//		hPrecisionChkbox.setState(true);
		
		speedChkbox = new LwCheckbox(DialogConstants.getDefaultLabel(Messages.getText("GPSDataActivityDialog.3")), LwCheckbox.CHECK);
//		speedChkbox.setState(true);
		vPrecisionChkbox = new LwCheckbox(DialogConstants.getDefaultLabel(Messages.getText("GPSDataActivityDialog.7")), LwCheckbox.CHECK);
//		vPrecisionChkbox.setState(true);
		
		HDOPChkbox = new LwCheckbox(DialogConstants.getDefaultLabel(Messages.getText("GPSDataActivityDialog.4")), LwCheckbox.CHECK);
//		HDOPChkbox.setState(true);
		
		
		//row 1//
		LwPanel row1Panel = new LwPanel();
		row1Panel.setLwLayout(new LwBorderLayout());
		
		LwPanel row1upPanel = new LwPanel();
		row1upPanel.setLwLayout(new LwFlowLayout());
		row1upPanel.add(l, positionChkbox);
		
		LwPanel row1downPanel = new LwPanel();
		row1downPanel.setLwLayout(new LwFlowLayout());
		row1downPanel.add(l, VDOPChkbox);
		
		row1Panel.add(LwBorderLayout.WEST, DialogConstants.getDefaultLabel(" "));
		row1Panel.add(LwBorderLayout.WEST, row1upPanel);
		row1Panel.add(LwBorderLayout.CENTER, row1downPanel);
		
//		LwPanel row1Panel = new LwPanel();
//		row1Panel.setLwLayout(new LwFlowLayout());
//		row1Panel.add(LwBorderLayout.WEST, DialogConstants.getDefaultLabel(""));
//		row1Panel.add(LwBorderLayout.CENTER, positionChkbox);
//		row1Panel.add(LwBorderLayout.EAST, VDOPChkbox);
		
		//row 2//
		LwPanel row2Panel = new LwPanel();
		row2Panel.setLwLayout(new LwFlowLayout());				
		row2Panel.add(l, altitudeChkbox);
		row2Panel.add(l, hPrecisionChkbox);
		
		//row 3//
		LwPanel row3Panel = new LwPanel();
		row3Panel.setLwLayout(new LwFlowLayout());				
		row3Panel.add(l, speedChkbox);
		row3Panel.add(l, vPrecisionChkbox);
		
		//row 4//
		LwPanel row4Panel = new LwPanel();
		row4Panel.setLwLayout(new LwFlowLayout());				
		row4Panel.add(l, HDOPChkbox);


		resp.add(l, row1Panel);
		resp.add(l, row2Panel);
		resp.add(l, row3Panel);
		resp.add(l, row4Panel);

		return resp;
	}

	public void actionPerformed(Object arg0, Object arg1) {
		// TODO Auto-generated method stub
		WindowManager.getInstance().closeActivePanel();
	}

	public String getTitle() {
		return Messages.getText("GPSDataActivityDialog.0");
	}
	
	public void setPosition(boolean p)
	{
		positionChkbox.setState(p);
	}
	
	public void setAltitude(boolean a)
	{
		altitudeChkbox.setState(a);
	}
	
	public void setSpeed(boolean s)
	{
		speedChkbox.setState(s);
	}
	
	public void setHDOP(boolean hd)
	{
		HDOPChkbox.setState(hd);
	}
	
	public void setVDOP(boolean vd)
	{
		VDOPChkbox.setState(vd);
	}

	public void setHPrecision(boolean hp)
	{
		hPrecisionChkbox.setState(hp);
	}

	public void setVPrecision(boolean vp)
	{
		vPrecisionChkbox.setState(vp);
	}

	
	public boolean getPosition()
	{
		return positionChkbox.getState();
	}
	
	public boolean getAltitude()
	{
		return altitudeChkbox.getState();
	}
	
	public boolean getSpeed()
	{
		return speedChkbox.getState();
	}
	
	public boolean getHDOP()
	{
		return HDOPChkbox.getState();
	}
	
	public boolean getVDOP()
	{
		return VDOPChkbox.getState();
	}

	public boolean getHPrecision()
	{
		return hPrecisionChkbox.getState();
	}

	public boolean getVPrecision()
	{
		return vPrecisionChkbox.getState();
	}
	
}
