/* gvSIG. Sistema de Información Geográfica de la Generalitat Valenciana

 *
 * Copyright (C) 2006 Prodevelop and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *   Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibáñez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *   +34 963862235
 *   gvsig@gva.es
 *   http://www.gvsig.gva.es
 *
 *    or
 *
 *   Prodevelop Integración de Tecnologías SL
 *   Conde Salvatierra de Álava , 34-10
 *   46004 Valencia
 *   Spain
 *
 *   +34 963 510 612
 *   +34 963 510 968
 *   gis@prodevelop.es
 *   http://www.prodevelop.es
 *
 *    or
 *
 *   Instituto de Robótica
 *   Apartado de correos 2085
 *   46071 Valencia
 *   (Spain)
 *   
 *   +34 963 543 577
 *   jjordan@robotica.uv.es
 *   http://robotica.uv.es
 *   
 */
package org.gvsig.mobile.extension.gps.dialog;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.awt.Graphics;
import java.awt.SystemColor;

import org.gvsig.i18n.Messages;
import org.gvsig.location.LocationProperties;
import org.gvsig.location.LocationProvider;
import org.gvsig.location.nmea.SatelliteData;
import org.gvsig.mobile.ui.lw.DialogConstants;
import org.gvsig.mobile.ui.lw.GvsigMobileGUIConstants;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.zaval.lw.LwPanel;

/**
 * This class draw the satellite dialog.
 * 
 * @author IRTIC. Universitat de València.
 *
 */
public class GPSSatelliteDialog extends LwPanel implements PropertyChangeListener {
	
	protected static Logger logger = LoggerFactory.getLogger(GPSSatelliteDialog.class);
	
	public static int X_CENTER = 0;
	public static int Y_CENTER;
	public static int RADIUS_30;
	public static int RADIUS_60;
	public static int RADIUS_85;
	public static int RADIUS_90;
	public static final int OFFSET_0 = 3 * GvsigMobileGUIConstants.GENERAL_SIZE_FACTOR;
	public static int BAR_X;
	public static int BAR_Y;
	public static int BAR_WIDTH;
	public static final int BAR_HEIGHT = 36 * GvsigMobileGUIConstants.GENERAL_SIZE_FACTOR;
	
	int n_sat_seen;
	SatelliteData[] sat_in_use;
	SatelliteData[] sat_infos;
	int[] prn_used;
	int n_sat_used;
	

	public GPSSatelliteDialog() {
	}
	
	/**
	 * Draw the satellite info
	 */
	public void paint(Graphics g) 
	{
		int x, y;
    	int cx, cy;
    	// Execute only once, but not in the constructor
    	// (because component hasn't still width nor heigth)
    	if (X_CENTER == 0) computeLayout();
		    	
    	// Draw elevation circles
    	cx=X_CENTER; cy=Y_CENTER;
    	g.setColor(SystemColor.DARK_GRAY);
    	g.fillOval(cx-RADIUS_90,cy-RADIUS_90,RADIUS_90*2,RADIUS_90*2);
    	g.setColor(SystemColor.BLACK);
    	g.fillOval(cx-RADIUS_85,cy-RADIUS_85,RADIUS_85*2,RADIUS_85*2); // Horizon line (elevation 5Âº)
    	g.setColor(SystemColor.GRAY);
    	g.drawOval(cx-RADIUS_30,cy-RADIUS_30,RADIUS_30*2,RADIUS_30*2);
    	g.drawOval(cx-RADIUS_60,cy-RADIUS_60,RADIUS_60*2,RADIUS_60*2);
    	g.drawOval(cx-RADIUS_90,cy-RADIUS_90,RADIUS_90*2,RADIUS_90*2);
    	// Draw azimuth lines
    	for (int i=0; i<180; i+=30) {
    		double phi = i * Math.PI / 180;
    		int ox = (int)(RADIUS_90 * Math.cos(phi));
    		int oy = (int)(RADIUS_90 * Math.sin(phi));
        	g.drawLine(cx-ox,cy-oy,cx+ox,cy+oy);
    	}
    	// Write cardinal points
		g.setFont(DialogConstants.DIALOG_SMALL_FONT);
    	g.drawString(Messages.getText("GPSSatCanvas.0"),cx+OFFSET_0,cy+3*OFFSET_0-RADIUS_90); // NORTH
    	g.drawString(Messages.getText("GPSSatCanvas.1"),cx+OFFSET_0,cy-OFFSET_0+RADIUS_90); // SOUTH
    	g.drawString(Messages.getText("GPSSatCanvas.2"),cx-3*OFFSET_0+RADIUS_90,cy+3*OFFSET_0); // EAST
    	g.drawString(Messages.getText("GPSSatCanvas.3"),cx+OFFSET_0-RADIUS_90,cy+3*OFFSET_0); // WEST
    	
    	// Write satellites in view and satellites used
    	g.drawString(Messages.getText("GPSSatCanvas.4"),OFFSET_0,3*OFFSET_0); // VISIBLE
    	g.drawString(Integer.toString(n_sat_seen),OFFSET_0,6*OFFSET_0);
    	g.drawString(Messages.getText("GPSSatCanvas.5"),OFFSET_0,BAR_Y-BAR_HEIGHT-OFFSET_0); // USED
        g.drawString(Integer.toString(n_sat_used),OFFSET_0,BAR_Y-BAR_HEIGHT-4*OFFSET_0);

    	for (int i=0;i<12;i++) {
    		x = i * BAR_X ;
    		y = BAR_Y;
    		// Draw background of the 12 SNR bars
        	g.setColor(SystemColor.GRAY);
			g.fillRoundRect(x,y-BAR_HEIGHT,BAR_WIDTH,BAR_HEIGHT,0,0);
	    	// For each satellite info register
    		if (i<n_sat_seen) {
    			// Get PRN of the satellite
        		int prn = sat_infos[i].getPrn();
        		// Print PRN with a colour according
        		// to its use in the positioning solution
    			boolean is_fix_sat = false;
    			// Since prn_used and sat_seen could be both null,
    			// the condition has to be defined in a complex way
    			if (prn_used!=null) {
        			for (int j=0;j<prn_used.length;j++)
        				if ((prn_used[j]==prn)&&(prn!=0))
        					is_fix_sat = true;
    			} else if (sat_infos!=null) {
    				if (sat_infos[i].getSNR()!=0)
    					is_fix_sat = true;
    			}

    			// Get SNR of the satellite (generally values from 0 to 50)
    			if (sat_infos!=null) {
        			int snr = sat_infos[i].getSNR();
        			if (snr>50) snr=50; // Most gps devices snr range between 0 and 50 but not all
        			double azim = sat_infos[i].getAzimuth(); // Azimuth, from 0 to 359
        			double elev = sat_infos[i].getElevation(); // Elevation, from 0 to 90
        			double phi = (double)azim * Math.PI / 180; // phi, from 0 to 2*PI
        			double rad = RADIUS_90 - RADIUS_90 * elev / 90; // radius
        			
        			// Fill SNR bar
        			if (is_fix_sat) g.setColor(SystemColor.BLUE);
        			else g.setColor(SystemColor.DARK_GRAY);
        			g.fillRoundRect(x,y-snr*BAR_HEIGHT/50,BAR_WIDTH,snr*BAR_HEIGHT/50,0,0);
        			
        			// Write PRN in the azimuth-elevation graph
        			if (is_fix_sat) g.setColor(SystemColor.BLUE);
        			else g.setColor(SystemColor.GRAY);
        			g.drawString(zeroPad(prn,2),(int)(cx+rad*Math.cos(phi))-2*OFFSET_0,(int)(cy+rad*Math.sin(phi))+OFFSET_0);
    			}

    			// Show the prn value
    			g.setColor(SystemColor.LIGHT_GRAY);
    			g.drawString(zeroPad(prn,2),x+OFFSET_0,y-OFFSET_0);
    		}
    	}
	}
	
	/**
	 * Compute size of elements of the layout
	 * which depend on screen size
	 */
	private void computeLayout() {
		int width = this.getWidth();
		int height = this.getHeight();
    	X_CENTER = width / 2;
    	Y_CENTER = (height - BAR_HEIGHT) / 2;
    	BAR_Y = height;
    	BAR_WIDTH = Math.round((width - 11 * GvsigMobileGUIConstants.GENERAL_SIZE_FACTOR) / 12.0f);
    	BAR_X = BAR_WIDTH + GvsigMobileGUIConstants.GENERAL_SIZE_FACTOR;
    	int radius_a = (height - 2 * OFFSET_0 - BAR_HEIGHT) / 2;
    	int radius_b = (width - 2 * OFFSET_0) / 2;
    	RADIUS_90 = Math.min(radius_a, radius_b);
    	RADIUS_85 = RADIUS_90 * 85 / 90;
    	RADIUS_60 = RADIUS_90 * 2 / 3;
    	RADIUS_30 = RADIUS_90 / 3;
	}
	
	/**
	 * Utility method to zero pad an integer value
	 * @param value
	 * @param n_digits
	 * @return
	 */
	private String zeroPad(int value, int n_digits) {
		String s=null;
		s = Integer.toString(value);
		while (s.length()<n_digits)
			s = '0' + s;
		return s;
	}

	public void providerStateChanged(LocationProvider provider, int newState) {
	}
	
	/**
	 * Update the satellites data used and seen.
	 */
	public void propertyChange(PropertyChangeEvent evt) {
		String key = evt.getPropertyName();
		if (key.equals(LocationProperties.GNSS_SAT_DATA)) {
			sat_infos = (SatelliteData[])evt.getNewValue();
			int used = 0;
			for (int i=0;i<sat_infos.length;i++)
			{
				if(sat_infos[i].isUsed() == true)
					used++;
			}
			
		    prn_used = new int[used]; 
		    int used_aux = used;
		    used = 0;
		    
		    for (int i=0;i<sat_infos.length;i++)
			{
				if(sat_infos[i].isUsed() == true)
				{
		    	    // Get PRN of satellites used
		    		prn_used[used] = sat_infos[i].getPrn();
		    		used++;
		    	}
			}
		    
		    // Get number of satellites used
	    	n_sat_used = used_aux;
	    	
	    	if ((n_sat_used==0)&&(prn_used!=null))
	    		n_sat_used = prn_used.length;
		
	    	if (sat_infos!=null)
		    	n_sat_seen = sat_infos.length;
		    else
		    	n_sat_seen = 0;
	    	
    		this.repaint();
	    }
	}
}
