/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2006 Prodevelop and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *   Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *   +34 963862235
 *   gvsig@gva.es
 *   http://www.gvsig.gva.es
 *
 *    or
 *
 *   Prodevelop Integracin de Tecnologas SL
 *   Conde Salvatierra de lava , 34-10
 *   46004 Valencia
 *   Spain
 *
 *   +34 963 510 612
 *   +34 963 510 968
 *   gis@prodevelop.es
 *   http://www.prodevelop.es
 *
 *    or
 *
 *   Instituto de Robtica
 *   Apartado de correos 2085
 *   46071 Valencia
 *   (Spain)
 *
 *   +34 963 543 577
 *   jjordan@robotica.uv.es
 *   http://robotica.uv.es
 *
 */

package org.gvsig.mobile.extension.gps.dialog;

import gnu.io.CommPortIdentifier;

import java.util.Enumeration;
import java.util.Properties;
import java.util.Vector;

import org.gvsig.i18n.Messages;
import org.gvsig.mobile.extension.gps.PrefsListener;
import org.gvsig.mobile.gps.GPSConstants;
import org.gvsig.mobile.ui.lw.DialogConstants;
import org.osgi.framework.BundleContext;
import org.zaval.lw.LwBorderLayout;
import org.zaval.lw.LwButton;
import org.zaval.lw.LwCombo;
import org.zaval.lw.LwCompList;
import org.zaval.lw.LwConstraints;
import org.zaval.lw.LwContainer;
import org.zaval.lw.LwGridLayout;
import org.zaval.lw.LwLabel;
import org.zaval.lw.LwPanel;
import org.zaval.lw.LwToolkit;


/**
 * This dialog enables the selection of the serial connection
 * @author manu
 *
 */
public class GPSSerialPrefsDialog extends OkCancelPrefsDialog {

	/*
	 * A cache for serial port names
	 */
	private String[] spNames = null;
	// row 1
	private LwCombo portCombo;
	private LwButton portButton;
	// row 2
	private LwCombo baudCombo;

	/**
	 * Constructor
	 */
	public GPSSerialPrefsDialog(BundleContext bc, Properties prefs, PrefsListener listener) {
		super(false, bc, prefs, listener);
	}

	/*
	 * (non-Javadoc)
	 * @see org.gvsig.mobile.ui.lw.OkCancelDialog#getTitle()
	 */
	public String getTitle() {
		return Messages.getText("GPSSerialPrefsDialog.title");
	}

	/*
	 * (non-Javadoc)
	 * @see org.gvsig.mobile.ui.lw.OkCancelDialog#buildMainPanel()
	 */
	public LwPanel buildMainPanel() {

		// --------- create controls: ----------------

		// labels
		final LwLabel portNameLabel = DialogConstants.getDefaultLabel(Messages.getText("GPSSerialPrefsDialog.port_name"));
		final LwLabel baudRateLabel = DialogConstants.getDefaultLabel(Messages.getText("GPSSerialPrefsDialog.baud_rate"));
		// resize labels to homogenize
		int width = Math.max((int)portNameLabel.getPreferredSize().width,
				(int)baudRateLabel.getPreferredSize().width);
		int height = Math.max((int)portNameLabel.getPreferredSize().height,
				(int)baudRateLabel.getPreferredSize().height);
		portNameLabel.setPSSize(width, height);
		baudRateLabel.setPSSize(width, height);

		// combos
		portCombo = getPortCombo();
		baudCombo = getBaudCombo();
		
		// buttons
		portButton = DialogConstants.getButton("options", "options", this);

		// --------- create layouts and fill the panel: ----------------
		LwConstraints rightC = DialogConstants.getTextFieldConstraints();
		LwConstraints leftC = DialogConstants.getLabelConstraints();

		// the panel
		LwPanel panel = new LwPanel();
		panel.setLwLayout(new LwBorderLayout());

		// a left column
		LwPanel left = new LwPanel();
		left.setLwLayout(new LwGridLayout(2, 1));
		
		// a right column, horizontal fill
		LwPanel right = new LwPanel();
		right.setLwLayout(new LwGridLayout(2, 1, LwToolkit.HORIZONTAL));		

		// fill left column
		left.add(leftC, portNameLabel);		
		left.add(leftC, baudRateLabel);				

		// fill right column
		LwContainer c = new LwPanel(); // a container for the combo + button
		c.setLwLayout(new LwBorderLayout());
		c.add(LwBorderLayout.CENTER, portCombo); // the combo takes more space
		c.add(LwBorderLayout.EAST, portButton);
		right.add(rightC, c);
		right.add(rightC, baudCombo);

		// add columns to panel
		panel.add(LwBorderLayout.WEST,left);
		panel.add(LwBorderLayout.CENTER,right);

		return panel;		
	}	

	private LwCombo getPortCombo() {
		final LwCombo ret = new LwCombo();
		final LwCompList list = new LwCompList();

//		LocationLibrary libLocation = new LocationLibrary();
//		CdcLocationLibrary cdcLibLocation = new CdcLocationLibrary();
//		libLocation.initialize();
//		cdcLibLocation.initialize();
//		libLocation.postInitialize();
//		cdcLibLocation.postInitialize();
		
//		ConnectionManager connManager = LocationLocator.getConnectionManager();
//		String[] portNames = connManager.getSerialPortNames(false);
		String[] portNames = getSerialPortNames(false);
		
		for (int i=0;i<portNames.length;i++) {
			list.addItem(portNames[i]);
		}
		list.addSelectionListener(this);		
		list.select(0);
		ret.setList(list);
		return ret;
	}
	
	private LwCombo getBaudCombo() {
		final LwCombo ret = new LwCombo();
		final LwCompList list = new LwCompList();
		list.addItem(GPSConstants.V_BAUD_RATE_4800);
		list.addItem(GPSConstants.V_BAUD_RATE_9600);
		list.addItem(GPSConstants.V_BAUD_RATE_19200);
		list.addItem(GPSConstants.V_BAUD_RATE_38400);
		list.addItem(GPSConstants.V_BAUD_RATE_57600);
		list.addItem(GPSConstants.V_BAUD_RATE_115200);
		list.select(0);
		ret.setList(list);
		return ret;
	}

	public String[] getSerialPortNames(boolean refresh) {
		// TODO use ConnectionManager method instead
		// Return cached values if the refresh flag is not set
		if ((refresh == true) || (spNames == null)) {
			// Otherwise, get serial port names
			Vector v = new Vector();
			Enumeration portList = CommPortIdentifier.getPortIdentifiers();
			while (portList.hasMoreElements()) {
				CommPortIdentifier portId =
					(CommPortIdentifier) portList.nextElement();
				if (portId.getPortType() == CommPortIdentifier.PORT_SERIAL) {
					v.add(portId.getName());
				}
			}
			spNames = (String[])v.toArray(new String[0]);
		}
		return spNames;
	}

	/*
	 * (non-Javadoc)
	 * @see org.gvsig.mobile.extension.gps.dialog.OkCancelPrefsDialog#collectPrefs()
	 */
	public Properties collectPrefs(Properties initPrefs) {
		initPrefs.setProperty(GPSConstants.K_PORT_NAME, ((LwLabel)portCombo.getList().getSelected()).getText());
		initPrefs.setProperty(GPSConstants.K_BAUD_RATE, ((LwLabel)baudCombo.getList().getSelected()).getText());
		return initPrefs;
	}

	/*
	 * (non-Javadoc)
	 * @see org.gvsig.mobile.extension.gps.dialog.OkCancelPrefsDialog#selectDefaults(java.util.Properties)
	 */
	public void selectDefaults(Properties initPrefs) {
		if (initPrefs.containsKey(GPSConstants.K_PORT_NAME)) {
			String portName = initPrefs.getProperty(GPSConstants.K_PORT_NAME);
			int i = comboIndexOf(portCombo, portName);
			if (i != -1)
				portCombo.getList().select(i);
		}
		if (initPrefs.containsKey(GPSConstants.K_BAUD_RATE)) {
			String baudRate = initPrefs.getProperty(GPSConstants.K_BAUD_RATE);
			int i = comboIndexOf(baudCombo, baudRate);
			if (i != -1)
				baudCombo.getList().select(i);
		}
	}
}
