/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2006 Prodevelop and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *   Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *   +34 963862235
 *   gvsig@gva.es
 *   http://www.gvsig.gva.es
 *
 *    or
 *
 *   Prodevelop Integracin de Tecnologas SL
 *   Conde Salvatierra de lava , 34-10
 *   46004 Valencia
 *   Spain
 *
 *   +34 963 510 612
 *   +34 963 510 968
 *   gis@prodevelop.es
 *   http://www.prodevelop.es
 *
 *    or
 *
 *   Instituto de Robtica
 *   Apartado de correos 2085
 *   46071 Valencia
 *   (Spain)
 *
 *   +34 963 543 577
 *   jjordan@robotica.uv.es
 *   http://robotica.uv.es
 *
 */

package org.gvsig.mobile.extension.gps.dialog;

import java.util.Properties;

import org.gvsig.i18n.Messages;
import org.gvsig.mobile.extension.gps.PrefsListener;
import org.gvsig.mobile.gps.GPSConstants;
import org.gvsig.mobile.ui.lw.DialogConstants;
import org.osgi.framework.BundleContext;
import org.zaval.lw.LwBorderLayout;
import org.zaval.lw.LwConstraints;
import org.zaval.lw.LwGridLayout;
import org.zaval.lw.LwLabel;
import org.zaval.lw.LwPanel;
import org.zaval.lw.LwTextField;
import org.zaval.lw.LwToolkit;
import org.zaval.lw.mask.BasicMaskValidator;
import org.zaval.lw.mask.LwMaskTextField;
import org.zaval.lw.mask.MaskedText;


/**
 * This dialog enables the selection of the socket GPSd connection
 * @author manu
 *
 */
public class GPSSocketPrefsDialog extends OkCancelPrefsDialog {

	// row 1
	private LwTextField hostNameText;
	// row 2
	private LwTextField portNumberText;

	/**
	 * Constructor
	 */
	public GPSSocketPrefsDialog(BundleContext bc, Properties prefs, PrefsListener listener) {
		super(false, bc, prefs, listener);
	}

	/*
	 * (non-Javadoc)
	 * @see org.gvsig.mobile.ui.lw.OkCancelDialog#getTitle()
	 */
	public String getTitle() {
		return Messages.getText("GPSSocketPrefsDialog.title");
	}	

	/**
	 * Fills the dialog with its controls
	 *
	 */
	public LwPanel buildMainPanel() {
		// --------- create controls: ----------------

		// labels
		final LwLabel hostNameLabel = DialogConstants.getDefaultLabel(Messages.getText("GPSSocketPrefsDialog.host_name"));
		final LwLabel portNumberLabel = DialogConstants.getDefaultLabel(Messages.getText("GPSSocketPrefsDialog.port_number"));
		// resize labels to homogenize
		int width = Math.max((int)hostNameLabel.getPreferredSize().width,
				(int)portNumberLabel.getPreferredSize().width);
		int height = Math.max((int)hostNameLabel.getPreferredSize().height,
				(int)portNumberLabel.getPreferredSize().height);
		hostNameLabel.setPSSize(width, height);
		portNumberLabel.setPSSize(width, height);

		// Text fields
		hostNameText = getHostNameText();
		portNumberText = getPortNumberText();
		
		// --------- create layouts and fill the panel: ----------------
		LwConstraints rightC = DialogConstants.getTextFieldConstraints();
		LwConstraints leftC = DialogConstants.getLabelConstraints();

		// the panel
		LwPanel panel = new LwPanel();
		panel.setLwLayout(new LwBorderLayout());

		// a left column
		LwPanel left = new LwPanel();
		left.setLwLayout(new LwGridLayout(2, 1));
		
		// a right column, horizontal fill
		LwPanel right = new LwPanel();
		right.setLwLayout(new LwGridLayout(2, 1, LwToolkit.HORIZONTAL));		

		// fill left column
		left.add(leftC, hostNameLabel);		
		left.add(leftC, portNumberLabel);				

		// fill right column
		right.add(rightC, hostNameText);
		right.add(rightC, portNumberText);

		// add columns to panel
		panel.add(LwBorderLayout.WEST,left);
		panel.add(LwBorderLayout.CENTER,right);

		return panel;		
	}	

	private LwTextField getHostNameText() {
		return new LwTextField ("", 255);
	}
	
	private LwTextField getPortNumberText() {
		LwMaskTextField mtf = new LwMaskTextField(new MaskedText("nnnnn"));
		mtf.setValidator (new BasicMaskValidator(0, 65535));
		mtf.write(0,"02947");
		return mtf;
	}

	private String zeroPad(String numberTxt, int digits) {
		int n = 0;
		try {
			n = Integer.parseInt(numberTxt);
		} catch (NumberFormatException e) {
		}
		StringBuffer buf = new StringBuffer(Integer.toString(n));
		int padding = digits - buf.length();
		for (int i=0;i<padding;i++) {
			buf.insert(0, '0');
		}
		return buf.toString();
	}
	
	/*
	 * (non-Javadoc)
	 * @see org.gvsig.mobile.extension.gps.dialog.OkCancelPrefsDialog#collectPrefs()
	 */
	public Properties collectPrefs(Properties initPrefs) {
		Properties prefs = new Properties();
		prefs.setProperty(GPSConstants.K_HOST_NAME, hostNameText.getText());
		int portNumber = 0;
		try {
			portNumber = Integer.parseInt(portNumberText.getText());
		} catch (NumberFormatException e) {
		}
		prefs.setProperty(GPSConstants.K_PORT_NUMBER, Integer.toString(portNumber));
		return prefs;
	}

	/*
	 * (non-Javadoc)
	 * @see org.gvsig.mobile.extension.gps.dialog.OkCancelPrefsDialog#selectDefaults(java.util.Properties)
	 */
	public void selectDefaults(Properties initPrefs) {
		if (initPrefs.containsKey(GPSConstants.K_HOST_NAME)) {
			hostNameText.setText(initPrefs.getProperty(GPSConstants.K_HOST_NAME));			
		}
		if (initPrefs.containsKey(GPSConstants.K_PORT_NUMBER)) {
			portNumberText.setText(zeroPad(initPrefs.getProperty(GPSConstants.K_PORT_NUMBER), 5));
		}
	}

}
