package org.gvsig.mobile.extension.gps.functionality;

import java.util.Properties;

import org.gvsig.mobile.core.concepts.Functionality;
import org.gvsig.mobile.core.interfaces.IGvSigMouseEvent;
import org.gvsig.mobile.extension.gps.PrefsListener;
import org.gvsig.mobile.extension.gps.dialog.GPSPrefsDialog;
import org.gvsig.mobile.gps.GPSConstants;
import org.gvsig.mobile.gps.GPSResolutor;
import org.osgi.framework.ServiceReference;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class GPSConfigFunc extends Functionality implements PrefsListener {

	private static final Logger logger = LoggerFactory.getLogger(GPSConfigFunc.class);

	public GPSConfigFunc() {
		super();
	}

	public void configure() {
	}

	public boolean execution(ServiceReference gvSIGMobileRef) {
		try {
			Properties gpsPrefs = GPSResolutor.getGPSPrefs();
			final GPSPrefsDialog d = new GPSPrefsDialog(this.getBundleContext(), gpsPrefs, this);
			d.showDialog();
			super.stop();
			return true;
		} catch (Exception e) {
			logger.error("GPS ERROR : \n"+e.fillInStackTrace().toString());
			return false;
		}
	}

	public void onNewMouseEventListened(IGvSigMouseEvent event) {
		// TODO Auto-generated method stub
		
	}

	/*
	 * (non-Javadoc)
	 * @see org.gvsig.mobile.extension.gps.PrefsListener#prefsChanged(java.lang.Object, java.util.Properties)
	 */
	public void prefsChanged(Object src, Properties prefs) {
		Properties gpsPrefs = GPSResolutor.getGPSPrefs();
		if (prefs.containsKey(GPSConstants.K_TARGET)) {
			String target = prefs.getProperty(GPSConstants.K_TARGET);
			// If target changes or target parameters change
			if (target.equals(GPSConstants.V_TARGET_COMM)) {
				if ((!target.equals(gpsPrefs.getProperty(GPSConstants.K_TARGET))) ||
						(!prefs.getProperty(GPSConstants.K_PORT_NAME).equals(
								gpsPrefs.getProperty(GPSConstants.K_PORT_NAME))) ||
						(!prefs.getProperty(GPSConstants.K_BAUD_RATE).equals(
								gpsPrefs.getProperty(GPSConstants.K_BAUD_RATE)))) {
					logger.debug("Changed configuration to a new serial port connection.");					
					logger.debug("Port Name: " + prefs.getProperty(GPSConstants.K_PORT_NAME));
					logger.debug("Baud Rate: " + prefs.getProperty(GPSConstants.K_BAUD_RATE));
				}
			} else if (target.equals(GPSConstants.V_TARGET_FILE)) {
				if ((!target.equals(gpsPrefs.getProperty(GPSConstants.K_TARGET))) ||
						(!prefs.getProperty(GPSConstants.K_FILE_NAME).equals(
								gpsPrefs.getProperty(GPSConstants.K_FILE_NAME))) ||
						(!prefs.getProperty(GPSConstants.K_INTERVAL).equals(
								gpsPrefs.getProperty(GPSConstants.K_INTERVAL)))) {
					logger.debug("Changed configuration to a new file connection.");					
					logger.debug("File Name: " + prefs.getProperty(GPSConstants.K_FILE_NAME));
					logger.debug("Interval: " + prefs.getProperty(GPSConstants.K_INTERVAL));
				}
			} else if (target.equals(GPSConstants.V_TARGET_BTSPP)) {
				if ((!target.equals(gpsPrefs.getProperty(GPSConstants.K_TARGET))) ||
						(!prefs.getProperty(GPSConstants.K_DEVICE_ADDR).equals(
								gpsPrefs.getProperty(GPSConstants.K_DEVICE_ADDR))) ||
						(!prefs.getProperty(GPSConstants.K_BAUD_RATE).equals(
								gpsPrefs.getProperty(GPSConstants.K_BAUD_RATE)))) {
					logger.debug("Changed configuration to a new bluetooth connection.");					
					logger.debug("Device Adrress: " + prefs.getProperty(GPSConstants.K_DEVICE_ADDR));
					logger.debug("Baud Rate: " + prefs.getProperty(GPSConstants.K_BAUD_RATE));
				}
			} else if (target.equals(GPSConstants.V_TARGET_HTTP)) {
				if ((!target.equals(gpsPrefs.getProperty(GPSConstants.K_TARGET))) ||
						(!prefs.getProperty(GPSConstants.K_FILE_NAME).equals(
								gpsPrefs.getProperty(GPSConstants.K_FILE_NAME))) ||
						(!prefs.getProperty(GPSConstants.K_INTERVAL).equals(
								gpsPrefs.getProperty(GPSConstants.K_INTERVAL)))) {
					logger.debug("Changed configuration to a new gpsd socket connection.");					
					logger.debug("Host Name: " + prefs.getProperty(GPSConstants.K_HOST_NAME));
					logger.debug("Port Number: " + prefs.getProperty(GPSConstants.K_PORT_NUMBER));
				}
			}

		}
		GPSResolutor.setGPSPrefs(prefs);
	}

}
