package org.gvsig.mobile.extension.wkprojection;

import java.util.HashMap;

import org.gvsig.mobile.extension.wkprojection.jhlabs.adapter.EllipsoidAdapter;
import org.gvsig.mobile.extension.wkprojection.jhlabs.adapter.MapProjectionAdapter;
import org.gvsig.mobile.extension.wkprojection.jhlabs.map.osm.OSMSphericalMercatorTransformation;
import org.gvsig.mobile.extension.wkprojection.jhlabs.map.proj.AlbersProjection;
import org.gvsig.mobile.extension.wkprojection.jhlabs.map.proj.Ellipsoid;
import org.gvsig.mobile.extension.wkprojection.jhlabs.map.proj.LambertConformalConicProjection;
import org.gvsig.mobile.extension.wkprojection.jhlabs.map.proj.MercatorProjection;
import org.gvsig.mobile.extension.wkprojection.jhlabs.map.proj.TransverseMercatorProjection;
import org.gvsig.mobile.libProjection.impl.CRS;
import org.gvsig.mobile.libProjection.impl.CRSExtension;
import org.gvsig.mobile.libProjection.impl.CRSResolutor;
import org.gvsig.mobile.libProjection.impl.Datum;
import org.gvsig.mobile.libProjection.impl.Trans7Params;
import org.gvsig.mobile.libProjection.interfaces.CRSConstants;
import org.gvsig.mobile.libProjection.interfaces.IExtendedDatum;
import org.gvsig.mobile.libProjection.interfaces.IMapProjection;
import org.gvsig.mobile.libProjection.interfaces.ITransformationAlgorithm;
import org.osgi.framework.BundleContext;
import org.osgi.framework.BundleException;

public class WellKnownCRSExtension extends CRSExtension {

	private EllipsoidAdapter e_int24, e_popsph, e_ck1880fr, e_airy, e_ck1866, e_grs80;
	private IExtendedDatum edad_ed50_eu, edad_ed50_es, edad_ed50_es_w, edad_ed50_es_cat,
	edad_ed50_es_bal, edad_popsph, edad_ntf, edad_uk, edad_mm_w, edad_mm_e,
	edad_mm_sa, edad_mm_si, edad_usa_nad, edad_usa_grs, edad_jap;

	public WellKnownCRSExtension(BundleContext bc) {
		super(bc);
	}

	public void loadEllipsoidAndDatums() throws BundleException {
		
		try {
			// ellipsoids:
			e_int24 = new EllipsoidAdapter(Ellipsoid.INT_1924); // europe 
			addEllipsoid(e_int24);
			e_popsph = new EllipsoidAdapter(Ellipsoid.POP_SPH); // google, osm etc
			addEllipsoid(e_popsph);
			e_ck1880fr = new EllipsoidAdapter(Ellipsoid.CLARKE_1880_IGN); // france
			addEllipsoid(e_ck1880fr);
			e_airy = new EllipsoidAdapter(Ellipsoid.AIRY); // uk
			addEllipsoid(e_airy);
			e_ck1866 = new EllipsoidAdapter(Ellipsoid.CLARKE_1866); // usa
			addEllipsoid(e_ck1866);
			e_grs80 = new EllipsoidAdapter(Ellipsoid.GRS_1980); // japan
			addEllipsoid(e_grs80);

			// datums
			edad_ed50_eu = new Datum("ED50 Europe Central", e_int24);
			addDatum(edad_ed50_eu);
			edad_ed50_es = new Datum("ED50 Spain Central", e_int24);
			addDatum(edad_ed50_es);
			edad_ed50_es_w = new Datum("ED50 Spain North West", e_int24);
			addDatum(edad_ed50_es_w);
			edad_ed50_es_cat = new Datum("ED50 Spain Catalonia", e_int24);
			addDatum(edad_ed50_es_cat);
			edad_ed50_es_bal = new Datum("ED50 Spain Balearic", e_int24);
			addDatum(edad_ed50_es_bal);
			edad_popsph = new Datum("POPSPH", e_popsph);
			addDatum(edad_popsph);
			edad_ntf = new Datum("NTF", e_ck1880fr);
			addDatum(edad_ntf);
			edad_uk = new Datum("OSGB36", e_airy);
			addDatum(edad_uk);
			edad_mm_w = new Datum("MonteMario Continent West", e_int24);
			addDatum(edad_mm_w);
			edad_mm_e = new Datum("MonteMario Continent East", e_int24);
			addDatum(edad_mm_e);
			edad_mm_sa = new Datum("MonteMario Sardinia", e_int24);
			addDatum(edad_mm_sa);
			edad_mm_si = new Datum("MonteMario Sicily", e_int24); // italy
			addDatum(edad_mm_si);
			edad_usa_nad = new Datum("NAD27 CONUS", e_ck1866); // usa mainland
			addDatum(edad_usa_nad);
			edad_usa_grs = new Datum("NAD1983 CONUS", e_grs80); // usa mainland usgs version
			addDatum(edad_usa_grs);
			edad_jap = new Datum("JDG2000", e_grs80); // japan
			addDatum(edad_jap);
		} catch (Throwable th) {
			throw new BundleException("While loading WK projections: " + th.getMessage());
		}
	}

	
	public void loadCRSs() throws BundleException {
		
		try {
			
			// projections
			HashMap hm = new HashMap();
			IMapProjection p;
			
			// ===========================================================================
			// OSM GOOGLE YAHOO BING
			// =====================================================================
			p = CRSResolutor.getProjection("Mercator", bc);
			p = ((MapProjectionAdapter) p).cloneWithoutParams();
			
			hm.clear();
			hm.put(IMapProjection.PARAM_PROJECTION_ELLIPSE_ABREV, "popsph");
			p.setParams(hm);
			p.setAbrev("EPSG:3857");
			p.initialize();
			addCRS(new CRS(p, edad_popsph, "EPSG:3857", bc));
			// ===========================================================================
			// SPAIN
			// 23030 =====================================================================
			p = CRSResolutor.getProjection("Transverse Mercator", bc);
			p = ((MapProjectionAdapter) p).cloneWithoutParams();
			
			hm.clear();
			hm.put(IMapProjection.PARAM_PROJECTION_ELLIPSE_ABREV, "intl");
			hm.put(IMapProjection.PARAM_PROJECTION_UTM_ZONE, new Integer(30));
			p.setParams(hm);
			p.setAbrev("EPSG:23030");
			p.initialize();
			addCRS(new CRS(p, edad_ed50_es, "EPSG:23030", bc));
			// 23031 bal =====================================================================
			/*
			p = CRSResolutor.getProjection("Transverse Mercator", bc);
			p = ((MapProjectionAdapter) p).cloneWithoutParams();
			
			hm.clear();
			hm.put(IMapProjection.PARAM_PROJECTION_ELLIPSE_ABREV, "intl");
			hm.put(IMapProjection.PARAM_PROJECTION_UTM_ZONE, new Integer(31));
			p.setParams(hm);
			p.setAbrev("EPSG:23031");
			p.initialize();
			addCRS(new CRS(p, edad_ed50_es_bal, "EPSG:23031 Spain Balearic", bc));
			*/
			// 23031 cat =====================================================================
			p = CRSResolutor.getProjection("Transverse Mercator", bc);
			p = ((MapProjectionAdapter) p).cloneWithoutParams();
			
			hm.clear();
			hm.put(IMapProjection.PARAM_PROJECTION_ELLIPSE_ABREV, "intl");
			hm.put(IMapProjection.PARAM_PROJECTION_UTM_ZONE, new Integer(31));
			p.setParams(hm);
			p.setAbrev("EPSG:23031");
			p.initialize();
			// addCRS(new CRS(p, edad_ed50_es_cat, "EPSG:23031 Spain Catalonia", bc));
			addCRS(new CRS(p, edad_ed50_es_cat, "EPSG:23031", bc));
			// 23029 =====================================================================
			p = CRSResolutor.getProjection("Transverse Mercator", bc);
			p = ((MapProjectionAdapter) p).cloneWithoutParams();
			
			hm.clear();
			hm.put(IMapProjection.PARAM_PROJECTION_ELLIPSE_ABREV, "intl");
			hm.put(IMapProjection.PARAM_PROJECTION_UTM_ZONE, new Integer(29));
			p.setParams(hm);
			p.setAbrev("EPSG:23029");
			p.initialize();
			// addCRS(new CRS(p, edad_ed50_es_w, "EPSG:23029 Spain North West", bc));
			addCRS(new CRS(p, edad_ed50_es_w, "EPSG:23029", bc));
			// =====================================================================
			// FR
			// =====================================================================
			p = CRSResolutor.getProjection("Lambert Conformal Conic", bc);
			p = ((MapProjectionAdapter) p).cloneWithoutParams();
			
			hm.clear();
			hm.put(IMapProjection.PARAM_PROJECTION_ELLIPSE_ABREV, "clrk80ign");
			hm.put(IMapProjection.PARAM_PROJECTION_LONGITUDE, new Double(2.33722917));
			hm.put(IMapProjection.PARAM_PROJECTION_LATITUDE, new Double(46.8));
			hm.put(IMapProjection.PARAM_PROJECTION_LAT_1, new Double(46.8));
			hm.put(IMapProjection.PARAM_PROJECTION_LAT_2, new Double(46.8));
			hm.put(IMapProjection.PARAM_PROJECTION_FALSE_E, new Double(600000));
			hm.put(IMapProjection.PARAM_PROJECTION_FALSE_N, new Double(2200000));
			hm.put(IMapProjection.PARAM_PROJECTION_KFACTOR, new Double(0.99987742));
			p.setParams(hm);
			p.setAbrev("EPSG:27572");
			p.initialize();
			addCRS(new CRS(p, edad_ntf, "EPSG:27572", bc));
			// =====================================================================
//			p = CRSResolutor.getProjection("Lambert Conformal Conic", bc);
//			p = ((MapProjectionAdapter) p).cloneWithoutParams();
//			
//			hm.clear();
//			hm.put(IMapProjection.PARAM_PROJECTION_ELLIPSE_ABREV, "clrk80ign");
//			hm.put(IMapProjection.PARAM_PROJECTION_LONGITUDE, new Double(2.33722917));
//			hm.put(IMapProjection.PARAM_PROJECTION_LATITUDE, new Double(46.8));
//			hm.put(IMapProjection.PARAM_PROJECTION_LAT_1, new Double(46.8));
//			hm.put(IMapProjection.PARAM_PROJECTION_LAT_2, new Double(46.8));
//			hm.put(IMapProjection.PARAM_PROJECTION_FALSE_E, new Double(600000));
//			hm.put(IMapProjection.PARAM_PROJECTION_FALSE_N, new Double(2200000));
//			p.setParams(hm);
//			p.setAbrev("EPSG:27572");
//			p.initialize();
//			addCRS(new CRS(p, edad_ntf, "EPSG:27572", bc));
//			
//			
//			lcc = new LambertConformalConicProjection(
//					Ellipsoid.CLARKE_1880_IGN, 2.33722917, 49.5, 49.5, 49.5, 600000, 200000);
//			lcc.setScaleFactor(0.999877341);
//			lcc.setCode("EPSG:27561");
//			mpad = new MapProjectionAdapter(lcc);
//			addProjection(mpad);
//			crad = new CRSAdapter(mpad, edad_ntf, "EPSG:27561");
//			addCRS(crad);
//			// =====================================================================
//			lcc = new LambertConformalConicProjection(
//					Ellipsoid.CLARKE_1880_IGN, 2.33722917, 46.8, 46.8, 46.8, 600000, 200000);
//			lcc.setScaleFactor(0.99987742);
//			lcc.setCode("EPSG:27562");
//			mpad = new MapProjectionAdapter(lcc);
//			addProjection(mpad);
//			crad = new CRSAdapter(mpad, edad_ntf, "EPSG:27562");
//			addCRS(crad);
//			// =====================================================================
//			lcc = new LambertConformalConicProjection(
//					Ellipsoid.CLARKE_1880_IGN, 2.33722917, 44.1, 44.1, 44.1, 600000, 200000);
//			lcc.setScaleFactor(0.999877499);
//			lcc.setCode("EPSG:27563");
//			mpad = new MapProjectionAdapter(lcc);
//			addProjection(mpad);
//			crad = new CRSAdapter(mpad, edad_ntf, "EPSG:27563");
//			addCRS(crad);
//			// =====================================================================
//			lcc = new LambertConformalConicProjection(
//					Ellipsoid.CLARKE_1880_IGN, 2.33722917, 42.165, 42.165, 42.165, 234.358, 185861.369);
//			lcc.setScaleFactor(0.99994471);
//			lcc.setCode("EPSG:27564");
//			mpad = new MapProjectionAdapter(lcc);
//			addProjection(mpad);
//			crad = new CRSAdapter(mpad, edad_ntf, "EPSG:27564");
//			addCRS(crad);
//			// =====================================================================
//			// UK
//			// =====================================================================
			p = CRSResolutor.getProjection("Transverse Mercator", bc);
			p = ((MapProjectionAdapter) p).cloneWithoutParams();
			
			hm.clear();
			hm.put(IMapProjection.PARAM_PROJECTION_ELLIPSE_ABREV, "airy");
			hm.put(IMapProjection.PARAM_PROJECTION_LONGITUDE, new Double(-2));
			hm.put(IMapProjection.PARAM_PROJECTION_LATITUDE, new Double(49));
			hm.put(IMapProjection.PARAM_PROJECTION_FALSE_E, new Double(400000));
			hm.put(IMapProjection.PARAM_PROJECTION_FALSE_N, new Double(-100000));
			hm.put(IMapProjection.PARAM_PROJECTION_KFACTOR, new Double(0.9996012717));
			p.setParams(hm);
			p.setAbrev("EPSG:27700");
			p.initialize();
			addCRS(new CRS(p, edad_uk, "EPSG:27700", bc));
//			// =====================================================================
//			// IT
//			// =====================================================================
//			tmp = new TransverseMercatorProjection(Ellipsoid.INT_1924, 9, 0, 0.9996, 1500000, 0);
//			tmp.setCode("EPSG:3003");
//			mpad = new MapProjectionAdapter(tmp);
//			addProjection(mpad);
//			crad = new CRSAdapter(mpad, edad_mm_w, "EPSG:3003 Italy Continental West");
//			addCRS(crad);
//			crad = new CRSAdapter(mpad, edad_mm_sa, "EPSG:3003 Italy Sardinia");
//			addCRS(crad);
//			// =====================================================================
//			tmp = new TransverseMercatorProjection(Ellipsoid.INT_1924, 15, 0, 0.9996, 2520000, 0);
//			tmp.setCode("EPSG:3004");
//			mpad = new MapProjectionAdapter(tmp);
//			addProjection(mpad);
//			crad = new CRSAdapter(mpad, edad_mm_e, "EPSG:3004 Italy Continental East");
//			addCRS(crad);
//			crad = new CRSAdapter(mpad, edad_mm_si, "EPSG:3004 Italy Sicily");
//			addCRS(crad);
//			// =====================================================================
//			// USA
//			// =====================================================================
//			AlbersProjection alb = new AlbersProjection();
//			alb.setProjectionLongitudeDegrees(-96);
//			alb.setProjectionLatitudeDegrees(23);
//			alb.setEllipsoid(Ellipsoid.CLARKE_1866);  // will use datum nad27
//			alb.setCode("USA_ACEA");
//			mpad = new MapProjectionAdapter(alb);
//			addProjection(mpad);
//			crad = new CRSAdapter(mpad, edad_usa_nad, "USA CONUS ACEA");
//			addCRS(crad);
//			// =====================================================================
//			alb = new AlbersProjection();
//			alb.setProjectionLongitudeDegrees(-96);
//			alb.setProjectionLatitudeDegrees(23);
//			alb.setEllipsoid(Ellipsoid.GRS_1980); // will use datum "D_North_American_1983"
//			alb.setCode("USA_ACEA_USGS");
//			mpad = new MapProjectionAdapter(alb);
//			addProjection(mpad);
//			crad = new CRSAdapter(mpad, edad_usa_grs, "USA CONUS ACEA USGS");
//			addCRS(crad);
//			// =====================================================================
//			// JAPAN
//			// =====================================================================
//			tmp = new TransverseMercatorProjection();
//			tmp.setEllipsoid(Ellipsoid.GRS_1980);
//			tmp.setUTMZone(51);
//			tmp.initialize();
//			tmp.setCode("EPSG:3097");
//			mpad = new MapProjectionAdapter(tmp);
//			addProjection(mpad);
//			crad = new CRSAdapter(mpad, edad_jap, "EPSG:3097");
//			addCRS(crad);
//			// =====================================================================
//			tmp = new TransverseMercatorProjection();
//			tmp.setEllipsoid(Ellipsoid.GRS_1980);
//			tmp.setUTMZone(52);
//			tmp.initialize();
//			tmp.setCode("EPSG:3098");
//			mpad = new MapProjectionAdapter(tmp);
//			addProjection(mpad);
//			crad = new CRSAdapter(mpad, edad_jap, "EPSG:3098");
//			addCRS(crad);
//			// =====================================================================
//			tmp = new TransverseMercatorProjection();
//			tmp.setEllipsoid(Ellipsoid.GRS_1980);
//			tmp.setUTMZone(53);
//			tmp.initialize();
//			tmp.setCode("EPSG:3099");
//			mpad = new MapProjectionAdapter(tmp);
//			addProjection(mpad);
//			crad = new CRSAdapter(mpad, edad_jap, "EPSG:3099");
//			addCRS(crad);
//			// =====================================================================
//			tmp = new TransverseMercatorProjection();
//			tmp.setEllipsoid(Ellipsoid.GRS_1980);
//			tmp.setUTMZone(54);
//			tmp.initialize();
//			tmp.setCode("EPSG:3100");
//			mpad = new MapProjectionAdapter(tmp);
//			addProjection(mpad);
//			crad = new CRSAdapter(mpad, edad_jap, "EPSG:3100");
//			addCRS(crad);
//			// =====================================================================
//			tmp = new TransverseMercatorProjection();
//			tmp.setEllipsoid(Ellipsoid.GRS_1980);
//			tmp.setUTMZone(55);
//			tmp.initialize();
//			tmp.setCode("EPSG:3101");
//			mpad = new MapProjectionAdapter(tmp);
//			addProjection(mpad);
//			crad = new CRSAdapter(mpad, edad_jap, "EPSG:3101");
//			addCRS(crad);
//			// =====================================================================
			
			


		} catch (Throwable th) {
			throw new BundleException("While loading WK projections: " + th.getMessage());
		}
	}

	
	
	public void loadProjectionTypes() throws BundleException {
		
		try {
			// projection types
			addProjection(new MapProjectionAdapter(new TransverseMercatorProjection(), bc));
			addProjection(new MapProjectionAdapter(new MercatorProjection(), bc));
			addProjection(new MapProjectionAdapter(new LambertConformalConicProjection(), bc));
			addProjection(new MapProjectionAdapter(new AlbersProjection(), bc));
		} catch (Throwable th) {
			throw new BundleException("While loading WK projections: " + th.getMessage());
		}
	}
	
	public void loadTransformations() {
		
		// =====================================================================
		// Transformations 
		// =====================================================================
		// =====================================================================
		ITransformationAlgorithm
		t7p = new Trans7Params(bc, edad_ed50_es.getAbrev() + CRSConstants.TOWGS84, edad_ed50_es,
				-131.0320,-100.2510,-163.3540,1.2438,0.0195,1.1436,9.3900);
		addTransformationAlgorithm(t7p);
		// =====================================================================
		t7p = new Trans7Params(bc, edad_ed50_es_w.getAbrev() + "_WGS84", edad_ed50_es_w,
				-178.383, -83.172, -221.293, -0.5401, 0.5319, 0.1263, 21.2);
		addTransformationAlgorithm(t7p);
		// =====================================================================
		t7p = new Trans7Params(bc, edad_ed50_es_cat.getAbrev() + "_WGS84", edad_ed50_es_cat,
				-136.65384, -141.46254, -167.298, 2.093087591, 0.001405488, 0.107708594, 11.545976);
		addTransformationAlgorithm(t7p);
		// =====================================================================
		t7p = new Trans7Params(bc, edad_ed50_es_bal.getAbrev() + "_WGS84", edad_ed50_es_bal,
				-181.4609, -90.2931, -187.1902, -0.1435, -0.4922, 0.3935, 17.57);
		addTransformationAlgorithm(t7p);
//		// =====================================================================
//		t7p = new Trans7Params(bc, edad_ed50_eu.getAbrev() + "_WGS84", edad_ed50_eu,
//				-87, -98, -121, 0, 0, 0, 0);
//		addTransformationAlgorithm(t7p);
//		// =====================================================================
		t7p = new OSMSphericalMercatorTransformation(
				edad_popsph.getAbrev() + "_WGS84",
				edad_popsph.getAbrev(),
				bc);
		addTransformationAlgorithm(t7p);
		// =====================================================================
		t7p = new Trans7Params(bc, edad_ntf.getAbrev() + "_WGS84", edad_ntf,
				-168, -60, 320, 0, 0, 0, 0);
		addTransformationAlgorithm(t7p);
		// =====================================================================
		t7p = new Trans7Params(bc, edad_uk.getAbrev() + "_WGS84", edad_uk,
				// 375, -111, 431, 0, 0, 0, 0);
				// 371, -112, 434, 0, 0, 0, 0);
		// 371, -112, 434 england
				446.448, -125.157, 542.060, -0.1502, -0.2470, -0.8421, -20.4894);
		addTransformationAlgorithm(t7p);

		
//		// =====================================================================
//		t7p = new Trans7Params(bc, edad_mm_e.getAbrev() + "_WGS84", edad_mm_e,
//				-104.1, -49.1, -9.9, -0.971, 2.917, -0.714, 11.68);
//		addTransformationAlgorithm(t7p);
//		// =====================================================================
//		t7p = new Trans7Params(bc, edad_mm_w.getAbrev() + "_WGS84", edad_mm_w,
//				-104.1, -49.1, -9.9, -0.971, 2.917, -0.714, 11.68);
//		addTransformationAlgorithm(t7p);
//		// =====================================================================
//		t7p = new Trans7Params(bc, edad_mm_sa.getAbrev() + "_WGS84", edad_mm_sa,
//				-168.6, -34.0, 38.6, 0.374, 0.679, 1.379, 9.48);
//		addTransformationAlgorithm(t7p);
//		// =====================================================================
//		t7p = new Trans7Params(bc, edad_mm_si.getAbrev() + "_WGS84", edad_mm_si,
//				-50.2, -50.4, 84.8, 0.690, 2.012, -0.459, 28.08);
//		addTransformationAlgorithm(t7p);
//		// =====================================================================
//		t7p = new Trans7Params(bc, edad_usa_nad.getAbrev() + "_WGS84", edad_usa_nad,
//				-8, 160, 176, 0, 0, 0, 0);
//		addTransformationAlgorithm(t7p);
//		// =====================================================================
//		t7p = new Trans7Params(bc, edad_usa_grs.getAbrev() + "_WGS84", edad_usa_grs,
//				-0.991, 1.9072, 0.5129, -0.02579, -0.00965, -0.01166, 0.0);
//		addTransformationAlgorithm(t7p);
//		// =====================================================================
//		t7p = new Trans7Params(bc, edad_jap.getAbrev() + "_WGS84", edad_jap,
//				0, 0, 0, 0, 0, 0, 0);
//		addTransformationAlgorithm(t7p);
//		
//		// =====================================================================
		
	}
}
