package org.gvsig.mobile.gpe;

import java.util.ArrayList;

import org.gvsig.fmap.dal.DALLocator;
import org.gvsig.fmap.dal.DataManager;
import org.gvsig.fmap.dal.feature.impl.DefaultFeatureStore;
import org.gvsig.fmap.dal.store.gpe.GPENewStoreParameters;
import org.gvsig.fmap.dal.store.gpe.GPEStoreProvider;
import org.gvsig.fmap.mapcontext.layers.vectorial.FLyrVect;
import org.gvsig.i18n.Messages;
import org.gvsig.mobile.core.concepts.Functionality;
import org.gvsig.mobile.core.extensions.interfaces.IFileProviderFunctionality;
import org.gvsig.mobile.core.extensions.interfaces.IFileType;
import org.gvsig.mobile.core.interfaces.IGvSigMouseEvent;
import org.gvsig.mobile.core.interfaces.TaskStatusListener;
import org.gvsig.mobile.ui.lw.MessageBox;
import org.gvsig.tools.observer.Observer;
import org.osgi.framework.BundleContext;
import org.osgi.framework.ServiceReference;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class GPEFuncWrite extends Functionality implements IFileProviderFunctionality {

	protected Logger logger = LoggerFactory.getLogger(GPEFuncWrite.class);
	protected String path = null;
	protected static DataManager dm = DALLocator.getDataManager();

	private ArrayList listeners = new ArrayList();
	
	private boolean must_stop = false;

	public static String ID = "GPEFuncWriteID";

	public GPEFuncWrite(BundleContext context) {
		super(ID, context);
	}
	
	public void cancel() {
		must_stop = true;
	}

	public String getFilePath() {
		return path;
	}

	public void setFilePath(String filePath) {
		path = filePath;
	}



	public void configure() { }

	public boolean execution(ServiceReference gvSIGMobileRef) {
		run();
		return true;
	}

	public void onNewMouseEventListened(IGvSigMouseEvent event) {
		// TODO Auto-generated method stub
		
	}
	
	public void run() {
		// =======================================
		must_stop = false;
		notifyStart();
		// =======================================
		
		
		if (dm == null)
			dm = DALLocator.getDataManager();
		
		GPENewStoreParameters nfsp = new GPENewStoreParameters();
		DefaultFeatureStore sourceStore = null;;
		try {
			FLyrVect lyr = (FLyrVect) getProps().get(FLyrVect.class.getName());
			if (lyr == null) {
				logger.error("No layer found in properties.");
			} else {
				sourceStore = (DefaultFeatureStore) lyr.getFeatureStore();
				// sourceStore.enableNotifications();
			}

			long count = sourceStore.getFeatureCount();
			nfsp.setFutureShapeCount(count);
			nfsp.setCRS(lyr.getProjection());
			nfsp.setFileName(path);
			nfsp.setDefaultFeatureType(sourceStore.getDefaultFeatureType());
			
			for (int i=0; i<listeners.size(); i++) {
				Object o = listeners.get(i); 
				if (o instanceof Observer) {
					nfsp.setObserver((Observer) o);
				}
			}

			GPEStoreProvider newstoreprov = new GPEStoreProvider(nfsp, sourceStore);
			newstoreprov.export(sourceStore, this.fileType);
			
		} catch (Throwable ex) {
			System.err.println("While writing GPE file: " + ex.getMessage());
			logger.error("While writing GPE file: " + ex.getMessage());

			MessageBox.showDialog(true, bc,
					Messages.getText("Error_while_writing file") + ": " + ex.getMessage(),
					null, MessageBox.MESSAGE_TYPE_ERROR, MessageBox.MESSAGE_ONLY_OK);
		}

		// =======================================
		if (!must_stop) {
			notifyEnd();
		}
		// =======================================
	}
	
	
	
	public TaskStatusListener addObserver(TaskStatusListener obs) {
		
		if (!listeners.contains(obs)) {
			listeners.add(obs);
		}
		return obs;
	}

	public boolean isActive() {
		return true;
	}

	public void removeAllObservers() {
		listeners.clear();
	}

	public TaskStatusListener removeObserver(TaskStatusListener obs) {
		if (listeners.remove(obs)) {
			return obs;
		} else {
			return null;
		}
	}
	
	private void notifyEnd() {
		int sz = listeners.size();
		for (int i=0; i<sz; i++) {
			((TaskStatusListener) listeners.get(i)).taskFinished(this);
		}
	}

	private void notifyStart() {
		int sz = listeners.size();
		for (int i=0; i<sz; i++) {
			((TaskStatusListener) listeners.get(i)).taskInited(this);
		}
	}
	
	private IFileType fileType;

	public IFileType getFileType() {
		return fileType;
	}

	public void setFileType(IFileType ft) {
		fileType = ft;
	}


}
