package org.gvsig.mobile.tool;

import java.util.ArrayList;
import java.util.Hashtable;
import java.util.Iterator;

import org.gvsig.mobile.core.exceptions.GvSigPropertyNotAttachedException;
import org.gvsig.mobile.core.extensions.interfaces.IFunctionality;
import org.gvsig.mobile.core.extensions.interfaces.ITask;
import org.gvsig.mobile.core.extensions.interfaces.ITool;
import org.gvsig.mobile.core.interfaces.TaskStatusListener;

/**
 * It listen the state of all tools to avoid two exclusive tools running at the
 * same time
 * 
 * @author jcarras
 * 
 */
public class ToolPool implements TaskStatusListener {

	Hashtable tools = new Hashtable();
	ArrayList exclusivetools = new ArrayList();
	ITool activeExclusiveTool = null;

	public ITool getActiveExclusiveTool() {
		return activeExclusiveTool;
	}

	public ToolPool() {
	}

	public void addTool(ITool tool) {
		try {
			if (!tools.containsKey(tool.getFunctionalityID())) {

				IFunctionality f = tool.getFuncSolved();
				if (f != null)
					f.addObserver(this);

				tools.put(tool.getFunctionalityID(), tool);
			
				if (tool.getToolType() == ITool.EXCLUSIVE_TOOL
						&& !exclusivetools.contains(tool)) {
					exclusivetools.add(tool);					
				}	
			}
		} catch (GvSigPropertyNotAttachedException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	public ITool removeTool(String key) {
		ITool tool = (ITool) tools.get(key);
		try {
			if (tool.getToolType() == ITool.EXCLUSIVE_TOOL
					&& exclusivetools.contains(tool))
				exclusivetools.remove(tool);
		} catch (GvSigPropertyNotAttachedException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		tools.remove(key);
		return tool;
	}

	public ITool getTool(String key) {
		return (ITool) tools.get(key);
	}

	public void taskFinished(ITask task) {
		if (task instanceof IFunctionality) {
			ITool tool = (ITool) tools.get(((IFunctionality) task).getID());
			if (activeExclusiveTool == tool)
				activeExclusiveTool = null;
				
		}
	}

	public void taskInited(ITask task) {
		if (task instanceof IFunctionality) {
			ITool tool = (ITool) tools.get(((IFunctionality) task).getID());
			try {				
				
				// Run immediately tools must end as soon as they have been
				// executed
				switch (tool.getToolType()) {
				case ITool.EXCLUSIVE_TOOL:
					if (activeExclusiveTool != tool) {
						activeExclusiveTool = tool;
						stopExclusiveTools(tool);
					}
					break;
				case ITool.RUN_INMEDIATELY_TOOL:
					task.stop();
					break;
				case ITool.STATE_TOOL:		
						
					
					break;
				}
			} catch (GvSigPropertyNotAttachedException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}
	}

	private void stopExclusiveTools(ITool exception) {
		Iterator it = exclusivetools.iterator();
		while (it.hasNext()) {
			ITool tool = (ITool) it.next();
			if (tool.getFuncSolved() != null)
				if (tool != exception && tool.getFuncSolved().isActive())
					tool.getFuncSolved().stop();
		}
	}

	public void taskProgress(ITask task, int progress) {

	}
	
	public boolean hasTools(String toolbarKey) {
		
		Iterator it = tools.keySet().iterator();
		
		ITool t;
		while(it.hasNext()) {
			t = (ITool)tools.get(it.next());
			try {
				if (t.getToolbarKey().compareTo(toolbarKey) == 0) return true;
			} catch (GvSigPropertyNotAttachedException e) {
				// TODO Auto-generated catch block
//				e.printStackTrace();
			}
		}
		return false;
	}
}
