package org.gvsig.mobile.ui.awt;

import java.awt.Container;
import java.awt.GraphicsEnvironment;
import java.awt.Image;
import java.awt.MediaTracker;
import java.awt.Toolkit;
import java.awt.image.BufferedImage;
import java.io.DataInputStream;
import java.io.File;
import java.net.MalformedURLException;
import java.net.URL;

import org.osgi.framework.Bundle;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class ImageUtils {

	static private Logger logger = LoggerFactory.getLogger(ImageUtils.class);

	public static MediaTracker mt = new MediaTracker(new Container());
	public static int imageCount = 0;
	public static String appDir = "";
	public static final int IMAGE_LOAD_TIMEOUT = 1000;
	public static String IMG_SKIN_PREFIX = "img24";
	public static String CORE_FOLDER = "";
	public static String CORE_VERSION = "";	

	public static void setAppDir(String str) {
		appDir = str;
	}
	public static String getAppDir(){
		return appDir;
	}

	//
	public static Image getResourceImage(String imgFileName) {
		
		// System.out.println("========== GET ICON 1: " + imgFileName);
		return _getResourceImage(CORE_FOLDER+"/", imgFileName, true, CORE_FOLDER, CORE_VERSION);		
	}




	public static Image getResourceImage(Bundle bundle, String imgFileName) {

		// System.out.println("========== GET ICON 2: " + imgFileName);
		String location = ImageUtils.parseBundleLocation(bundle.getLocation());
		String name = bundle.getSymbolicName();
		String version = bundle.getHeaders().get("Bundle-Version").toString();

		return _getResourceImage(location, imgFileName, false, name, version);		
	}
	
	public static File getBundleFile(Bundle bundle, String filename) {
		String location = ImageUtils.parseBundleLocation(bundle.getLocation());
		String name = bundle.getSymbolicName();
		String version = bundle.getHeaders().get("Bundle-Version").toString();

		return _getBundleFile(location, filename);		
	}
	
	private static File _getBundleFile(String location, String fileName) {
		
		String rootDir = appDir; 
		
		String path = new StringBuffer()
		.append(rootDir).append(location)
		.append("/").append(fileName).toString();
		
		File resp = new File(path);
		if (!resp.exists()) {
			logger.warn("Requested file does not exist: " + path);
		}
		return resp;
	}
	
	public static byte[] getFileInIconsFile(Bundle bundle, String jarsuffix, String fullFileName) {

		String location = ImageUtils.parseBundleLocation(bundle.getLocation());
		String name = bundle.getSymbolicName();
		String version = bundle.getHeaders().get("Bundle-Version").toString();

		return _getResourceFile(location, jarsuffix, fullFileName, name, version);		
	}

	
	private static Image _getResourceImage(String location, String fileName, boolean root) {
		URL imgf = null;
		
		String rootDir = root ? appDir : ""; 
		
		String url = new StringBuffer().append("jar:file:")
		.append(rootDir).append(location)
		.append("icons.jar!").append("/").append("img")
		.append("/").append(IMG_SKIN_PREFIX).append(
				"/").append(fileName).append(".png")
		.toString();
		
//		System.out.println(url);

		try {
			imgf = new URL(url);
		} catch (MalformedURLException e) {
			logger.error("While getting image: " + e.getMessage());
		}

		try {
			Image img = Toolkit.getDefaultToolkit().createImage(imgf);
			mt.addImage(img, imageCount);
			mt.waitForID(imageCount, IMAGE_LOAD_TIMEOUT);
			mt.removeImage(img);

			return img;
		} catch (Throwable th) {
			logger.error("While getting downloaded image: " + th.getMessage());
			return null;
		}
	}
	
	private static Image _getResourceImage(String location, String fileName, boolean root, String name, String version) {
		URL imgf = null;
		
		String rootDir = root ? appDir : ""; 
		
		String url = new StringBuffer().append("jar:file:")
		.append(rootDir).append(location).append(name).append("-").append(version).append("-")
		.append("icons.jar!").append("/").append("img")
		.append("/").append(IMG_SKIN_PREFIX).append(
				"/").append(fileName).append(".png")
		.toString();
		
		if (url.indexOf("edition_gps") != -1) {
			System.out.println("====== ICON URL = " + url);
		}

		try {
			imgf = new URL(url);
		} catch (MalformedURLException e) {
			logger.error("While getting image: " + e.getMessage());
			return null;
		}

		try {
			Image img = Toolkit.getDefaultToolkit().createImage(imgf);
			mt.addImage(img, imageCount);
			mt.waitForID(imageCount, IMAGE_LOAD_TIMEOUT);
			mt.removeImage(img);
			return img;
		} catch (Throwable th) {
			logger.error("While getting downloaded image: " + th.getMessage());
			return null;
		}
	}

	private static byte[] _getResourceFile(
			String location,
			String jarsuffix,
			String fullFileName,
			String name,
			String version) {

		String url = new StringBuffer().append("jar:file:")
		.append(location).append(name).append("-").append(version).append("-")
		.append(jarsuffix).append(".jar!").append("/").append(fullFileName)
		.toString();

		byte[] acum = new byte[4097];
		byte[] resp = null;

		try {
			URL furl = new URL(url);
			DataInputStream is = new DataInputStream(furl.openStream());
			int sz = is.read(acum);
			if (sz == acum.length) {
				logger.error("Res file too long (max 4096 bytes): " + sz);
				is.close();
				return acum;
			} else {
				resp = new byte[sz];
				System.arraycopy(acum, 0, resp, 0, sz);
				is.close();
				return resp;
			}
			

		} catch (Exception e) {
			logger.error("While getting image: " + e.getMessage());
		}
		return new byte[0];
	}
	
	
	/*
	 * public static Image createImage(String path){ return
	 * Toolkit.getDefaultToolkit().getImage(path); }
	 */

	public static BufferedImage createImage(int w, int h) {
		return GraphicsEnvironment.getLocalGraphicsEnvironment()
				.getDefaultScreenDevice().getDefaultConfiguration()
				.createCompatibleImage(w, h);
	}
	
	public static String parseBundleLocation(String location) {
		//TODO -> ALBERTO: poner este mtodo en una clase 'Utilities'
		int index = 0;
		
		while ( index != -1 ) {		
			index = location.indexOf(':');	
			location = location.substring(index+1);
		}
		
//		System.out.println(location);		
		
		if (location.charAt(location.length()-1) != File.pathSeparatorChar) {
			location = location.substring(0, location.lastIndexOf("/"));
//			System.out.println(location);
//			location = location.substring(0, location.length()-4);
			location += "/";
		}
		
		String os = System.getProperty("os.name");		
		
		if (location.startsWith("../..") && !os.toUpperCase().startsWith("WIN")) {
			location = location.substring(5);
		}
		
		return location;
	}
}
