package org.gvsig.mobile.ui.awt;

import java.awt.Dimension;
import java.awt.Insets;

import org.gvsig.mobile.ui.lw.ISelfCloseListener;
import org.gvsig.mobile.ui.lw.UIConstants;
import org.zaval.lw.Layoutable;
import org.zaval.lw.LwBorderLayout;
import org.zaval.lw.LwComponent;
import org.zaval.lw.LwDesktop;
import org.zaval.lw.LwFrame;
import org.zaval.lw.LwLayer;
import org.zaval.lw.LwPanel;
import org.zaval.lw.LwToolkit;
import org.zaval.lw.LwWinLayer;
import org.zaval.lw.LwWindow;
import org.zaval.lw.event.LwKeyEvent;
import org.zaval.lw.event.LwKeyListener;
import org.zaval.lw.event.LwMouseEvent;
import org.zaval.lw.event.LwWinListener;

public class WindowManager implements LwKeyListener, LwWinListener {

	private static WindowManager instance;
	private LwFrame mainFrame;

	public static WindowManager getInstance() {
		if (instance == null) {
			instance = new WindowManager();
		}
		return instance;
	}

	private WindowManager() {
		
	}
	
	public Dimension getMainSize() {
		if (mainFrame == null) {
			return null;
		} else {
			return mainFrame.getSize();
		}
	}

	public void showMain(LwPanel panel) {
		mainFrame = new LwFrame();
		mainFrame.getRoot().setLwLayout(new LwBorderLayout());
		// mainFrame.getRoot().addKeyListener(this);
		// mainFrame.getRoot().addWindowListener(this);
		mainFrame.getRoot().add(LwBorderLayout.CENTER, panel);

		// mainFrame.setState(LwFrame.MAXIMIZED_BOTH);

		int w_aux = Math.min(480, UIConstants.SCREEN_SIZE.width); 
		int h_aux = UIConstants.SCREEN_SIZE.height; 
		
		int vert_skip = 26 * w_aux / 240;
		
		mainFrame.setSize(w_aux, h_aux - vert_skip);
		
//		mainFrame.setSize(240, 320);
		
		mainFrame.setVisible(true);
//		mainFrame.getRoot().setOpaque(false);
	}

	public void show(LwPanel panel, String title) {
		LwDesktop desktop = LwToolkit.getDesktop(mainFrame.getRoot());

		LwWinLayer winLayer = (LwWinLayer) desktop.getLayer(LwWinLayer.ID);
//		winLayer.removeAll();
//		TBMenuManager.getInstance().mouseExited(null);
		LwWindow win1 = new LwWindow(title) {
			public void startDragged(LwMouseEvent e) {
				
			}
			
			public void endDragged(LwMouseEvent e) {
				
			}
			
			public void mouseDragged(LwMouseEvent e) {
			    
			}
		};

		int locx = (mainFrame.getWidth() - win1.getWidth()) / 2;
		int locy = (mainFrame.getHeight() - win1.getHeight()) / 2;

		if (panel instanceof IWindowInfo) {
			IWindowInfo info = (IWindowInfo) panel;
			if (info.isMaximized()) {
				Dimension d = mainFrame.getSize();
				Insets ins = mainFrame.getInsets();

				win1.setSize(d.width - ins.left - ins.right, d.height - ins.top
						- ins.bottom);
				locx = 0;
				locy = 0;
				win1.setSizeable(false);
			} else {
				Dimension d = info.getDesiredSize();
				win1.setSize(d.width, d.height);
				locx = (mainFrame.getWidth() - win1.getWidth()) / 2;
				locy = (mainFrame.getHeight() - win1.getHeight()) / 2;
				win1.setSizeable(true);
			}
		} else {
			Dimension d = panel.getPreferredSize();
			win1.setSize(d.width, d.height);
			locx = (mainFrame.getWidth() - win1.getWidth()) / 2;
			locy = (mainFrame.getHeight() - win1.getHeight()) / 2;
		}
		

		
		win1.getRoot().setLwLayout(new LwBorderLayout());
		win1.getRoot().add(LwBorderLayout.CENTER, panel);
		win1.setLocation(locx, locy);	
		

//		win1.setBackground(Color.WHITE);
//		 panel.getViewMan(true).setBg(new LwImgRender(ImageUtils
//		 .getResourceImage("star_bg"), LwView.STRETCH));
//		 panel.setOpaque(false);
		// winLayer.setOpaque(false);
		// win1.setOpaque(false);
//		winLayer.setBackground(Color.WHITE);
//		 winLayer.setOpaque(false);

		winLayer.add(win1, this, LwWinLayer.MODAL_WIN);

	}

	/**
	 * Closes active panel of the current LwWinLayer. Use this method to close
	 * modal panels from a button key pressed event or similar.
	 */
	public void closeActivePanel() {
		final LwDesktop desktop = LwToolkit.getDesktop(mainFrame.getRoot());
		final LwWinLayer winLayer = (LwWinLayer) desktop
				.getLayer(LwWinLayer.ID);
		final LwWindow active = ((LwWindow) (winLayer.getActive()));
		try {
			appWindowClosing(active);
			winLayer.remove(active);
		} catch (Exception e) {
			System.err.println("Nothing to close: " + e.getMessage());
		}
	}
	
	public void closeActiveUnlessMain() {
		final LwDesktop desktop = LwToolkit.getDesktop(mainFrame.getRoot());
		final LwWinLayer winLayer = (LwWinLayer) desktop
				.getLayer(LwWinLayer.ID);
		final LwWindow active = ((LwWindow) (winLayer.getActive()));
		
		final LwComponent aux = active;
		if (aux == mainFrame || aux == null) {
			// do not close main
			return;
		}
		
		try {
			appWindowClosing(active);
			winLayer.remove(active);
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	public void close(LwComponent arg0) {
		arg0.setVisible(false);
		if (arg0 instanceof LwWindow) {
			final LwDesktop desktop = LwToolkit.getDesktop(mainFrame.getRoot());
			final LwWinLayer winLayer = (LwWinLayer) desktop
					.getLayer(LwWinLayer.ID);
			LwWindow win = (LwWindow) arg0;
			try {
				winLayer.remove(win);
			} catch (Exception e) {
				e.printStackTrace();
			}
		}
		
		if (arg0 == mainFrame) {
			System.exit(0);
		}

	}



	public void appWindowClosing(LwWindow win) {
		
		Layoutable lyt = win.getRoot().get(0);
		if (lyt != null && lyt instanceof ISelfCloseListener) {
			ISelfCloseListener iscl = (ISelfCloseListener) lyt;
			iscl.closeEventFollows();
		}
	}

	// public void winDeactivated(LwLayer arg0, LwComponent arg1) { }
	// public void winOpened(LwLayer arg0, LwComponent arg1) {	}
	// public void winActivated(LwLayer arg0, LwComponent arg1) { }

	public void keyPressed(LwKeyEvent arg0) {
		if (arg0.getKeyCode() == 11) 
		close(arg0.getLwComponent());
	}

	public void keyReleased(LwKeyEvent arg0) { }
	public void keyTyped(LwKeyEvent arg0) { }

	public void winActivated(LwLayer arg0, LwComponent arg1) { }
	public void winClosed(LwLayer arg0, LwComponent arg1) { }
	public void winDeactivated(LwLayer arg0, LwComponent arg1) { }
	public void winOpened(LwLayer arg0, LwComponent arg1) { }

}
