package org.gvsig.mobile.ui.awt.tools;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Frame;
import java.awt.Graphics;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.Vector;

import org.gvsig.mobile.core.auxfunctions.ShowMessage;
import org.gvsig.mobile.core.auxfunctions.StringConstructor;
import org.gvsig.mobile.core.exceptions.GvSigPropertyNotAttachedException;
import org.gvsig.mobile.core.extensions.interfaces.ITool;
import org.zaval.lw.LayoutContainer;
import org.zaval.lw.Layoutable;
import org.zaval.lw.LwLayout;
import org.zaval.lw.LwPanel;

/**
 * It's a group of toolwrappers
 * 
 * @author jcarras
 * 
 */
public class ToolBar extends LwPanel {

	public static final int ALIGNEMENT_HORIZONTAL = 0;
	public static final int ALIGNEMENT_VERTICAL = 1;

	Color color = new Color(0, 0, 0, 255);
	Frame parent;
	int nextX = 0;
	Hashtable tools = new Hashtable();
	Vector order = new Vector();	

	String key = null;

	// int prefw = 0;
	// int prefh = 0;

	int alignement = ALIGNEMENT_HORIZONTAL;

	public ToolCombo helpComponent;	

	public ToolBar(String key, int alignement) {
		this.alignement = alignement;
		setLwLayout(new TBLayout(alignement));
		this.key = key;
		setOpaque(true);
	}

	public String getKey() {
		return key;
	}

	public void setKey(String key) {
		this.key = key;
	}

	public void paint(Graphics g) {
		g.setColor(color);
		if (alignement == ALIGNEMENT_HORIZONTAL) {
			// int width = (order.size() - 1) *
			// ((ToolComponent)order.get(0)).img.getWidth(null);

			g.fillRect(getPreferredSize().width, 0, getSize().width,
					getPreferredSize().height);
			// System.out.println(getPreferredSize().width +", " + 0+", " +
			// getSize().width+", " + getPreferredSize().height);
		} else {
			g.fillRect(0, getPreferredSize().height, getPreferredSize().width,
					getSize().height);
		}
		super.paint(g);
	}

	public ToolComponent getToolComponentAt(int position) {
		int size = order.size();

		ToolComponent temp = null;
		for (int i = 0; i < size; i++) {
			temp = (ToolComponent) order.get(i);
			if (temp.getTool().getPosition() == position
					&& !(temp instanceof DropDownComponent)) {
				return temp;
			}
		}
		return null;
	}

	// public ToolComponent getToolComponentAt(String position) {
	// return (ToolComponent)orderedTools.get(position);
	// }

	public void addTool(ToolComponent toolC) {

		// this.addComboTools((ComboTool) tool);

		ITool tool = toolC.getTool();
		try {
			if (!tools.containsKey(tool.getKey())) {
				tools.put(tool.getKey(), toolC);
				order.add(toolC);
				// orderedTools.put(String.valueOf(toolC.getTool().getPosition()),
				// toolC);
				add(toolC);

				// incrementPrefSize(toolC);
			}
		} catch (GvSigPropertyNotAttachedException e) {
			e.printStackTrace();
		}
	}

	public ToolComponent removeTool(ToolComponent tool) {
		// ToolComponent tool = (ToolComponent)tools.get(key);
		try {
			tools.remove(tool.getTool().getKey());
		} catch (GvSigPropertyNotAttachedException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		order.remove(tool);
		// orderedTools.remove(String.valueOf(tool.getTool().getPosition()));
		remove(tool);
		return tool;
	}

	public void removeTool(String key) {
		// ToolComponent tool = (ToolComponent)tools.get(key);
		try {
			ToolComponent toolC = (ToolComponent) tools.get(key);
			tools.remove(key);

			if (toolC != null) {
				order.remove(toolC);
				remove(toolC);
			}

			this.getLwLayout().layout(this);
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	public Iterator getTools() {
		return order.iterator();
	}

	public int getAlignment() {
		return alignement;
	}

	public void setAlignment(int alignment) {
		this.alignement = alignment;
	}

	public boolean hasTools() {
		if (tools != null)
			return tools.size() != 0;
		return false;
	}

	// public void paint(Graphics g){
	// g.setColor(Color.yellow);
	// g.fillRect(0, 0, getWidth(), getHeight());
	// super.paint(g);
	// }

	class TBLayout implements LwLayout {
		int alignement = ALIGNEMENT_HORIZONTAL;
		int prX = 0;
		int prY = y;

		public TBLayout(int alignement) {
			this.alignement = alignement;
		}

		public void componentAdded(Object id, Layoutable lw, int index) {

			if (lw instanceof DropDownComponent
					&& ((DropDownComponent) lw).isHelpButton)
				return;

			if (alignement == ALIGNEMENT_HORIZONTAL) {
				if (lw.getHeight() > prY)
					prY = lw.getHeight();
				prX += lw.getWidth();
			} else {
				if (lw.getWidth() > prX)
					prX = lw.getWidth();
				prY += lw.getHeight();
			}
			// System.out.println(prX + ", " + prY);
			// setPSSize(prX, prY);
			// setSize(prX, prY);
			ShowMessage.Show(StringConstructor.Construct(new String[] {
					String.valueOf(prX), ", ", String.valueOf(prY) }), true);
		}

		public void componentRemoved(Layoutable lw, int index) {
			
			if (alignement == ALIGNEMENT_HORIZONTAL) {
				prX -= lw.getWidth();
			} else {
				prY -= lw.getHeight();
			}
		}

		public Dimension calcPreferredSize(LayoutContainer target) {
			int w = prX;
			int h = prY;
			return new Dimension(w, h);
		}

		public Vector orderTools() {			
			Vector temp = (Vector) order.clone();
			Hashtable positions = new Hashtable();
			Vector values = new Vector();

			final int size = order.size();

			ToolComponent curr = null;
			DropDownComponent drop = null;
			ToolComponent tc = null;

			for (int i = 0; i < size; i++) {
				curr = (ToolComponent) order.get(i);
				// if the toolbar contains toolbarbuttons do nothing
				if (curr instanceof ToolBarButton) {
					break;
					// help button is always at last position
				} else if (curr instanceof DropDownComponent) {
					temp.remove(curr);
					temp.insertElementAt(curr, temp.size());
				} else {
					if (curr instanceof ToolCombo) {
						positions
								.put(new Integer(curr.getTool().getPosition()),
										curr);
						values.add(new Integer(curr.getTool().getPosition()));
					}
					temp.remove(curr);
					int pos = curr.getTool().getPosition() >= temp.size() ? temp
							.size() - 1
							: curr.getTool().getPosition();
					pos = pos < 0 ? 0 : pos;
					temp.insertElementAt(curr, pos);
				}
			}

			int j = values.size() - 1;
			for (int i = temp.size() - 1; i >= 0; i--) {
				curr = (ToolComponent) temp.get(i);

				if (curr instanceof DropDownComponent) {
					if (!((DropDownComponent) curr).isHelpButton) {
						if (j >= 0) {
							Integer n = (Integer) values.get(j);
							if (n.intValue() >= 0) {
								tc = (ToolComponent) positions
										.get((Integer) values.get(j));
								int pos = temp.indexOf(tc);
								temp.remove(i);
								temp.insertElementAt(curr, pos + 1);
								j--;
								i++;
							}
						}
					}
				} else {
					break;
				}
			}			 

			// order = temp;			
			return temp;
			
		}

		// TODO: ALBERTO -> optimizar este mtodo
		// public Vector orderTools() {
		// System.out.println("order tools");
		// Vector temp = (Vector) order.clone();
		//
		// final int size = order.size();
		//			
		// ToolComponent curr = null;
		// DropDownComponent drop = null;
		// ToolComponent tc = null;
		//			
		// for (int i = 0; i < size; i++) {
		// curr = (ToolComponent) order.get(i);
		// //if the toolbar contains toolbarbuttons do nothing
		// if (curr instanceof ToolBarButton) {
		// break;
		// //help button is always at last position
		// } else if (curr instanceof DropDownComponent
		// && ((DropDownComponent) curr).isHelpButton) {
		// temp.remove(curr);
		// temp.insertElementAt(curr, temp.size());
		// } else if (curr.getTool().getPosition() >= 0) {
		// int index = temp.indexOf(curr);
		// temp.remove(curr);
		// // toolcombo
		// if (curr instanceof ToolCombo) {
		// // we remove the DropDownComponent of the ToolCombo
		// // so !(curr instanceof DropDownComponent) is always
		// // true
		//						
		// while(!(temp.get(index) instanceof DropDownComponent)) {
		// if(index > temp.size() - 1) index = -1;
		// index++;
		// }
		// drop = (DropDownComponent) temp.get(index);
		// temp.remove(index);
		// i += 1;
		//
		// index = curr.getTool().getPosition() < temp.size() ? curr
		// .getTool().getPosition()
		// : temp.size() - 1;
		// index = index < 0 ? 0 : index;
		// try {
		// tc = (ToolComponent) temp.get(index);
		// } catch (Exception e){
		// System.out.println("error while ordering tools");
		// }
		//						
		// if(tc != null)
		//
		// if (tc instanceof DropDownComponent
		// && !((DropDownComponent) tc).isHelpButton) {
		//							
		// index = curr.getTool().getPosition() - 1 <= temp
		// .size() ? curr.getTool().getPosition() - 1
		// : temp.size() - 1;
		// index = index < 0 ? 0 : index;
		// tc = (ToolComponent) temp.get(index);
		// if (curr.getTool().getPosition() < tc.getTool()
		// .getPosition()) {
		// int pos = curr.getTool().getPosition() - 1 >= temp
		// .size() ? temp.size()-1 : curr.getTool()
		// .getPosition() - 1;
		// temp.insertElementAt(drop, pos);
		// temp.insertElementAt(curr, pos);
		// } else {
		// int pos = curr.getTool().getPosition() + 1 >= temp
		// .size() ? temp.size()-1 : curr.getTool()
		// .getPosition() + 1;
		// temp.insertElementAt(drop, pos);
		// temp.insertElementAt(curr, pos);
		// }
		// } else if (tc instanceof ToolCombo) {
		// if (curr.getTool().getPosition() < tc.getTool()
		// .getPosition()) {
		// int pos = curr.getTool().getPosition() >= temp
		// .size() ? temp.size()-1 : curr.getTool()
		// .getPosition();
		// temp.insertElementAt(drop, pos);
		// temp.insertElementAt(curr, pos);
		// } else {
		// int pos = curr.getTool().getPosition() + 2 >= temp
		// .size() ? temp.size()-1 : curr.getTool()
		// .getPosition() + 2;
		// temp.insertElementAt(drop, pos);
		// temp.insertElementAt(curr, pos);
		// }
		// } else {
		// if (curr.getTool().getPosition() < tc.getTool()
		// .getPosition()) {
		// int pos = curr.getTool().getPosition() >= temp
		// .size() ? temp.size()-1 : curr.getTool()
		// .getPosition();
		// temp.insertElementAt(drop, pos);
		// temp.insertElementAt(curr, pos);
		// } else {
		// int pos = curr.getTool().getPosition() + 1 >= temp
		// .size() ? temp.size()-1 : curr.getTool()
		// .getPosition() + 1;
		// temp.insertElementAt(drop, pos);
		// temp.insertElementAt(curr, pos);
		// }
		// }
		// // a toolcomponent
		// } else {
		// // we get the component at the position we want to
		// // insert
		// index = curr.getTool().getPosition() < temp
		// .size() ? curr.getTool().getPosition()
		// : temp.size() - 1;
		// index = index < 0 ? 0 : index;
		// try {
		// tc = (ToolComponent) temp.get(index);
		// } catch (Exception e) {
		// System.out.println("error while ordering tools");
		// }
		//						
		// if(tc != null)
		//						
		// // if it's a dropdowncomponent we get the previous
		// // component (toolcombo)
		// if (tc instanceof DropDownComponent
		// && !((DropDownComponent) tc).isHelpButton) {
		//
		// index = curr.getTool().getPosition() - 1 <= temp
		// .size() ? curr.getTool().getPosition() - 1
		// : temp.size() - 1;
		// index = index < 0 ? 0 : index;
		// tc = (ToolComponent) temp.get(index);
		// // put the new component before
		// if (curr.getTool().getPosition() < tc.getTool()
		// .getPosition()) {
		// int pos = curr.getTool().getPosition() >= temp
		// .size() ? temp.size()-1 : curr.getTool()
		// .getPosition();
		// temp.insertElementAt(curr, pos);
		// // put the new component after the
		// // dropdowncomponent
		// } else {
		// int pos = curr.getTool().getPosition() + 1 >= temp
		// .size() ? temp.size()-1 : curr.getTool()
		// .getPosition() + 1;
		// temp.insertElementAt(curr, pos);
		// }
		// // if it's a toolcombo
		// } else if (tc instanceof ToolCombo) {
		// // put the new component before
		// if (curr.getTool().getPosition() < tc.getTool()
		// .getPosition()) {
		// int pos = curr.getTool().getPosition() >= temp
		// .size() ? temp.size()-1 : curr.getTool()
		// .getPosition();
		// temp.insertElementAt(curr, pos);
		// // put the new component after the
		// // dropdowncomponent
		// } else {
		// int pos = curr.getTool().getPosition() + 2 >= temp
		// .size() ? temp.size()-1 : curr.getTool()
		// .getPosition() + 2;
		// temp.insertElementAt(curr, pos);
		// }
		// // if it's a toolcomponent
		// } else {
		// if (curr.getTool().getPosition() < tc.getTool()
		// .getPosition()) {
		// int pos = curr.getTool().getPosition() >= temp
		// .size() ? temp.size()-1 : curr.getTool()
		// .getPosition();
		// temp.insertElementAt(curr, pos);
		// } else {
		// int pos = curr.getTool().getPosition() + 1 >= temp
		// .size() ? temp.size()-1 : curr.getTool()
		// .getPosition() + 1;
		// temp.insertElementAt(curr, pos);
		// }
		// }
		// }
		// }
		// }
		//
		// // if(helpComponent != null)
		// // helpComponent.setTools(temp);
		//			
		// order = temp;
		// return temp;
		// }

		public void layout(LayoutContainer target) {
			// locate all components
			int currX = getInsets().left;
			int currY = getInsets().top;
			// System.out.println("---------");
			// Iterator it = this.orderRecursivelly(order).iterator();
			Iterator it = orderTools().iterator();
			// Iterator it = order.iterator();

			while (it.hasNext()) {
				ToolComponent tc = (ToolComponent) it.next();
				// System.out.println(tc.getTool().getKey());

				// Help tool is located at the end of the toolBar
				if (tc instanceof DropDownComponent
						&& ((DropDownComponent) tc).isHelpButton) {
					tc.setLocation(getSize().width - getInsets().right
							- tc.getWidth(), currY);
				} else {
					tc.setLocation(currX, currY);
					if (alignement == ALIGNEMENT_HORIZONTAL) {
						currX += tc.getWidth();
					} else {
						if (tools.size() != 0)
							currY += tc.getHeight();
						else
							tc.setVisible(false);
					}
				}
			}
		}

	}
}
