/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2006 Prodevelop and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *   Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *   +34 963862235
 *   gvsig@gva.es
 *   http://www.gvsig.gva.es
 *
 *    or
 *
 *   Prodevelop Integracin de Tecnologas SL
 *   Conde Salvatierra de lava , 34-10
 *   46004 Valencia
 *   Spain
 *
 *   +34 963 510 612
 *   +34 963 510 968
 *   gis@prodevelop.es
 *   http://www.prodevelop.es
 *
 *    or
 *
 *   Instituto de Robtica
 *   Apartado de correos 2085
 *   46071 Valencia
 *   (Spain)
 *   
 *   +34 963 543 577
 *   jjordan@robotica.uv.es
 *   http://robotica.uv.es
 *   
 */
package org.gvsig.mobile.ui.awt.tools;

import java.awt.Graphics;
import java.util.Vector;

import org.gvsig.mobile.core.concepts.ComboTool;
import org.gvsig.mobile.core.concepts.DefaultToolBarFunctionality;
import org.gvsig.mobile.core.concepts.SwitcherTool;
import org.gvsig.mobile.core.exceptions.GvSigPropertyNotAttachedException;
import org.gvsig.mobile.core.extensions.interfaces.IFunctionality;
import org.gvsig.mobile.core.extensions.interfaces.ITask;
import org.gvsig.mobile.core.extensions.interfaces.ITool;
import org.gvsig.mobile.core.interfaces.TaskStatusListener;
import org.gvsig.mobile.ui.awt.ImageUtils;
import org.osgi.service.wireadmin.Wire;
import org.zaval.lw.LwPanel;

public class ToolCombo extends ToolComponent implements TaskStatusListener {

	private Vector tools;
	private int index = 0;
	private DropDownComponent drop;
	protected boolean isHelpButton = false;	

	public ToolCombo(ITool comboTool, LwPanel mainFrame, boolean isHelpButton) {
		super(((org.gvsig.mobile.core.concepts.ComboTool) comboTool)
				.getToolAt(0));		

		this.isHelpButton = isHelpButton;
		SwitcherTool tool = new SwitcherTool();

		DefaultToolBarFunctionality f = null;
		try {
			f = new DefaultToolBarFunctionality(
					comboTool.getKey()+"_DROP",
					comboTool.getBundleContext());
		} catch (GvSigPropertyNotAttachedException e2) {
			// TODO Auto-generated catch block
			e2.printStackTrace();
		}
		
		tool.setFunctionalityID(f.getID());		
		
		tool.setBundleContext(comboTool.getBundleContext());
		
		try {
			tool.setKey(comboTool.getKey()+"_DROP");
		} catch (GvSigPropertyNotAttachedException e1) {
			// TODO Auto-generated catch block
			e1.printStackTrace();
		}
		
		if (isHelpButton) {
			try {
				tool.setKey(comboTool.getKey()+ "_HELP_");
			} catch (GvSigPropertyNotAttachedException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
			tool.setIcon("menutoolbar_dropdown_black");
		} else {
			tool.setIcon("menu_combo_down_black");
		}		
		
		tool.getBundleContext().registerService(IFunctionality.class.getName(), f,
				f.getProps());

		this.tools = ((ComboTool) comboTool).getTools();
		drop = new DropDownComponent(this, tool, tools, isHelpButton);
		
		final int size = ((ComboTool)comboTool).getTools().size();
		
		ITool t;
		for(int i = 0; i<size; i++) {
			t = ((ITool)((ComboTool)comboTool).getTools().get(i));
			if (t.getFuncSolved() != null)
				t.getFuncSolved().addObserver(this);
		}
	}

	public void paint(Graphics g) {
		if (!isHelpButton) {
//			try {			
//					img = ImageUtils.getResourceImage(tool.getBundleContext().getBundle(), tool.getIcon());
//					
//			} catch (GvSigPropertyNotAttachedException e) {
//				// TODO Auto-generated catch block
//				e.printStackTrace();
//			}
			
			super.paint(g);		
		
			
		} else {
			this.setPSSize(0, 0);
			this.setSize(0, 0);
			// setLocation(mainFrame.getWidth() - drop.img.getWidth(null), 0);
			setOpaque(false);
		}		
	}

	public DropDownComponent getDropDown() {
		return drop;
	}

	public ITool getTool() {		
		return (ITool) tools.elementAt(index);
	}
	
	public void setTools(Vector tools) {		
		this.tools = tools;
	}

	public void activateTool(int index) {		
		if (this.index == index) changeState();
		else {
			if(tool.getFuncSolved().isActive()) {
				changeState();
			}
			this.index = index;
			tool = this.getTool();
			updateIcon();
			changeState();			
		}		
	}

	public void producersConnected(Wire[] wires) {
		// TODO Auto-generated method stub
		// System.out.println("PRODUCERS CONNECTED");
		
	}

	public void updated(Wire wire, Object value) {
		// TODO Auto-generated method stub
		// System.out.println("UPDATED");
		
	}
	
	public boolean removeTool(ITool tool) {
		if(tools != null && tools.contains(tool)) {
			tools.remove(tool);
			return true;
		} else {
			return false;
		}		
	}
	
	public int numOfTools() {
		if (tools != null) {
			return tools.size();
		} 
		return -1;
	}
	
	public void taskInited(ITask task) {		
		
		final Vector v = tools;
		final int size = v.size();
		
		ITool t;
		for(int i = 0; i<size; i++) {
			t = (ITool)v.get(i);
			if (t.getFuncSolved() != null && t.getFuncSolved() == task) {
				tool = t;
				updateIcon();
			}
		}
		
		super.taskInited(task);
	}
	
	protected void updateIcon() {
		try {
			img_orange = ImageUtils.getResourceImage(
					tool.getBundleContext().getBundle(), tool.getIcon() + ".png.orange");
			img_black = ImageUtils.getResourceImage(
					tool.getBundleContext().getBundle(), tool.getIcon() + ".png.black");
		} catch (GvSigPropertyNotAttachedException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}		
	}
}