/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2006 Prodevelop and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *   Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *   +34 963862235
 *   gvsig@gva.es
 *   http://www.gvsig.gva.es
 *
 *    or
 *
 *   Prodevelop Integracin de Tecnologas SL
 *   Conde Salvatierra de lava , 34-10
 *   46004 Valencia
 *   Spain
 *
 *   +34 963 510 612
 *   +34 963 510 968
 *   gis@prodevelop.es
 *   http://www.prodevelop.es
 *
 *    or
 *
 *   Instituto de Robtica
 *   Apartado de correos 2085
 *   46071 Valencia
 *   (Spain)
 *   
 *   +34 963 543 577
 *   jjordan@robotica.uv.es
 *   http://robotica.uv.es
 *   
 */

package org.gvsig.mobile.ui.lw;

import java.io.File;

import org.gvsig.mobile.core.auxfunctions.ResourceReader;
import org.gvsig.mobile.ui.awt.IWindowInfo;
import org.gvsig.mobile.ui.awt.WindowManager;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.zaval.lw.LwBorderLayout;
import org.zaval.lw.LwComposite;
import org.zaval.lw.LwPanel;

/**
 * A panel with a FileChooserDialog
 * @author aromeu
 *
 */
public class FileChooserDialog extends OkCancelDialog implements IWindowInfo,
		LwComposite, IValidStateListener {

	final static private Logger logger =
		LoggerFactory.getLogger(FileChooserDialog.class);
	
	private FilePanel filePanel;		
	private FileChooserListener fileListener;	
	private String title = "";
	private String root;
	private String[][] filter;
	
	private int dialogType = DIALOG_TYPE_SELECT; 
	public final static int DIALOG_TYPE_SELECT = 0;
	public final static int DIALOG_TYPE_CREATE_NEW = 1;
	
	/**
	 * The constructor
	 * @param listener A FileChooseListener
	 * @param root - The root directory
	 * @param filter - the file filter. The filter is a two-dimensional 
	 * array where the first element corresponds to the extension filter 
	 * title and the second element specifies a file extension filter.
	 */
	public FileChooserDialog(
			FileChooserListener listener,
			String _root,
			String[][] filter,
			int dlgtype) {		
		super(true, null);	
		
		dialogType = dlgtype;
		this.fileListener = listener;
		this.filter = filter;
		this.root = _root;
		
		if(root != null) {
			if(filter == null) {
				filePanel = new FilePanel(root, true, this);
			} else {
				filePanel = new FilePanel(root, filter, this);
			}
		} else {
			if(filter == null)  {
				filePanel = new FilePanel(AppSettings.lastPath, true, this);
			} else {
				filePanel = new FilePanel(AppSettings.lastPath, filter, this);
			}
		}

		if (dlgtype == DIALOG_TYPE_CREATE_NEW) {
			filePanel.setFileFiledEditable(true);
		}
		
		filePanel.addActionListener(this);

		add(LwBorderLayout.CENTER, filePanel);
	}	

	public void actionPerformed(Object src, Object data) {
		if (src == getOkButton()) {			
			File s = filePanel.getSelectedFile();
			String resp = s.getAbsolutePath();
			
			switch (dialogType) {
			case DIALOG_TYPE_CREATE_NEW:
				if (ResourceReader.isDirectory(s)) {
					resp = resp + File.separator + filePanel.getFileFieldText();
				}
				break;
			case DIALOG_TYPE_SELECT:
				break;
			default:
				logger.error("Unexpected type: " + dialogType);
			break;
			}
			
			WindowManager.getInstance().closeActivePanel();
			// give time to allow window disappear
			try { Thread.sleep(UIConstants.WAIT_TO_REFRESH_UI_MILISECS); } catch (Exception e) { }
			this.fileListener.fileSelected(resp);
			
		} else if (src == getCancelButton()) {
			WindowManager.getInstance().closeActivePanel();
		}	
	}	

	public LwPanel buildMainPanel() {		
		
		return new LwPanel();
	}
	
	public String getTitle() {
		return title;
	}
	
	public void setTitle(String title) {
		this.title = title;
	}
	
	public void setFileFieldEditable(boolean v) {
		if (filePanel == null) {
			
		} else {
			filePanel.setFileFiledEditable(v);
		}
	}

	public void newState(boolean valid) {
		getOkButton().setEnabled(valid);
		getOkButton().repaint();
	}
}
