/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2006 Prodevelop and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *   Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *   +34 963862235
 *   gvsig@gva.es
 *   http://www.gvsig.gva.es
 *
 *    or
 *
 *   Prodevelop Integracin de Tecnologas SL
 *   Conde Salvatierra de lava , 34-10
 *   46004 Valencia
 *   Spain
 *
 *   +34 963 510 612
 *   +34 963 510 968
 *   gis@prodevelop.es
 *   http://www.prodevelop.es
 *
 *    or
 *
 *   Instituto de Robtica
 *   Apartado de correos 2085
 *   46071 Valencia
 *   (Spain)
 *   
 *   +34 963 543 577
 *   jjordan@robotica.uv.es
 *   http://robotica.uv.es
 *   
 */

package org.gvsig.mobile.ui.lw;

import java.awt.Image;
import java.awt.Toolkit;
import java.io.File;

import org.gvsig.mobile.core.auxfunctions.Defaults;
import org.gvsig.mobile.core.auxfunctions.ResourceReader;
import org.gvsig.mobile.ui.awt.ImageUtils;

/**
 * Utility class to read resource files.
 * 
 * @see java.awt.MediaTracker
 * 
 * @author jldominguez
 *
 */
public class IconResourceReader {
	
//	private static Logger logger = Logger.getLogger(IconResourceReader.class);
	
	/**
	 * Icon shown while is a finish point selected.
	 */
	public static Image GPS_FINISH_ICON;
	/**
	 * Icon shown while the app is processing.
	 */
	public static Image PROCESSING_ICON;
	/**
	 * Array of GPS icons (16 positions)
	 */
	public static Image[] GPS_CURSOR_ICON;
	
	
	public static int PROCESS_ICON_X = 90;
	public static int PROCESS_ICON_Y = 105; //110;
	public static int PROCESS_ICON_SIZE = 60;

	public static String ICON_PATH = "";
	public static String DEFAULT_ICON_DIR="icons";
	public static String ICONS_JAR="DefaultIcons.jar!";
	public static String JAR_PREFIX="jar:file:";
	
	
	public static void updateIconPaths(String img_subfolder, int factor) {
		ICON_PATH =
			ResourceReader.resourceDir + File.separator+ DEFAULT_ICON_DIR
			+ File.separator + ICONS_JAR + ResourceReader.FILE_SEPARATOR_INSIDE_JAR;
		//IMG_SKIN_PATH = "img" + File.separator + img_subfolder;
		ResourceReader.IMG_SKIN_PATH = JAR_PREFIX + ICON_PATH + "img" + ResourceReader.FILE_SEPARATOR_INSIDE_JAR + img_subfolder;
		ResourceReader.IMG_SKIN_PREFIX = img_subfolder + "_";

		switch (factor){
		case 2:
			ResourceReader.IMG_SKIN_PATH = JAR_PREFIX + ICON_PATH + "img" + ResourceReader.FILE_SEPARATOR_INSIDE_JAR + "img48";
			ResourceReader.IMG_SKIN_PREFIX = "img48_";
			break;
		case 3:
			ResourceReader.IMG_SKIN_PATH = JAR_PREFIX + ICON_PATH + "img" + ResourceReader.FILE_SEPARATOR_INSIDE_JAR + "img72";
			ResourceReader.IMG_SKIN_PREFIX = "img72_";
			break;
		}
		// ----------------------------
		
		PROCESSING_ICON = ResourceReader.getResourceImage(ResourceReader.IMG_SKIN_PATH, "processing.png");
		GPS_FINISH_ICON = ResourceReader.getResourceImage(ResourceReader.IMG_SKIN_PATH, "finish_flag_white.png");
		GPS_CURSOR_ICON = new Image[ResourceReader.GPS_CURSOR_ICON_COUNT];
		for (int i=0; i<ResourceReader.GPS_CURSOR_ICON_COUNT; i++) {
			try {
				GPS_CURSOR_ICON[i] = ResourceReader.getResourceImage(ResourceReader.IMG_SKIN_PATH, "gps_cursor_" + i + ".png");
			} catch (Exception ex) {
//				logger.error("Unable to read GPS icon # " + i + " : " + ex.getMessage());
			}
		}
	}

	static {

		// ------ Image folder and prefix according to DPI
	}
	
	
	/**
	 * Detects screen resolution and calculate a size factor to multiply by to adjust buttons size and layouts.
	 * In phoneME & OpenMoko, screen resolutions affect the  size of the buttons
	 * @return Size factor by which buttons and other widgets dimensions should
	 *         be multiplied by to get the same visual effect (the same visual size)
	 *         running on all platforms
	 */
	public static int getScreenSizeFactor(){
		int dpi;
		
		dpi = (int) (Toolkit.getDefaultToolkit().getScreenResolution());
//		logger.debug("Device screen resolution: " + dpi);
		
		dpi = dpi / 96;
		
		//Comment below line for builds!!!!
		// dpi = 2;
		
		switch (dpi) {
		case 2:
			return 2;
		case 3:
			return 3;
		default:
			return 1;
		}

	}

	/**
	 * gets a file from the working directory
	 * @param dir subfolder
	 * @param file_name file name
	 * @return the file object
	 */
	public static File getWorkingDirFile(String dir, String file_name) {
		File resp = new File(
				Defaults.WORKSPACE
				+ File.separator
				+ dir
				+ File.separator
				+ file_name);
		return resp;
	}
	
	/**
	 * Gets the working subfolder
	 * @param dir subfolder name
	 * @return the working subfolder
	 */
	public static File getWorkingDir(String dir) {
		File resp = new File(
				Defaults.WORKSPACE
				+ File.separator
				+ dir);
		return resp;
	}	

	
	/**
	 * Gets Language dir
	 * @param bundleName
	 * @param dir
	 * @return
	 */
	public static File getLanguageFolder(String bundleDir, String dir) {
		String filePath = ImageUtils.appDir + bundleDir + File.separator + dir;
		// System.out.println(filePath);
		File resp = new File(filePath);
		return resp;
	}
}
