/**
 *     Caption: J2SE/J2ME(Personal Profile) Light-Weight Visual Components Library
 *
 *     @author:     Andrei Vishnevsky
 *     @version:    4.50
 *
 * The latest product version is always available from the product's homepage:
 * http://www.lwvcl.com/
 *
 * Copyright (C) 2001-2005  Zaval Creative Engineering Group
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * (version 2) as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */
package org.zaval.lw.misc;

import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.LineNumberReader;
import java.lang.reflect.Constructor;
import java.util.Stack;
import java.util.Vector;

import org.zaval.data.Item;
import org.zaval.data.Tree;
import org.zaval.data.TreeModel;
import org.zaval.lw.LwAdvViewMan;
import org.zaval.lw.LwCheckbox;
import org.zaval.lw.LwCombo;
import org.zaval.lw.LwComponent;
import org.zaval.lw.LwFlowLayout;
import org.zaval.lw.LwImage;
import org.zaval.lw.LwImgRender;
import org.zaval.lw.LwLabel;
import org.zaval.lw.LwPanel;
import org.zaval.lw.LwStButton;
import org.zaval.lw.LwTextField;
import org.zaval.lw.LwTextRender;
import org.zaval.lw.LwToolkit;
import org.zaval.lw.LwView;
import org.zaval.lw.tree.LwTree;

public class LwUtils
{
   public static TreeModel loadTreeModel (String[] args, Class clazz)
   {
      Stack      stack = new Stack ();
      Item       root  = nii(args[0], clazz);
      TreeModel  model = new Tree(root);
      int        depth = 0;
      stack.push (root);

      Item prev = null;
      for (int i=1; i < args.length; i++)
      {
        if (prev != null && args[i].indexOf (' ', depth) == depth)
        {
           depth++;
           stack.push (prev);
        }
        else
        {
          while (depth > 0 && args[i].charAt(depth-1) != ' ')
          {
            depth--;
            stack.pop();
          }
        }

        Item n = nii (args[i].trim(), clazz);
        model.add ((Item)stack.peek(), n);
        prev = n;
      }
      return model;
   }

   public static TreeModel loadTreeModel (InputStream is, Class clazz)
   throws IOException
   {
     LineNumberReader lr = new LineNumberReader(new InputStreamReader(is));
     Vector           v  = new Vector();
     String           ln = null;
     while ((ln=lr.readLine()) != null)
       if (ln.length() > 0) v.addElement(ln);
     String[] res = new String[v.size()];
     for (int i=0;i<res.length; i++) res[i] = (String)v.elementAt(i);
     return loadTreeModel(res, clazz);
   }

   public static LwTree loadTree (InputStream is, Class clazz)
   throws IOException
   {
     return new LwTree (loadTreeModel(is, clazz));
   }

   public static void toggleAllItem (LwTree tree, boolean b) {
     tg(tree, tree.getModel().getRoot(), b);
   }

   public static boolean fillEditorByData (Object obj, LwComponent editor)
   {
     if (obj != null)
     {
        String value = obj.toString();
        if (editor instanceof LwTextField)
        {
          ((LwTextField)editor).setText (value);
        }
        else
        if (editor instanceof LwCheckbox)
        {
          int b = Integer.parseInt(value.substring(0, value.indexOf(';')));
          ((LwCheckbox)editor).setState(b == 1);
        }
        else
        if (editor instanceof LwCombo)
        {
          LwCombo combo = (LwCombo)editor;
          combo.getList().removeAllItems();
          int index    = value.indexOf(';');
          int selected = Integer.parseInt((index > 0)?value.substring(0, index):value);
          while (index > 0)
          {
            int j = value.indexOf (';', index + 1);
            combo.getList().addItem((j > 0)?value.substring(index + 1, j):value.substring(index + 1));
            index = j;
          }
          ((LwCombo)editor).getList().select(selected);
        }
        else
          throw new IllegalArgumentException();
        return true;
     }
     return false;
   }

   public static String fetchEditedData (Object data, LwComponent editor)
   {
     if (data != null)
     {
       String value = data.toString();
       if (editor instanceof LwCheckbox)
         return (((LwCheckbox)editor).getState()?1:0) + value.substring(value.indexOf (';'));

       if (editor instanceof LwCombo)
         return ((LwCombo)editor).getList().getSelectedIndex() + value.substring(value.indexOf (';'));

       if (editor instanceof LwTextField)
         return ((LwTextField)editor).getText();

       throw new IllegalArgumentException ();
     }
     return null;
   }

   public static String fetchRenderedValue (String value)
   {
     int i = value.indexOf(';');
     if (i > 0)
     {
       int selected = Integer.parseInt(value.substring(0, i));
       while (selected > 0)
       {
         i = value.indexOf(';', i + 1);
         selected--;
       }
       int j = value.indexOf (';', i + 1);
       value = (j >= 0)?value.substring (i + 1, j):value.substring (i + 1);
     }
     return value;
   }

   public static LwComponent makeImageLabel (LwTextRender label, LwImgRender r)
   {
     LwPanel p = new LwPanel();
     p.setLwLayout (new LwFlowLayout (LwToolkit.LEFT, LwToolkit.CENTER, LwToolkit.HORIZONTAL, 6));
     p.add (new LwImage(r));
     p.add (new LwLabel (label));
     return p;
   }

   public static LwStButton makeStButton (String rsName)
   {
     LwStButton   bt  = new LwStButton();
     LwAdvViewMan man = new LwAdvViewMan();
     man.put ("st.over",     LwToolkit.getView(rsName.concat(".over")));
     man.put ("st.pressed",  LwToolkit.getView(rsName.concat(".pressed")));
     man.put ("st.out",      LwToolkit.getView(rsName.concat(".out")));
     man.put ("st.disabled", LwToolkit.getView(rsName.concat(".disabled")));
     bt.setViewMan(man);
     return bt;
   }

   private static void tg (LwTree tree, Item root, boolean b)
   {
      TreeModel model = tree.getModel ();
      if (model.hasChildren (root))
      {
         if (tree.getItemMetrics(root).isOpen() != b)
           tree.toggle (root);

         for (int i=0; i<model.getChildrenCount(root); i++)
           tg (tree, model.getChildAt(root, i), b);
      }
   }

   private static Item nii(Object data, Class clazz)
   {
     try {
       if (clazz == null) return new Item (data);
       else
       if (clazz.equals (org.zaval.lw.misc.CItem.class))
       {
         String s = (String)data;
         int    i = s.lastIndexOf ('$');
         if (i < 0) i = s.lastIndexOf ('@');

         if (i > 0)
         {
           data = s.substring(0, i);
           CItem ci = new CItem (data);
           ci.setIcon ((s.charAt (i) == '$')?new LwImgRender(LwToolkit.getImage(s.substring(i + 1)), LwView.ORIGINAL)
                                            :(LwImgRender)LwToolkit.getStaticObj(s.substring(i + 1)));
           return ci;
         }
         return new CItem (data);
       }
       else
       {
         Constructor cn = clazz.getConstructor(new Class[] { java.lang.Object.class } );
         return (Item)cn.newInstance (new Object[] { data });
       }
     }
     catch (Exception e) {}
     return null;
   }
}
